% Tejendra Dhakal
% 02/01/2012
% LDEO, Columbia University

% This Script parses the radar file and plots lines one at a time with a
% pause.
%%
clc; clear all; close all;
fs = 150e6;     % Sampling Frequency
sw1 = 80e-6;    % Sampling Period 10 us Chirp
sw2 = 80e-6;    % Sampling Period 1 us Chrip
datalen10us = fs*sw1;   % Number of samples per sweep 10 us Chirp
datalen1us = fs*sw2;    % Number of samples per sweep 1 us Chirp
stacking_factor = 16;
conversion_factor = 1;%1.177/4095; % * stacking_factor; % Volt/unit
miny = -35e3;
maxy = 35e3;
t10us = 0:1/fs:(sw1-1/fs);
t1us = 0:1/fs:(sw2-1/fs);

line_num_max = 1000;% Line numbers to replay
% fid1 = fopen('C:\Users\Tej\Desktop\Icepod\DICE\Matlab\datafile.dat','r'); % File namw with Ext
[filename, filepath,~] = uigetfile('*.dice','Select a dice file to plot')
fp = fullfile(filepath,filename);
fid1 = fopen(fp); % File namw with Ext
% fid1 = fopen('M:\F000_20140312_161346_0035.rad','r'); % File namw with Ext
dispMetadata = 1;

fseek(fid1,0,'eof');   % Go to the end of the file
filesize = ftell(fid1);    % Get the filesize
fseek(fid1,0,'bof');   % Go to the beginning of the file
% Some files do not start with START frame
header = zeros(4,1);
footer = zeros(4,1);
% Radar Start Time Sec 32b

metaDataCh1_10us = struct('TimeOffset',0,'PPSCount',0,'GateNum',0,'TrigNum',0,...
    'SampleCount',0,'AdcChan',0,'FifoOverflow',0,'LinkNum',0,'PacketCount',0,...
    'StartTime','');
metaDataCh2_10us = metaDataCh1_10us;
metaDataCh1_1us = metaDataCh1_10us;
metaDataCh2_1us = metaDataCh1_10us;
while 1 %find first header
    header(1) = fread(fid1,1,'int8');% read out header
    if(header(1) == -91)
        header(2) = fread(fid1,1,'int8');
        if(header(2) == 3)
            header(3) = fread(fid1,1,'int8');
            if(header(3) ==2)
                header(4) = fread(fid1,1,'int8');
                if(header(4) == 1)
                    break;
                end
            end
        end
    end
end
framesread = 0;
i = 0;
while(i<line_num_max)
    %10us Ch1
    % Meta Data Format:
    % Time Stamp Sample Clock Offset 32b
    % Time Stamp PPS Count 32b
    % Reserved 16b-- Gate Number 16b
    % Reserved 16b -- Trigger Number 16b
    % Valid Sample Count 32b
    % Packet Count 12b-- Filler 3b-- Link Number 9b-- Filler 3b--Overflow Flag 1b -- ADC Channel# 4b
    % DMA Destinaiton Address Low 32b
    % Radar Start Time Sec 32b
    adcCh1_10us=fread(fid1,datalen10us,'int16')*conversion_factor;
    metaDataCh1_10us.TimeOffset = fread(fid1,1,'uint32');
    metaDataCh1_10us.PPSCount = fread(fid1,1,'uint32');
    metaDataCh1_10us.GateNum = fread(fid1,1,'uint16');
    temp = fread(fid1,1,'uint16');  % read out filler bits
    metaDataCh1_10us.TrigNum = fread(fid1,1,'uint16');
    temp = fread(fid1,1,'uint16');  % read out filler bits
    metaDataCh1_10us.SampleCount = fread(fid1,1,'uint32');
    metaDataCh1_10us.AdcChan = fread(fid1,1,'ubit4');
    metaDataCh1_10us.FifoOverflow = fread(fid1,1,'ubit1');
    temp = fread(fid1,1,'ubit3');  % read out filler bits
    metaDataCh1_10us.LinkNum = fread(fid1,1,'ubit9');
    temp = fread(fid1,1,'ubit3');  % read out filler bits
    metaDataCh1_10us.PacketCount = fread(fid1,1,'ubit12');
    temp = fread(fid1,1,'uint32');  % read out filler bits
    metaDataCh1_10us.StartTime = datestr(fread(fid1,1,'uint32','l')/86400 +  datenum(1970,1,1,0,0,0));
    footer = fread(fid1,4,'int8');
    framesread = framesread +1;
    if(mod(metaDataCh1_10us.LinkNum,4) ~= 0)   %Then this is 1us data not 10 us skip this and read next
        header = fread(fid1,4,'int8');% read out header
        continue;
    end
    %     adcCh1_10us = adcCh1_10us/16*Res;
    if header ~= [-91 3 2 1]'
        disp('Frame Error');
    end
    if footer ~= [-91 1 2 3]'
        disp('Frame Error');
    end
    %10us Ch2
    header = fread(fid1,4,'int8');% read out header
    adcCh2_10us=fread(fid1,datalen10us,'int16')*conversion_factor;
    metaDataCh2_10us.TimeOffset = fread(fid1,1,'uint32');
    metaDataCh2_10us.PPSCount = fread(fid1,1,'uint32');
    metaDataCh2_10us.GateNum = fread(fid1,1,'uint16');
    temp = fread(fid1,1,'uint16');  % read out filler bits
    metaDataCh2_10us.TrigNum = fread(fid1,1,'uint16');
    temp = fread(fid1,1,'uint16');  % read out filler bits
    metaDataCh2_10us.SampleCount = fread(fid1,1,'uint32');
    metaDataCh2_10us.AdcChan = fread(fid1,1,'ubit4');
    metaDataCh2_10us.FifoOverflow = fread(fid1,1,'ubit1');
    temp = fread(fid1,1,'ubit3');  % read out filler bits
    metaDataCh2_10us.LinkNum = fread(fid1,1,'ubit9');
    temp = fread(fid1,1,'ubit3');  % read out filler bits
    metaDataCh2_10us.PacketCount = fread(fid1,1,'ubit12');
    temp = fread(fid1,1,'uint32');  % read out filler bits
    metaDataCh2_10us.StartTime = datestr(fread(fid1,1,'uint32','l')/86400 +  datenum(1970,1,1,0,0,0));
    footer = fread(fid1,4,'int8');
    framesread = framesread +1;
    if(metaDataCh1_10us.LinkNum ~= metaDataCh2_10us.LinkNum)   %Then this is 1us data not 10 us start over
        header = fread(fid1,4,'int8');% read out header
        continue;
    end
    
    %     adcCh2_10us = adcCh2_10us/16*Res;
    if header ~= [-91 3 2 1]'
        disp('Frame Error');
    end
    if footer ~= [-91 1 2 3]'
        disp('Frame Error');
    end
    %1us Ch1
    header = fread(fid1,4,'int8');% read out header
    adcCh1_1us=fread(fid1,datalen1us,'int16')*conversion_factor;
    
    metaDataCh1_1us.TimeOffset = fread(fid1,1,'uint32');
    metaDataCh1_1us.PPSCount = fread(fid1,1,'uint32');
    metaDataCh1_1us.GateNum = fread(fid1,1,'uint16');
    temp = fread(fid1,1,'uint16');  % read out filler bits
    metaDataCh1_1us.TrigNum = fread(fid1,1,'uint16');
    temp = fread(fid1,1,'uint16');  % read out filler bits
    metaDataCh1_1us.SampleCount = fread(fid1,1,'uint32');
    metaDataCh1_1us.AdcChan = fread(fid1,1,'ubit4');
    metaDataCh1_1us.FifoOverflow = fread(fid1,1,'ubit1');
    temp = fread(fid1,1,'ubit3');  % read out filler bits
    metaDataCh1_1us.LinkNum = fread(fid1,1,'ubit9');
    temp = fread(fid1,1,'ubit3');  % read out filler bits
    metaDataCh1_1us.PacketCount = fread(fid1,1,'ubit12');
    temp = fread(fid1,1,'uint32');  % read out filler bits
    metaDataCh1_1us.StartTime = datestr(fread(fid1,1,'uint32','l')/86400 +  datenum(1970,1,1,0,0,0));
    footer = fread(fid1,4,'int8');
    %     adcCh1_1us = adcCh1_1us/16*Res;
    framesread = framesread +1;
    if header ~= [-91 3 2 1]'
        disp('Frame Error');
    end
    if footer ~= [-91 1 2 3]'
        disp('Frame Error');
    end
    %1us Ch2
    header = fread(fid1,4,'int8');% read out header
    adcCh2_1us=fread(fid1,datalen1us,'int16')*conversion_factor;
    metaDataCh2_1us.TimeOffset = fread(fid1,1,'uint32');
    metaDataCh2_1us.PPSCount = fread(fid1,1,'uint32');
    metaDataCh2_1us.GateNum = fread(fid1,1,'uint16');
    temp = fread(fid1,1,'uint16');  % read out filler bits
    metaDataCh2_1us.TrigNum = fread(fid1,1,'uint16');
    temp = fread(fid1,1,'uint16');  % read out filler bits
    metaDataCh2_1us.SampleCount = fread(fid1,1,'uint32');
    metaDataCh2_1us.AdcChan = fread(fid1,1,'ubit4');
    metaDataCh2_1us.FifoOverflow = fread(fid1,1,'ubit1');
    temp = fread(fid1,1,'ubit3');  % read out filler bits
    metaDataCh2_1us.LinkNum = fread(fid1,1,'ubit9');
    temp = fread(fid1,1,'ubit3');  % read out filler bits
    metaDataCh2_1us.PacketCount = fread(fid1,1,'ubit12');
    temp = fread(fid1,1,'uint32');  % read out filler bits
    metaDataCh2_1us.StartTime = datestr(fread(fid1,1,'uint32','l')/86400 +  datenum(1970,1,1,0,0,0));
    footer = fread(fid1,4,'int8');
    %     adcCh2_1us = adcCh2_1us/16*Res;
    
    framesread = framesread +1;
    if header ~= [-91 3 2 1]'
        disp('Frame Error');
    end
    if footer ~= [-91 1 2 3]'
        disp('Frame Error');
    end
    figure(1)
    subplot 211
    plot(t10us*1e6,adcCh1_10us)
    title({'Adc Acquired Data 10us';'Ch1';['Frame:', num2str(framesread)]})
    xlabel('Time, \muS')
    ylabel('Amplitude')
    axis([0 80 miny maxy])
    grid
    subplot 212
    plot(t10us*1e6,adcCh2_10us)
    title('Ch2');
    xlabel('Time, \muS')
    ylabel('Amplitude')
    axis([0 80 miny maxy])
    grid
     figure(2)
    subplot 211
    plot(t1us*1e6,adcCh1_1us)
    title({'Adc Acquired Data 1us';'Ch1';['Frame:', num2str(framesread)]})
    xlabel('Time, \muS')
    ylabel('Amplitude')
    axis([0 80 miny maxy])
    grid
    subplot 212
    plot(t1us*1e6,adcCh2_1us)
    title('Ch2');
    xlabel('Time, \muS')
    ylabel('Amplitude')
    axis([0 80 miny maxy])
    grid
    header = fread(fid1,4,'int8');% read out next header
    if ftell(fid1) > filesize - 48160
        break;
    end
    i = i+1;

    pause(0.5);
end
% figure; 
framesread
numFramesSkipped = framesread - line_num_max*4

% for i = 1:datalen10us
% adcCh1_10us(i) = fread(fid1,1,'bit12');
% temp = fread(fid1,1,'bit4');
% end

fclose(fid1);
% t10us = 0:1/fs:(sw1-1/fs);
% t1us = 0:1/fs:(sw2-1/fs);
%
% rxout = xcorr(adcCh1_10us./32768,Sig10us./32768);
% rxoutCh1_10us = abs(rxout(datalen10us:2*datalen10us-1));
% rxout = xcorr(adcCh2_10us./32768,Sig10us./32768);
% rxoutCh2_10us = abs(rxout(datalen10us:2*datalen10us-1));
% rxout = xcorr(adcCh1_1us./32768,Sig1us./32768);
% rxoutCh1_1us = abs(rxout(datalen1us:2*datalen1us-1));
% rxout = xcorr(adcCh2_1us./32768,Sig1us./32768);
% rxoutCh2_1us = abs(rxout(datalen1us:2*datalen1us-1));
%
% figure
% subplot 211
% plot(t10us*1e6,adcCh1_10us)
% % plot(adcCh1_10us)
% title({'Adc Acquired Data 10us';'Ch1'})
% xlabel('Time, \muS')
% ylabel('Amplitude')
% grid
% % axis([0 80 -33000 33000])
% subplot 212
% plot(t10us*1e6,adcCh2_10us)
% % plot(adcCh2_10us)
% title('Ch2');
% xlabel('Time, \muS')
% ylabel('Amplitude')
% grid
% % axis([0 80 -33000 33000])
%
% % figure
% % subplot 211
% % plot([1:datalen10us],10*log10(abs(adcCh1_10us)))
% % title({'Adc Acquired Data 10us';'Ch1'})
% % xlabel('Time, \muS')
% % ylabel('Amplitude')
% % subplot 212
% % plot(1:datalen10us,10*log10(abs(adcCh2_10us)))
% % title('Ch2');
% % xlabel('Time, \muS')
% % ylabel('Amplitude')
% %
% % figure
% % subplot 211
% % plot([1:datalen1us],10*log10(rxoutCh1_10us))
% % title({'Compressed Return 10us';'Ch1'})
% % xlabel('Sample #, \muS')
% % ylabel('Amplitude')
% % subplot 212
% % plot(1:datalen1us,10*log10(rxoutCh2_10us))
% % title('Ch2');
% % xlabel('Time, \muS')
% % ylabel('Amplitude')
% %
% %
% %
% figure
% subplot 211
% plot(t1us*1e6,adcCh1_1us)
% % plot(adcCh1_1us)
% title({'Adc Acquired Data 1us';'Ch1'})
% xlabel('Time, \muS')
% ylabel('Amplitude')
% grid
% % axis([0 80 -33000 33000])
% subplot 212
% plot(t1us*1e6,adcCh2_1us)
% % plot(adcCh2_1us)
% title('Ch2');
% xlabel('Time, \muS')
% ylabel('Amplitude')
% grid
% % axis([0 80 -33000 33000])
% %
% % figure
% % subplot 211
% % plot([1:datalen1us],10*log10(abs(adcCh1_1us)))
% % title({'Adc Acquired Data 1us';'Ch1'})
% % xlabel('Time, \muS')
% % ylabel('Amplitude')
% % subplot 212
% % plot(1:datalen1us,10*log10(abs(adcCh2_1us)))
% % title('Ch2');
% % xlabel('Time, \muS')
% % ylabel('Amplitude')
% %
% % figure
% % subplot 211
% % plot([1:datalen1us],10*log10(rxoutCh1_1us))
% % title({'Compressed Return 1us';'Ch1'})
% % xlabel('Sample #, \muS')
% % ylabel('Amplitude')
% % subplot 212
% % plot(1:datalen1us,10*log10(rxoutCh2_1us))
% % title('Ch2');
% % xlabel('Time, \muS')
% % ylabel('Amplitude')
%
%
% L = length(t10us);
% NFFT = 2^nextpow2(L);   % Next power of 2 from length of y
% F10Ch1 = 2*abs(fft(double(adcCh1_10us),NFFT))/L;
% fif = fs/2*linspace(0,1,NFFT/2+1);
% F10Ch2 = 2*abs(fft(double(adcCh2_10us),NFFT))/L;
% figure
% subplot 211
% plot(fif/1e6,10*log10(F10Ch1(1:NFFT/2+1)))
% title({'Amplitude Spectrum of IF 10us Sig';'Ch1'})
% xlabel('Frequency (MHz)')
% ylabel('|F_Tx_IF| (dB)')
% grid on
% subplot 212
% plot(fif/1e6,10*log10(F10Ch2(1:NFFT/2+1)))
% title({'Amplitude Spectrum of IF 10us Sig';'Ch2'})
% xlabel('Frequency (MHz)')
% ylabel('|F_Tx_IF| (dB)')
% grid on
%
% % L = length(t1us);
% % NFFT = 2^nextpow2(L);   % Next power of 2 from length of y
% % F1Ch1 = 2*abs(fft(double(adcCh1_1us),NFFT))/L;
% % fif = fs/2*linspace(0,1,NFFT/2+1);
% % F1Ch2 = 2*abs(fft(double(adcCh2_1us),NFFT))/L;
% % figure
% % subplot 211
% % plot(fif/1e6,10*log10(F1Ch1(1:NFFT/2+1)))
% % title({'Amplitude Spectrum of IF 1us Sig';'Ch1'})
% % xlabel('Frequency (MHz)')
% % ylabel('|F_Tx_IF| (dB)')
% % grid on
% % subplot 212
% % plot(fif/1e6,10*log10(F1Ch2(1:NFFT/2+1)))
% % title({'Amplitude Spectrum of IF 1us Sig';'Ch2'})
% % xlabel('Frequency (MHz)')
% % ylabel('|F_Tx_IF| (dB)')
% % grid on