% clear all; close all; clc;
% Constants
c = 299792456;
c_ice = 1.68e8;
%Radar Parameters
f_c = 188e6;
bw = 60e6;
fs = 150e6;
pw1 = 10e-6;
pw2 = 1e-6;
f_dot1 = bw./pw1;
f_min = f_c - bw/2;
acq_time = 80e-6;           
phiTx = 0;
phiRx = 0;
Conversion_Factor = 1.125/4096; % for the digitizer volts/unit
maxAmplitude = 32767;           % max amplitude for 16 bits

% Signal Specs- change these to test the processing code
% minPowerdBm is the smallest signal return(after 16 stacking) you expect
% use rxNoiseFigure.m to compute the minimum detectable power after
% stacking (OutputSigAfterHwStackingdB)
minPowerdBm = -30;              % Expected signal power after 16 hw stacking
minAmplitude = dbm2vpp(minPowerdBm)/2/Conversion_Factor; % Amplitude equivalent to minPowerdBm
minSNRdB = -50;                 % Minimun expected SNR after hw stacking (relative to minPowerdbm)
noiseAmplitude = minAmplitude/(sqrt(10^(minSNRdB/10)));      %% noise Amplitude computed using minSNRdB

if(noiseAmplitude >= 32000)
    warning('Noise level too high, should be well below 32000')
end

% Setup big targets and respective amplitudes here. This doesn't effect the
% minPowerdBm or the noise
big_target = [189,1000,1600,1800,3600];     % one way range in meter.
big_target_rFactor = [.1,2.2,1,1,1.7];   % Relative amplitude factors for big targets. 1 = same as normal target
target = [189,1000:200:3600];           % this must includ all of the big targets
target_time = 2*target/c_ice;
sig_amp = target./target.^2;            % scale amplitude by square of range
tSample = 0:1/fs:acq_time-1/fs;         % 0 - 80 us sampling time
tTx = 0:1/fs:pw1-1/fs;                  % time scale for Tx 

% This is where the amplitude for big targets are adjusted
for i=1:length(big_target)
sig_amp(target == big_target(i)) = sig_amp(target == big_target(i))*big_target_rFactor(i);
end
A = sig_amp./(max(sig_amp))*maxAmplitude;   % normalize signal amplitude
A = A*minAmplitude/min(A);                  % scale by minimum Amplitude

%This plots target power with range
figure 
plot(target, vpp2dbm(A*2*1.125/4095),target,vpp2dbm(noiseAmplitude*2*1.125/4095)*ones(size(target)));
title('Return Power vs Range')
xlabel('Target Range')
ylabel('Return Power, dBm')
legend('Signal Power', 'Noise Power')
grid

%% Generate Received echo by adding returns for each target.
% T1 = 0:1/fs:acq_time-1/fs ;
RxIF = zeros(length(tSample),length(target));
for j = 1:length(target_time);
    for i = find(tSample>= target_time(j) & tSample<(target_time(j)+pw1)); % find each target time and generate 10 us chirp for that time + 10us
        RxIF(i,j) = A(j)/sqrt(2)*cos(2*pi*(f_min.*(tSample(i)-target_time(j)) + f_dot1/2*(tSample(i)-target_time(j)).^2) + phiRx)...
                    -A(j)/sqrt(2)*sin(2*pi*(f_min.*(tSample(i)-target_time(j)) + f_dot1/2*(tSample(i)-target_time(j)).^2) + phiRx);
    end
end
TxIF = 1/sqrt(2)*cos(2*pi*(f_min.*tTx + f_dot1/2*tTx.^2) + phiTx) - 1/sqrt(2)*sin(2*pi*(f_min.*tTx + f_dot1/2*tTx.^2) + phiTx);
RxIF = sum(RxIF,2); % Add all the pulses together
%%
figure
subplot 211
plot(tSample,RxIF);
title('RxIF');
subplot 212
plot(tTx,TxIF);
title('TxIF');

figure
plot(tTx*1e6,TxIF)
title('10 us Chirp')
title('Tx Pulse');
xlabel('Sample Time \muS');
ylabel('Amplitude');

L_tx = length(TxIF);
NFFT = 2^nextpow2(L_tx);   % Next power of 2 from length of y
F_TxIF = 2*abs(fft(TxIF,NFFT))/L_tx;
fif = fs/2*linspace(0,1,NFFT/2+1);
figure
plot(fif/1e6,20*log10(F_TxIF(1:NFFT/2+1)))
title('Amplitude Spectrum of TXIF')
xlabel('Frequency (MHz)')
ylabel('|F_Tx_IF| (dB)')
grid on

L_rx = length(RxIF);
% Add noise with noiseAmplitude for correlation test
RxIFInt16 = round(RxIF + randn(L_rx,1)*noiseAmplitude); % Round to account for bit quantization
TxIF = round(TxIF./max(TxIF)*maxAmplitude);

figure
plot(tSample*1e6,int16(RxIFInt16));
title('Simulated Radar Data');
xlabel('Sample Time \muS');
ylabel('Amplitude');
NFFT = 2^nextpow2(L_rx);   % Next power of 2 from length of y
F_RxIF = 2*abs(fft(RxIFInt16,NFFT))/L_rx;
fif = fs/2*linspace(0,1,NFFT/2+1);
figure
plot(fif/1e6,20*log10(F_RxIF(1:NFFT/2+1)))
title('Amplitude Spectrum of RxIF')
xlabel('Frequency (MHz)')
ylabel('|F_Tx_IF| (dB)')
grid on

rxout = xcorr(RxIFInt16./max(RxIFInt16),TxIF./max(TxIF));
figure
plot(tSample*c_ice/2, 20*log10(abs(rxout(L_rx:2*L_rx-1))));
% plot(tSample*c_ice/2, rxout(L_rx:2*L_rx-1));
title('Target Locations')
xlabel('Range (m)')

%% Rx signal only without noise is saved. Noise equal to noiseAmplitude10us will be added by writeRxFakeData.m 
RxIF10us = RxIF;
sig10usref = TxIF';
noiseAmplitude10us = noiseAmplitude;
save('rxDat10us', 'RxIF10us','sig10usref','noiseAmplitude10us');



