clear all; close all; clc;
% Constants
c = 299792456;
c_ice = 1.68e8;
%Radar Parameters
f_c = 188e6;
bw = 60e6;
fs = 150e6;
pw1 = 3e-6;
pw2 = 1e-6;
f_dot1 = bw./pw1;
f_dot2 = bw./pw2;
f_min = f_c - bw/2;
acq_time = 80e-6;           
phiTx = 0;
phiRx = 0;   
Conversion_Factor = 1.125/4096; % volts/unit
maxAmplitude = 32767;
% Signal Specs- change these to test the processing code
minPowerdBm = -10;       % (dBm) Minimum signal digitizer can see;
minAmplitude = dbm2vpp(minPowerdBm)/2/Conversion_Factor; % min Amp in bits
maxSNRdB = -30;              %(dB) Speficy the SNR at the digitizer input. 
noiseAmplitude = minAmplitude/(sqrt(10^(maxSNRdB/10)));      %% 0dB SNR
if(noiseAmplitude >= 32000)
    warning('Noise level too high, should be well below 4000')
end
% Setup big targets and respective amplitudes
big_target = [189,1000,1100,1200,1500]; 
big_target_rFactor = [.1,2.2,1,1.4,1.7];   % Relative amplitude factor
target = [189,1000:100:2000];       % this must includ all of the big targets
target_time = 2*target/c_ice;
sig_amp = target./target.^2;        % scale amplitude by square of range
tSample = 0:1/fs:acq_time-1/fs;
tTx = 0:1/fs:pw2-1/fs; 
for i=1:length(big_target)
sig_amp(target == big_target(i)) = sig_amp(target == big_target(i))*big_target_rFactor(i);
end
A = sig_amp./(max(sig_amp))*maxAmplitude; % normalize signal amplitude
A = A*minAmplitude/min(A);  % scale by minimum Amplitude

figure 
plot(target, vpp2dbm(A*2*1.125/4095),target,vpp2dbm(noiseAmplitude*2*1.125/4095)*ones(size(target)));
title('Return Power vs Range')
xlabel('Target Range')
ylabel('Return Power, dBm')
legend('Signal Power', 'Noise Power')
grid

T1 = 0:1/fs:acq_time-1/fs ;
RxIF = zeros(length(tSample),length(target));
for j = 1:length(target_time);
    for i = find(tSample>= target_time(j) & tSample<(target_time(j)+pw2));
        RxIF(i,j) = A(j)/sqrt(2)*cos(2*pi*(f_min.*(tSample(i)-target_time(j)) + f_dot2/2*(tSample(i)-target_time(j)).^2) + phiRx)...
                    -A(j)/sqrt(2)*sin(2*pi*(f_min.*(tSample(i)-target_time(j)) + f_dot2/2*(tSample(i)-target_time(j)).^2) + phiRx);
    end
end
TxIF = 1/sqrt(2)*cos(2*pi*(f_min.*tTx + f_dot2/2*tTx.^2) + phiTx) - 1/sqrt(2)*sin(2*pi*(f_min.*tTx + f_dot2/2*tTx.^2) + phiTx);
RxIF = sum(RxIF,2);

figure
subplot 211
plot(tSample,RxIF);
title('RxIF');
subplot 212
plot(tTx,TxIF);
title('TxIF');

figure
plot(tTx*1e6,TxIF)
title('1 us Chirp')
title('Tx Pulse');
xlabel('Sample Time \muS');
ylabel('Amplitude');

L_tx = length(TxIF);
NFFT = 2^nextpow2(L_tx);   % Next power of 2 from length of y
F_TxIF = 2*abs(fft(TxIF,NFFT))/L_tx;
fif = fs/2*linspace(0,1,NFFT/2+1);
figure
plot(fif/1e6,20*log10(F_TxIF(1:NFFT/2+1)))
title('Amplitude Spectrum of TXIF')
xlabel('Frequency (MHz)')
ylabel('|F_Tx_IF| (dB)')
grid on

L_rx = length(RxIF);
RxIFInt16 = round(RxIF + randn(L_rx,1)*noiseAmplitude);
TxIF = round(TxIF./max(TxIF)*maxAmplitude);

figure
plot(tSample*1e6,int16(RxIFInt16));
title('Simulated Radar Data');
xlabel('Sample Time \muS');
ylabel('Amplitude');
NFFT = 2^nextpow2(L_rx);   % Next power of 2 from length of y
F_RxIF = 2*abs(fft(RxIFInt16,NFFT))/L_rx;
fif = fs/2*linspace(0,1,NFFT/2+1);
figure
plot(fif/1e6,20*log10(F_RxIF(1:NFFT/2+1)))
title('Amplitude Spectrum of RxIF')
xlabel('Frequency (MHz)')
ylabel('|F_Tx_IF| (dB)')
grid on

fid = fopen('1usRxSignal.dat','w');

rxout = xcorr(RxIFInt16./max(RxIFInt16),TxIF./max(TxIF));
figure
plot(tSample*c_ice/2, 20*log10(abs(rxout(L_rx:2*L_rx-1))));
% plot(tSample*c_ice/2, rxout(L_rx:2*L_rx-1));
title('Target Locations')
xlabel('Range (m)')

RxIF1us = RxIF;
sig1usref = TxIF';
noiseAmplitude1us = noiseAmplitude;
save('rxDat1us', 'RxIF1us','sig1usref','noiseAmplitude1us');