% Tejendra Dhakal
% 02/01/2012
% LDEO, Columbia University

% This Script parses the radar file and plots specified line from the file
% each line = 10 us Chirp from Ch1, 10 us Chirp from Ch2, 1 us Chirp from
% Ch1 and 1 us Chirp from Ch2
%%
 clc; clear all; 
 %close all;
fs = 150e6;     % Sampling Frequency
sw1 = 80e-6;    % Sampling Period 10 us Chirp
sw2 = 80e-6;    % Sampling Period 1 us Chrip
datalen10us = 12000;    %fs*sw1;   % Number of samples per sweep 10 us Chirp
datalen1us = 12000;     %fs*sw2;    % Number of samples per sweep 1 us Chirp
pw1 = 3e-6;         % pulse width for 1st plse (10 us)
pw2 = 3e-6;         % pulse width for 2nd pulse( p2)
bw = 60e6;          % Bandwidth
fc = 38e6;          % Center Frequency

% Reference signal processing
usetukey = false;           % enable to use tukey taper on reference signal- also specify taper below
taper = 0.3;            % Adjustable parameter to go with the taper (0-1).
usewin = false;
wintype = 'kaiser';      % specifies which window to use on reference signal before fft, 'cheb' ro 'kaiser'
winparam = 2.5;          % Changes depending on wintype, kaiser- (1-10) typical, for 'cheb' this is rolloff in dBs (70-100)

% low pass filter
[z,p,k] = butter(8,bw/fs*1.1,'low');
dicelpf = zp2sos(z,p,k);

% Chan = 1;       % Select channel number to process
fftrange1 = 4000:11000;
fftrange10 = 4000:11000;
% fftrange1 = 1196:12000;
% fftrange10 = 1196:12000;
numHWStacks = 16;

% plot Ascope with time in x axis
plotvsTime = 0; 

% save figures?
saveAscope = 0;
saveFFTfig = 0;
saveCompressed = 0;

% use Complex signal for compression
useIQ = 1;  

% use ideal reference signal -> generated in this script
IdealRef = false;

Conversion_factor = 1125/4096; %mv/bit
elephantstoVolts = 1.125/4096/numHWStacks;

% line_num = 800;
line_num = 10;

filename = 'F:\AN04-F1004-TestFile.dice';
% filename = 'F:\ANT4_F1001_20170912_214832_0040.dice';
if exist('filename','var')
    if ~exist(filename,'file')
        clear filename;
    end
end
    
if exist('filename','var')  
fid1 = fopen(filename,'r'); % File namw with Ext
else
[filename, filepath,~] = uigetfile('*.dice','Select a dice file to plot')
fp = fullfile(filepath,filename);
fid1 = fopen(fp); % File namw with Ext
end

% Res = 1125/4096; % full scale mV/unit
reffile = 'E:\Users\Tej\Desktop\GIT\DICE\DiceTookit\chirps_nl\RefnLSignalsIdeal.mat';

dispMetadata = 1;

fseek(fid1,0,'eof');   % Go to the end of the file
filesize = ftell(fid1);    % Get the filesize
fseek(fid1,0,'bof');   % Go to the beC:\Users\Tej\Desktop\Icepod\DICE\Matlab\LabTest\9-19-15-Lab_preAn2Test\Ch1TxLoopBackginning of the file
% Some files do not start with START frame
header = zeros(4,1);
footer = zeros(4,1);
% Meta Data Format:
% Time Stamp Sample Clock Offset 32b
% Time Stamp PPS Count 32b
% Reserved 16b-- Gate Number 16b
% Reserved 16b -- Trigger Number 16b
% Valid Sample Count 32b
% Packet Count 12b-- Filler 3b-- Link Number 9b-- Filler 3b--Overflow Flag 1b -- ADC Channel# 4b
% DMA Destinaiton Address Low 32b
% Radar Start Time Sec 32b


% Notch filter setting:
usenotchfilter = 0;     % Use notch filters to take  out specific frequencies- change the frequencies in the code as required
nfBW = 0.3*1e6;             % Hz, Bandwidth
numOrder = 8;              % Bandwidth Attenuation -dB at nfBWadcCh2_1us
notchList = [9.11]*1e6;    % Hz, Add a frequency here to notch it out
if(usenotchfilter)
    numNotch = numel(notchList);
    for ff = 1:numNotch
        %             [b, a] = iirnotch(notchList(ff)/(fs/2), nfBW/(fs/2), nfAtten);
        %             [b, a] = butter(12,[(notchList(ff)-nfBW/2) (notchList(ff)+nfBW/2)]/fs/2, 'stop');
        %             [bb_b, bb_a] = butter(8,[fc-bw/2*1.03 fc+bw/2*1.03]/fs/2,'bandpass');
        %            notchFilterObj(ff) = design(fdesign.bandstop('N,F3dB1,F3dB2', 12, notchList(ff)-nfBW/2, notchList(ff)+nfBW/2, fs), 'butter');
        %              notchFilterObj(ff) = design(fdesign.notch('N,F0,BW',numOrder,notchList(ff),nfBW,fs));
        %             notchFilterObj(ff) = designfilt('bandstopfir','FilterOrder',numOrder, ...
        %          'HalfPowerFrequency1', notchList(ff)-nfBW/2,'HalfPowerFrequency2',notchList(ff)+nfBW/2, ...
        %          'SampleRate',fs);
%         notchFilterObj(ff) = designfilt('bandstopfir','FilterOrder',1500, ...
%             'CutoffFrequency1', notchList(ff)-nfBW/2,'CutoffFrequency2',notchList(ff)+nfBW/2, ...
%             'SampleRate',fs);
        notchFilterObj{ff} = designfilt('bandstopiir','FilterOrder',numOrder, ...
            'HalfPowerFrequency1', notchList(ff)-nfBW/2,'HalfPowerFrequency2',notchList(ff)+nfBW/2, ...
            'SampleRate',fs);
    end
    clear ff
end

metaDataCh1_10us = struct('TimeOffset',0,'PPSCount',0,'GateNum',0,'TrigNum',0,...
    'SampleCount',0,'AdcChan',0,'FifoOverflow',0,'LinkNum',0,'PacketCount',0,...
    'DestAddr',0,'StartTime','');
metaDataCh2_10us = metaDataCh1_10us;
metaDataCh1_1us = metaDataCh1_10us;
metaDataCh2_1us = metaDataCh1_10us;
while 1 %find first header
    header(1) = fread(fid1,1,'int8');% read out header
    if(header(1) == -91)
        header(2) = fread(fid1,1,'int8');
        if(header(2) == 3)
            header(3) = fread(fid1,1,'int8');
            if(header(3) ==2)
                header(4) = fread(fid1,1,'int8');
                if(header(4) == 1)
                    break;
                end
            end
        end
    end
end
framesread = 0;
i = 0;
while(i<line_num)
    %     if (i >= 63)
    %         keyboard;
    %     end
    %10us Ch1
    % Meta Data Format:
    % Time Stamp Sample Clock Offset 32b
    % Time Stamp PPS Count 32b
    % Reserved 16b-- Gate Number 16b
    % Reserved 16b -- Trigger Number 16b
    % Valid Sample Count 32b
    % Packet Count 12b-- Filler 3b-- Link Number 9b-- Filler 3b--Overflow Flag 1b -- ADC Channel# 4b
    % DMA Destinaiton Address Low 32b
    % Radar Start Time Sec 32b
    
    adcCh1_10us=fread(fid1,datalen10us,'int16');
    metaDataCh1_10us.TimeOffset = fread(fid1,1,'uint32');
    metaDataCh1_10us.PPSCount = fread(fid1,1,'uint32');
    metaDataCh1_10us.GateNum = fread(fid1,1,'uint16');
    temp = fread(fid1,1,'uint16');  % read out filler bits
    metaDataCh1_10us.TrigNum = fread(fid1,1,'uint16');
    temp = fread(fid1,1,'uint16');  % read out filler bits
    metaDataCh1_10us.SampleCount = fread(fid1,1,'uint32');
    metaDataCh1_10us.AdcChan = fread(fid1,1,'ubit4');
    metaDataCh1_10us.FifoOverflow = fread(fid1,1,'ubit1');
    temp = fread(fid1,1,'ubit3');  % read out filler bits
    metaDataCh1_10us.LinkNum = fread(fid1,1,'ubit9');
    temp = fread(fid1,1,'ubit3');  % read out filler bits
    metaDataCh1_10us.PacketCount = fread(fid1,1,'ubit12');
    metaDataCh1_10us.DestAddr = fread(fid1,1,'uint32');  % DMA destination address
    metaDataCh1_10us.StartTime = datestr(fread(fid1,1,'uint32','l')/86400 +  datenum(1970,1,1,0,0,0));
    footer = fread(fid1,4,'int8');
    framesread = framesread +1;
    if(mod(metaDataCh1_10us.LinkNum,4) ~= 0)   %Then this is 1us data not 10 us skip this and read next
        header = fread(fid1,4,'int8');% read out header
        continue;
    end
    %     adcCh1_10us = adcCh1_10us/16*Res;
    if header ~= [-91 3 2 1]'
        disp('Frame Error');
    end
    if footer ~= [-91 1 2 3]'
        disp('Frame Error');
    end
    %10us Ch2
    header = fread(fid1,4,'int8');% read out header
    adcCh2_10us=fread(fid1,datalen10us,'int16');
    metaDataCh2_10us.TimeOffset = fread(fid1,1,'uint32');
    metaDataCh2_10us.PPSCount = fread(fid1,1,'uint32');
    metaDataCh2_10us.GateNum = fread(fid1,1,'uint16');
    temp = fread(fid1,1,'uint16');  % read out filler bits
    metaDataCh2_10us.TrigNum = fread(fid1,1,'uint16');
    temp = fread(fid1,1,'uint16');  % read out filler bits
    metaDataCh2_10us.SampleCount = fread(fid1,1,'uint32');
    metaDataCh2_10us.AdcChan = fread(fid1,1,'ubit4');
    metaDataCh2_10us.FifoOverflow = fread(fid1,1,'ubit1');
    temp = fread(fid1,1,'ubit3');  % read out filler bits
    metaDataCh2_10us.LinkNum = fread(fid1,1,'ubit9');
    temp = fread(fid1,1,'ubit3');  % read out filler bits
    metaDataCh2_10us.PacketCount = fread(fid1,1,'ubit12');
    metaDataCh2_10us.DestAddr = fread(fid1,1,'uint32');  % DMA destination address
    metaDataCh2_10us.StartTime = datestr(fread(fid1,1,'uint32','l')/86400 +  datenum(1970,1,1,0,0,0));
    footer = fread(fid1,4,'int8');
    framesread = framesread +1;
    if(metaDataCh1_10us.LinkNum ~= metaDataCh2_10us.LinkNum)   %Then this is 1us data not 10 us start over
        header = fread(fid1,4,'int8');% read out header
        continue;
    end
    
    %     adcCh2_10us = adcCh2_10us/16*Res;
    if header ~= [-91 3 2 1]'
        disp('Frame Error');
    end
    if footer ~= [-91 1 2 3]'
        disp('Frame Error');
    end
    %1us Ch1
    header = fread(fid1,4,'int8');% read out header
    adcCh1_1us=fread(fid1,datalen1us,'int16');
    
    metaDataCh1_1us.TimeOffset = fread(fid1,1,'uint32');
    metaDataCh1_1us.PPSCount = fread(fid1,1,'uint32');
    metaDataCh1_1us.GateNum = fread(fid1,1,'uint16');
    temp = fread(fid1,1,'uint16');  % read out filler bits
    metaDataCh1_1us.TrigNum = fread(fid1,1,'uint16');
    temp = fread(fid1,1,'uint16');  % read out filler bits
    metaDataCh1_1us.SampleCount = fread(fid1,1,'uint32');
    metaDataCh1_1us.AdcChan = fread(fid1,1,'ubit4');
    metaDataCh1_1us.FifoOverflow = fread(fid1,1,'ubit1');
    temp = fread(fid1,1,'ubit3');  % read out filler bits
    metaDataCh1_1us.LinkNum = fread(fid1,1,'ubit9');
    temp = fread(fid1,1,'ubit3');  % read out filler bits
    metaDataCh1_1us.PacketCount = fread(fid1,1,'ubit12');
    metaDataCh1_1us.DestAddr = fread(fid1,1,'uint32');  % DMA destination address
    metaDataCh1_1us.StartTime = datestr(fread(fid1,1,'uint32','l')/86400 +  datenum(1970,1,1,0,0,0));
    footer = fread(fid1,4,'int8');
    %     adcCh1_1us = adcCh1_1us/16*Res;
    framesread = framesread +1;
    if header ~= [-91 3 2 1]'
        disp('Frame Error');
    end
    if footer ~= [-91 1 2 3]'
        disp('Frame Error');
    end
    %1us Ch2
    header = fread(fid1,4,'int8');% read out header
    adcCh2_1us=fread(fid1,datalen1us,'int16');
    metaDataCh2_1us.TimeOffset = fread(fid1,1,'uint32');
    metaDataCh2_1us.PPSCount = fread(fid1,1,'uint32');
    metaDataCh2_1us.GateNum = fread(fid1,1,'uint16');
    temp = fread(fid1,1,'uint16');  % read out filler bits
    metaDataCh2_1us.TrigNum = fread(fid1,1,'uint16');
    temp = fread(fid1,1,'uint16');  % read out filler bits
    metaDataCh2_1us.SampleCount = fread(fid1,1,'uint32');
    metaDataCh2_1us.AdcChan = fread(fid1,1,'ubit4');
    metaDataCh2_1us.FifoOverflow = fread(fid1,1,'ubit1');
    temp = fread(fid1,1,'ubit3');  % read out filler bits
    metaDataCh2_1us.LinkNum = fread(fid1,1,'ubit9');
    temp = fread(fid1,1,'ubit3');  % read out filler bits
    metaDataCh2_1us.PacketCount = fread(fid1,1,'ubit12');
    metaDataCh2_1us.DestAddr = fread(fid1,1,'uint32');  % DMA destination address
    metaDataCh2_1us.StartTime = datestr(fread(fid1,1,'uint32','l')/86400 +  datenum(1970,1,1,0,0,0));
    footer = fread(fid1,4,'int8');
    %     adcCh2_1us = adcCh2_1us/16*Res;
    
    framesread = framesread +1;
    if header ~= [-91 3 2 1]'
        disp('Frame Error');
    end
    if footer ~= [-91 1 2 3]'
        disp('Frame Error');
    end
    header = fread(fid1,4,'int8');% read out next header
    if ftell(fid1) > filesize - 48160
        break;
    end
    i = i+1;
end
if(dispMetadata ==1)
    metaDataCh1_10us
    metaDataCh2_10us
    metaDataCh1_1us
    metaDataCh2_1us
end

framesread
numFramesSkipped = framesread - line_num*4

fclose(fid1);
t10us = linspace(0,datalen10us*1/fs-1/fs,datalen10us);
t1us = linspace(0,datalen1us*1/fs-1/fs,datalen1us);

adcCh1_1us = detrend(adcCh1_1us,'constant')*elephantstoVolts;
adcCh2_1us = detrend(adcCh2_1us,'constant')*elephantstoVolts;
adcCh1_10us = detrend(adcCh1_10us,'constant')*elephantstoVolts;
adcCh2_10us = detrend(adcCh2_10us,'constant')*elephantstoVolts;

if usenotchfilter
    for ff = 1:numNotch
        adcCh1_10us = filter(notchFilterObj(ff),adcCh1_10us);
        adcCh1_1us = filter(notchFilterObj(ff),adcCh1_1us);
        adcCh2_10us = filter(notchFilterObj(ff),adcCh2_10us);
        adcCh2_1us = filter(notchFilterObj(ff),adcCh2_1us);
    end
end

if(exist('reffile','var') && IdealRef ==0)
    load(reffile)
    ref10us = sig10usref;
    ref1us = sig1usref;
    if isrow(ref10us)
        ref10us = ref10us';
    end
    if isrow(ref1us)
        ref1us = ref1us';
    end
    Nrefp2 = length(ref1us);
    Nrefp1 = length(ref10us);
    ref1us = ref1us/max(ref1us);
    ref10us = ref10us/max(ref10us);
    t1 = linspace(0,Nrefp1-1,Nrefp1)'/fs;
    t2 = linspace(0,Nrefp2-1,Nrefp2)'/fs;
else
    % Generate Reference signals
    t1 = [0:1/fs:pw1-1/fs]';
    t2 = [0:1/fs:pw2-1/fs]';
    % compute chirp phase
    freq1 = fc-bw/2 + bw/pw1*t1/2;
    freq2 = fc-bw/2 + bw/pw2*t2/2;
    fprintf('Using Ideal Reference\n');
    ref10us = exp(1j*2*pi*freq1.*t1);
    ref1us = exp(1j*2*pi*freq2.*t2);
    clear freq1 freq2
    % Figure out size of reference signals:
    Nrefp2 = length(ref1us);
    Nrefp1 = length(ref10us);
end

% Apply taper to reference signal:
if usetukey
    ref1us = ref1us.*tukeywin(Nrefp2,taper);
    ref10us = ref10us.*tukeywin(Nrefp1,taper);
end

if usewin
    if strcmp(wintype,'kaiser')
        ref1us = ref1us.*kaiser(Nrefp2,winparam);
        ref10us = ref10us.*kaiser(Nrefp1,winparam);
    elseif strcmp(wintype,'cheb')
        ref1us = ref1us.*chebwin(Nrefp2,winparam);
        ref10us = ref10us.*chebwin(Nrefp1,winparam);
    else
        warning('PulseCompress:InvalidWinType',...
            'Parameter "wintype" must be a string equal to "cheb" or "kaiser"\n No window used!')
    end
end


if(plotvsTime)
    aScope(1) = figure;
    ch110us = subplot(2,1,1);
    plot(t10us*1e6,adcCh1_10us)
    title({'Adc Acquired Data 10us';'Ch1'})
    xlabel('Time, \muS')
    ylabel('Amplitude, mV')
    grid
    ch210us = subplot(2,1,2);
    plot(t10us*1e6,adcCh2_10us)
    title('Ch2');
    xlabel('Time, \muS')
    ylabel('Amplitude, mV')
    grid
    linkaxes([ch110us ch210us],'xy')
    aScope(2) =figure
    ch11us = subplot(2,1,1);
    plot(t1us*1e6,adcCh1_1us)
    title({'Adc Acquired Data 1us';'Ch1'})
    xlabel('Time, \muS')
    ylabel('Amplitude, mV')
    grid
    ch21us = subplot(2,1,2);
    plot(t1us*1e6,adcCh2_1us)
    title('Ch2');
    xlabel('Time, \muS')
    ylabel('Amplitude, mV')
    grid
    linkaxes([ch11us ch21us],'xy')
    
else
    aScope(1) = figure;
    ch110us = subplot(2,1,1);
    plot(adcCh1_10us)
    title({'Adc Acquired Data 10us';'Ch1'})
    xlabel('Time, \muS')
    ylabel('Amplitude, mV')
    grid
    ch210us = subplot(2,1,2);
    plot(adcCh2_10us)
    title('Ch2');
    xlabel('Time, \muS')
    ylabel('Amplitude, mV')
    grid
    linkaxes([ch110us ch210us],'xy')
    aScope(2)= figure;
    ch11us= subplot(2,1,1);
    plot(adcCh1_1us)
    title({'Adc Acquired Data 1us';'Ch1'})
    xlabel('Time, \muS')
    ylabel('Amplitude, mV')
    grid
    ch21us = subplot(2,1,2);
    plot(adcCh2_1us)
    title('Ch2');
    xlabel('Time, \muS')
    ylabel('Amplitude, mV')
    grid
    linkaxes([ch11us ch21us],'xy')
end

L = length(t10us);
NFFT = 2^nextpow2(L);   % Next power of 2 from length of y
F10Ch1 = 2*abs(fft(double(adcCh1_10us(fftrange10)).*hann(numel(fftrange10)),NFFT))/L;
fif = fs/2*linspace(0,1,NFFT/2+1);
F10Ch2 = 2*abs(fft(double(adcCh2_10us(fftrange10)).*hann(numel(fftrange10)),NFFT))/L;

fftplots(1) = figure;
subplot 211
plot(fif/1e6,10*log10(F10Ch1(1:NFFT/2+1)))
title({'Amplitude Spectrum of IF 10us Sig';'Ch1'})
xlabel('Frequency (MHz)')
ylabel('|F_Tx_IF| (dB)')
grid on
subplot 212
plot(fif/1e6,10*log10(F10Ch2(1:NFFT/2+1)))
title({'Amplitude Spectrum of IF 10us Sig';'Ch2'})
xlabel('Frequency (MHz)')
ylabel('|F_Tx_IF| (dB)')
grid on

L = length(t1us);
NFFT = 2^nextpow2(L);   % Next power of 2 from length of y
F1Ch1 = 2*abs(fft(double(adcCh1_1us(fftrange1)).*hann(numel(fftrange1)),NFFT))/L;
fif = fs/2*linspace(0,1,NFFT/2+1);
F1Ch2 = 2*abs(fft(double(adcCh2_1us(fftrange1)).*hann(numel(fftrange1)),NFFT))/L;

fftplots(2) = figure;
subplot 211
plot(fif/1e6,10*log10(F1Ch1(1:NFFT/2+1)))
title({'Amplitude Spectrum of IF 1us Sig';'Ch1'})
xlabel('Frequency (MHz)')
ylabel('|F_Tx_IF| (dB)')
grid on
subplot 212
plot(fif/1e6,10*log10(F1Ch2(1:NFFT/2+1)))
title({'Amplitude Spectrum of IF 1us Sig';'Ch2'})
xlabel('Frequency (MHz)')
ylabel('|F_Tx_IF| (dB)')
grid on

if(useIQ == 1)
    ref10us =  conj((cos(2*pi*fc*t1) - 1i*sin(2*pi*fc*t1)).*ref10us);
    ref1us =  conj((cos(2*pi*fc*t2) - 1i*sin(2*pi*fc*t2)).*ref1us);
    
    adcCh1_10us = filtfilt(dicelpf,1,(cos(2*pi*fc*t10us) - 1i*sin(2*pi*fc*t10us))'.*adcCh1_10us);
    adcCh1_1us = filtfilt(dicelpf,1,(cos(2*pi*fc*t1us) - 1i*sin(2*pi*fc*t1us))'.*adcCh1_1us);
    adcCh2_10us = filtfilt(dicelpf,1,(cos(2*pi*fc*t10us) - 1i*sin(2*pi*fc*t10us))'.*adcCh2_10us);
    adcCh2_1us = filtfilt(dicelpf,1,(cos(2*pi*fc*t1us) - 1i*sin(2*pi*fc*t1us))'.*adcCh2_1us);
end

radechoCh1_10us = xcorr(adcCh1_10us,ref10us);
radechoCh1_10us = radechoCh1_10us(datalen10us:2*datalen10us-1);
radechoCh1_1us = xcorr(adcCh1_1us,ref1us);
radechoCh1_1us = radechoCh1_1us(datalen1us:2*datalen1us-1);
radechoCh2_10us = xcorr(adcCh2_10us,ref10us);
radechoCh2_10us = radechoCh2_10us(datalen10us:2*datalen10us-1);
radechoCh2_1us = xcorr(adcCh2_1us,ref1us);
radechoCh2_1us = radechoCh2_1us(datalen1us:2*datalen1us-1);

%% Compressed Signal Plots
figCompressed(1) = figure;
ch110us = subplot(2,1,1);
plot(abs(radechoCh1_10us))
title({'Compressed';'10usCh1'})
ylabel('Amplitude')
grid

ch210us = subplot(2,1,2);
plot(abs(radechoCh2_10us))
title({'Compressed';'10usCh2'})
ylabel('Amplitude')
grid
% linkaxes([ch110us ch210us],'xy')
figCompressed(2) =figure;
ch11us = subplot(2,1,1);
plot(abs(radechoCh1_1us))
title({'Compressed';'1usCh1'})
ylabel('Amplitude')
grid

ch21us = subplot(2,1,2);
plot(abs(radechoCh2_1us))
title({'Compressed';'1usCh2'})
ylabel('Amplitude')
grid
% linkaxes([ch11us ch21us],'xy')

if(saveAscope)
    savefig(aScope,[filename(1:end-5),'_aScope','.fig']);
end
if(saveFFTfig)
    savefig(fftplots,[filename(1:end-5),'_fft','.fig']);
end
if(saveCompressed)
    savefig(figCompressed,[filename(1:end-5),'_Compressed','.fig']);
end