%% This reads the dice data file and averages all the echos in the file to 
% produce a reference chirp for radar processing. 

close all; clear all; clc;
fs = 150e6;     % Sampling Frequency
sw1 = 80e-6;    % Sampling Period 10 us Chirp
sw2 = 80e-6;    % Sampling Period 1 us Chrip
fc = 38e6;
bw = 60e6;
datalen10us = int32(fs*sw1);   % Number of samples per sweep 10 us Chirp
datalen1us = int32(fs*sw2);    % Number of samples per sweep 1 us Chirp
t10us = 0:1/fs:(sw1-1/fs);
t1us = 0:1/fs:(sw2-1/fs);
numTracesToAverage = 1000;     % Not too many if surface reflection is being used

% comment this to manually select a file every time
% fid1 = fopen('E:\DiceTestDataRS1\raw\DICE\F1006\AN02_F1006_20151117_080732_0160.dice','r'); % File namw with Ext

% select a file to start
if ~exist('fid1','var')
[filename,pathname] = uigetfile('*.dice','Select a DICE file to process');
fid1 = fopen(fullfile(pathname,filename));
end

fseek(fid1,0,'eof');   % Go to the end of the file
filesize = ftell(fid1);    % Get the filesize
fseek(fid1,0,'bof');   % Go to the beginning of the file
% Some files do not start with START frame
header = zeros(4,1);
footer = zeros(4,1);
metadata = zeros(8,1);
adcCh1_10us = zeros(datalen10us,1);
adcCh2_10us = zeros(datalen10us,1);
adcCh1_1us = zeros(datalen1us,1);
adcCh2_1us = zeros(datalen1us,1);
sig10usch1 = adcCh1_10us;
sig10usch2 = adcCh2_10us;
sig1usch1 = adcCh1_1us;
sig1usch2 = adcCh2_1us;
    
linesread = 0;
while 1 %find first header
    header(1) = fread(fid1,1,'int8');% read out header
    if(header(1) == -91)
        header(2) = fread(fid1,1,'int8');
        if(header(2) == 3)
            header(3) = fread(fid1,1,'int8');
            if(header(3) ==2)
                header(4) = fread(fid1,1,'int8');
                if(header(4) == 1)
                    break;
                end
            end
        end
    end
end
while (linesread < numTracesToAverage)
adcCh1_10us=fread(fid1,datalen10us,'int16');
metaDataCh1_10us = fread(fid1,8,'uint32');
footer = fread(fid1,4,'int8');
adcCh1_10us = adcCh1_10us/16;
linesread = linesread +1;
if header ~= [-91 3 2 1]' 
    disp('Frame Error');
end
if footer ~= [-91 1 2 3]' 
    disp('Frame Error');
    end

header = fread(fid1,4,'int8');% read out header
adcCh2_10us=fread(fid1,datalen10us,'int16');
metaDataCh2_10us = fread(fid1,8,'uint32');
footer = fread(fid1,4,'int8');
adcCh2_10us = adcCh2_10us/16;
linesread = linesread +1;
if header ~= [-91 3 2 1]' 
    disp('Frame Error');
end
if footer ~= [-91 1 2 3]' 
    disp('Frame Error');
end
header = fread(fid1,4,'int8');% read out header
adcCh1_1us=fread(fid1,datalen1us,'int16');
metaDataCh1_1us = fread(fid1,8,'uint32');
footer = fread(fid1,4,'int8');
adcCh1_1us = adcCh1_1us/16;
linesread = linesread +1;
if header ~= [-91 3 2 1]' 
    disp('Frame Error');
end
if footer ~= [-91 1 2 3]' 
    disp('Frame Error');
end
header = fread(fid1,4,'int8');% read out header
adcCh2_1us=fread(fid1,datalen1us,'int16');
metaDataCh2_1us = fread(fid1,8,'uint32');
footer = fread(fid1,4,'int8');
adcCh2_1us = adcCh2_1us/16;

linesread = linesread +1;
if header ~= [-91 3 2 1]' 
    disp('Frame Error');
end
if footer ~= [-91 1 2 3]' 
    disp('Frame Error');
end
if(linesread >1)
    sig10usch1 = (sig10usch1 +  detrend(adcCh1_10us,'constant'))/2;
    sig10usch2 = (sig10usch2 +  detrend(adcCh2_10us,'constant'))/2;
    sig1usch1 = (sig1usch1 +  detrend(adcCh1_1us,'constant'))/2;
    sig1usch2 = (sig1usch2 +  detrend(adcCh2_1us,'constant'))/2;
else
    sig10usch1 = detrend(adcCh1_10us,'constant');
    sig10usch2 = detrend(adcCh2_10us,'constant');
    sig1usch1 = detrend(adcCh1_1us,'constant');
    sig1usch2 = detrend(adcCh2_1us,'constant');
end
if ftell(fid1) > filesize - 48160
    break;
end 
header = fread(fid1,4,'int8');% read out header
end

figure
ax(1) = subplot(2,1,1);
plot(sig10usch1)
grid
title('10us Signal Ch1')
xlabel('Amplitude')
ylabel('Samples')
ax(2) = subplot(2,1,2);
plot(sig1usch1)
grid
title('1us Signal Ch1')
xlabel('Amplitude')
ylabel('Samples')
linkaxes([ax(1) ax(2)],'xy')
figure
ax(1) = subplot(2,1,1);
plot(sig10usch2)
grid
title('10us Signal Ch2')
xlabel('Amplitude')
ylabel('Samples')
ax(2) = subplot(2,1,2);
plot(sig1usch2)
grid
title('1us Signal Ch2')
xlabel('Amplitude')
ylabel('Samples')
linkaxes([ax(1) ax(2)],'xy')

L = length(t1us);
NFFT = 2^nextpow2(L);   % Next power of 2 from length of y
F10Ch1 = 2*abs(fft(double(sig10usch1(336:1835)),NFFT))/L;
fif = fs/2*linspace(0,1,NFFT/2+1)';
F1Ch1 = 2*abs(fft(double(sig1usch1(320:469)),NFFT))/L;
figure
subplot 211
plot(fif/1e6,10*log10(F10Ch1(1:NFFT/2+1)))
title({'Amplitude Spectrum of IF 10us Sig';'Ch1'})
xlabel('Frequency (MHz)')
ylabel('|F_Tx_IF| (dB)')
grid on
subplot 212
plot(fif/1e6,10*log10(F1Ch1(1:NFFT/2+1)))
title({'Amplitude Spectrum of IF 1us Sig';'Ch1'})
xlabel('Frequency (MHz)')
ylabel('|F_Tx_IF| (dB)')
grid on
NFFT = 2^nextpow2(L);   % Next power of 2 from length of y
F10Ch2 = 2*abs(fft(double(sig10usch2(336:1835)),NFFT))/L;
F1Ch2 = 2*abs(fft(double(sig1usch2(330:469)),NFFT))/L;
figure
subplot 211
plot(fif/1e6,10*log10(F10Ch2(1:NFFT/2+1)))
title({'Amplitude Spectrum of IF 10us Sig';'Ch2'})
xlabel('Frequency (MHz)')
ylabel('|F_Tx_IF| (dB)')
grid on
subplot 212
plot(fif/1e6,10*log10(F1Ch2(1:NFFT/2+1)))
title({'Amplitude Spectrum of IF 1us Sig';'Ch2'})
xlabel('Frequency (MHz)')
ylabel('|F_Tx_IF| (dB)')
grid on
% ind = find(fif>=7.5e6 & fif<=68.5e6);
% xdata = fif(ind);
% ydata10 = F10Ch1(ind);
% ydata1 = F1Ch1(ind);
% fit10usCh1 = createFit(fif(ind),F10Ch1(ind),12);
% fit10usCh2 = createFit(fif(ind),F10Ch2(ind));
% fit1usCh1 = createFit(fif(ind),F1Ch1(ind));
% fit1usCh2 = createFit(fif(ind),F1Ch2(ind));

