% Tej Dhakal 04/02/2015
% LDEO, Columbia University
% This Plots DICE data processed with processing code v1.0 and above
% A png file with 600 dpi resolution is saved with 'figname'

clc; close all; clearvars;
plotterVersion = 3.16;
%% Data path and plot times
% User will be prompted if incorrect
% dataPath = '/local/data/hestia/theater/Antarctica/20162017AN03/Radar/DICE/level2/Stack01/L890/mat';
dataPath = 'E:\trunk\icepod\antarctica\20162017\radar\dice\sar\stack01\gs\mat\';
%outputpath = '/home/dhakal/diceTestPlots/';     % comment this to save figures in dataPath\plots\

startplottime = '01-Nov-2015 00:00:00';
endplottime = '30-Dec-2016 00:00:00';

plotSig = {'3usCh2'};       % one of '3usCh1', '3usCh2', '1usCh1' or '1usCh2 Keep this format - no spaces

%% Plot Configurations
QuickPlotAll = true;        % enable this to plot all files in the directory without prompt
multiplePlots = true;      % setting this to 0 will generate single plot from start time to end time

%%
surfZero = true;            % Sets zero at surface at the begining of the plot
refsurf = 2500;             % ft, below the aircraft (air distance) - if no surface is found this will be used
% userRefElev = 2500;       % ft. uncomment for use. comment to use first gps elev as reference axes and tick labels
elevcorrection = true;      % corrects elevation using gps, if available. Elevation is adjusted relative to a refelev

%% Figure stuff
plotXaxis = 'dist';         % 'dist' or 'time'. Will plot data against distance or time. This is automatically added to displayAxes List
displayAxes = {plotXaxis,'time','pos','dist'}; % 'time','dist','pos','' ->time in HH:MM:SS, distance in km, Lat/Lon in degrees , or just regular datetick along the bottom
plotPeriod = 60;            % minutes or km depending on plotXaxis type. Only used if multiplePlots = true;
showScale = false;          % This will put the distance scale in the image
tickSpacing = 0;           % same unit as plotPeriod minutes or km, set it to zero to use maxXticksNum (defined later)
clims = [-60,40];           % Change limits to change image contrast
ylims = [100, -500];        % [above_surface -below_surface] meters in ice about the surface, ignored if surfZero = false
cmap = 'gray';              % valid colormap string
invertcaxis = true;         % 0 or 1
reversexaxis = false;       % reverse x axis direction
cutterH = 8;               % horizontal decimation, uncomment to use - 1 for pulse compressed & 8 for sar used by default otherwise
% cutterV = 1;              % vertical decimation, uncomment to use - 1 by default otherwise
smoothCutter = true;       % When enabled, applies smoothing window equal to decimation factor
diffPlot = false;            % Pop Layers using vertical derivative

%% figure display and printing parameters
contUpdateFig = false;      % draws the figure every file - slower
printFigure = false;         % clearing this will plot wihout saving the pngs
pngpagesize = [15,8];       % inches
pdfpagesize = [11,8];       % inches
pngresolution = 600;        % dpi
pdfresolution = 300;        % dpi
% renderer for bitmap for pdf
if verLessThan('matlab','8.4')  % earlier than 2014b
    pdfrenderer = 'zbuffer';
else
    pdfrenderer = 'opengl';
end
pngrenderer = 'painters';
outputPdf = false;          % create pdf of image
outputPng = true;           % create pnf of image

%% More Figure Stuff
% These settings are used to automatically determine whether the plot
% direction need to be reversed - not done yet!
autoPlotDirChange = false;  % Enable Automatic plot direction reversal, if false 'reversexaxis' value will take effect
linePlotDir = 'EW';         % WE (West Antarctica to East Antractica) and vice-versa
tieLinePlotDir = 'SN';      % NS (North to South), SN (South to North) 'True' direction -(not grid)

% Following settings are used to scale the png image x and y axes when multiplePlots are disabed
% these scalers make sure that the x an y scales on plots with varying x
% and y axes have identical ppi
maxLineLength = 600;        % km
maxLineDepth = 2000;        % m - this is the total range from plane to most distant target plotted
maxPageSize = [120, 40];    % in

% Crude way of detecting if image is looge to print in a single image
% Plotter seems to fail @ 600 dpi for 600 m by 450 km image
% Lets keep the image size smaller than that for multiplot = false cases
breakUpImage = false;        % if true, breaks up a line into multiple images as needed
singleImageLimit = 600*300/10/1;    %m-km image area 600 m * 450 km /10 cutterH/1 cutterV
minImageLen = 150;          % km, Ticks wont look right if image is shorter than this

% surfaces and layers
surfType = 'first';          % 'first','max','both' or 'none'. first = closest to aircraft, max = maximum power
surfLineColor = {'-m'};            % LineSpec for surface(s)
surfLegend = {'First'};     % Legends for surface line(s)
legendPos = 'NorthEast';    % legend location - try help legend
%
%
%
%
%
%% Path for support functions required for sir processing
matlab_common_path = '/local/data/hestia/software/GIT2/common/Matlab';
if ~(exist(matlab_common_path,'dir')==7)
    warning('diceProcessConfig:InvalidPath','Invalid path for matlab common functions. Please update the path');
else
    addpath(matlab_common_path);
end

%% Set horizontal decimation factor based on data type
if ~exist('cutterH','var')
    if sardata
        cutterH = 8;
    else
        cutterH = 1;
    end
end
if ~exist('cutterV','var')
    cutterV = 1;
end
%% smoothing for plots
if diffPlot                 % set color limits for diff plot
    clims = [-2 5];         %#ok<UNRCH>
end
if diffPlot     % use 8 or cutterV/cutterH (if >8), if diffPlot
    x_smooth_win = 50;
    y_smooth_win = 8;
else
    if ~smoothCutter
        x_smooth_win = 1;
        y_smooth_win = 1;
    else
        x_smooth_win = cutterH;
        y_smooth_win = cutterV;
    end
end
detrendy = false;
%
%

%% Close the figures once saved. Prevent figures from filling up the memory
closeFigures = false;
if(multiplePlots && ~closeFigures)
    warning('dicePlotTool:figuresInMem','Set closeFigure to true to avoid memory issues when multiplePlots is true');
end

%% Additional Info
surveySpeed = 100;          % m/s, will be used as backup if no gps is available.

%% This is for plotting already-picked layers in the echogram
% plot other available surfaces - these are lines across the image
% each line is required to have a time vector and a surface vector
% the variable names, data files and location is defined here
% The data is assumed to be a single mat file for the entire flight/line
% plotted. Time should be in unix time and layer data in index number
% of the radar echo

plotLayers = false;         % flag to enable plotting of layers
numLayers = 1;              % number of additional surface to plot

% For each additional surface assign the following by incrementing the
% cell index

%%%%%%%%%% Layer 1 %%%%%%%%%%%%%%%%%%

layerDataPath{1} = 'E:\AN1\F1014\mat\GrountTrackSurface_Lidar.mat';   % full path and name of the layer mat file
layerVar{1} = 'surfHell';           % variable name for the interface. This is the index number in the radar echo.
layerTimeVar{1} = 'GPSUTC';         % time variable for the layer - this need to be in unix seconds
layerLegend{1} = 'Lidar Surface';   % label for the layer to print
layerLegendPos{1} = 'NorthEast';    % legend position- try help legend. all scatter plots use
layerLineSpec{1} = 'r.';            % Color and Line specification for the layer to be plotted
layerCdataVar{1} = 'Cdata';         % Color array for scatter plots  
layerPlotType{1} = 'line';          % 'line' or 'scatter' uses 'plot' or 'scatter' functions
layerDataType{1} = 'Elev';         % Y-axis  type
% Following layer data types are currently supported
% 'Index' :     This is index number for Dice echogram. Use this for plotting
%               layers from dice Bed-Picks and such
% 'RangeDistIce': This is range (distance) from the aircraft. This could be
%               used for plotting layer from another instrument in the same
%               aircraft, SIR for example. Distance must be in terms of
%               IceDistance (scaled by cIce/cAir.
% 'RangeDistTrue': This is range (distance) from the aircraft. This could be
%               used for plotting layer from another instrument in the same
%               aircraft, SIR  or Lidar for example. Distance must be the
%               true distance (not scaled by cIce/cAir)
% 'DepthDist':  This is depth from the Ice Suraface (-ve below +ve above)
%               any positive values must use cIce/cAir scaling. Can be used
%               to plot data from sources other than IcePod Instruments
% 'RangeTime':  This is the two way time for speed of light from the
%               aircraft. Useful for ploting layers from same or differnt
%               radar form the same a/c.
% 'DepthTime':  This is two way travel time for speed of light from the
%               Ice Surface. Can be used to plot picked layers from same or
%               different radar on IcePod or other platforms
% 'Elev':       This is the Ellipsoid height of the layer to be plotted.
%               This can be used to plot layers from any source including
%               the IcePod Lidar.

%%%%%%%%%% Layer n %%%%%%%%%%%%%%%%%%

% Repeat info above for additional layers
% layerDataPath{n} = '';   % full path to the layer file
% and so on..

%% Radar parameters and constants
cAir = 299792458;                     % m/s
cIce = 1.68e8;                        % m/s

%%  Don't touch these
TxInd = 318;
processChan = [1 2 3 4];
processChanAttbr = {'3usCh1';'3usCh2';'1usCh1';'1usCh2'};
chanAvail = false(size(processChanAttbr));
plotSigChan = ismember(processChanAttbr,plotSig);
outFileDateFormat = 'yyyymmdd';
outFileTimeFormat = 'HHMMSS';
dateField = 3;
timeField = 4;
levelField = 6;
chanField = 5;
fileFieldSep = '_';

% used of printing ticks to file later
latNS = @(x)char((x>=0)*78+(x<0)*83);
lonEW = @(x)char((x>=0)*69+(x<0)*87);

if exist('userRefElev','var')
    userRefElev = userRefElev/3.28;
end

layerData = cell(1,numLayers);
layerTime = cell(1,numLayers);
xaxisDataLayer = cell(1,numLayers);
for nnl = 1:numLayers
    layerData{nnl} = NaN;
    layerTime{nnl} = NaN;
    xaxisDataLayer{nnl} = NaN;
end
layersLoaded = false;
allechotimeutc = [];
allhell = [];
alllat = [];
alllon = [];
allSurfIndRad = [];

maxXticksNum =9;    %% maximum number of ticks used in a figure
% if tickSpacing is > 0, it will be used instead of maxXtickNum
clearvars nnl;

%% Generate a warning if image is not being printed
if ~printFigure
    warning('dicePlotTool:figureNotSaved','Figures will not be saved! \nset printFigure = true to save figures')
    if(closeFigures)
        closeFigures = false; %#ok<UNRCH>
    end
end

%% If quick plot is enabled use the data path specified after verifying it
if (QuickPlotAll)
    if exist('dataPath','var') && ~ isempty(dataPath)
        if ~(dataPath(end) == filesep)
            dataPath = [dataPath filesep];
        end
        if ~(exist(dataPath,'dir')==7)
            warning('dicePlotTool:noValidDataPath','Data path %s is not valid',dataPath);
            QuickPlotAll = false;
        end
    else
        warning('dicePlotTool:invalidDataPth','<dataPath> not defined, Quick Plot will be disabled');
        QuickPlotAll = false;
    end
end

%% if quick plot is not enabled or data path is not valid (see pervious section) prompt user for data path
if  ~QuickPlotAll
    if usejava('desktop')
        dataPath = uigetdir(pwd,'Select dice processed data directory to plot');
    else
        dataPath = input('Enter the data directory to plot (''/data'' or ''e:\''):\n');
    end
    if isempty(dataPath) || numel(dataPath)<2
        error('dicePlotTool:noDirSelected','Aborted: no directory selected');
    end
    dataPath = [dataPath filesep];
end
%
fprintf('Plotting files in dir: %s\n',dataPath);

%% Check and create output directories
if ~exist('outputpath','var')
    outputpath = [dataPath,'..',filesep];
else
    outputpath = [outputpath filesep];
end
if outputPdf && printFigure
    pdfoutputpath = [outputpath 'pdf' filesep];
    if exist(pdfoutputpath,'dir')==0
        mkdir(pdfoutputpath);
    end
end
if outputPng && printFigure
    pngoutputpath = [outputpath 'png' filesep];
    if exist(pngoutputpath,'dir')==0
        mkdir(pngoutputpath);
    end
end
if printFigure && (outputPng || outputPdf)
    txtoutputpath = [outputpath 'txt' filesep];
    if exist(txtoutputpath,'dir')==0
        mkdir(txtoutputpath);
    end
end

%% Find and sort data files and establish available channels
for chan_num = 1:numel(processChan)
    level1Files{processChan(chan_num)}=dir([dataPath,['*',processChanAttbr{processChan(chan_num)},'*.mat']]); %#ok<SAGROW>
    numfiles=numel(level1Files{processChan(chan_num)});
    if(numfiles == 0)
        %
        chanAvail(chan_num) = false;
    else
        chanAvail(chan_num) = true;
        for ii = 1:numfiles
            fileNameFields = strsplit(level1Files{processChan(chan_num)}(ii).name,fileFieldSep);
            try
                old_fileformat = 0;
                level1Files{processChan(chan_num)}(ii).utc = matlab2unixtime(datenum([fileNameFields{dateField},fileNameFields{timeField}],[outFileDateFormat,outFileTimeFormat]));
            catch
                warning('dicePlotTool:wrongFileNameFormat','File name format incorrect - Support for older file names will be removed in future relases')
                warning('off','dicePlotTool:wrongFileNameFormat')
                level1Files{processChan(chan_num)}(ii).utc = matlab2unixtime(datenum([fileNameFields{dateField+1},fileNameFields{timeField+1}],[outFileDateFormat,outFileTimeFormat]));
                old_fileformat = 1;
            end
        end
        [~,sx] = sort([level1Files{processChan(chan_num)}.utc]);
        level1Files{processChan(chan_num)} = level1Files{processChan(chan_num)}(sx); %#ok<SAGROW>
    end
end
if all(~chanAvail)
    error('dicePlotTool:noDataToPlot','No data available to plot\n Please check the data dir');
end
if all(~plotSigChan)
    warning('dicePlotTool:invalidSigType','Invalid signal type selected')
end

%% If Quick plot is enabled, prompt user for channels to plot, if only one channel is availabe use that
if ~QuickPlotAll || all(~(plotSigChan & chanAvail))
    if numel(find(chanAvail)) ==1
        warning('dicePlotTool:singleChanAvail','Only one channel available, plotting %s',processChanAttbr{chanAvail});
        plotSigChan = chanAvail;
    else
        prompt = {'Select a signal to plot:'};
        dlg_title = 'Signal Select';
        availableSigs = processChanAttbr(chanAvail);
        if  usejava('desktop')
            plotSig = listdlg('PromptString',prompt,'SelectionMode','single','ListString',availableSigs);
        else
            fprintf('Select a signal to plot (1-%d)\n',numel(availableSigs));
            for kk = 1:numel(availableSigs)
                fprintf('%d. %s\n',availableSigs{kk});
            end
            plotSig = input(':');
            if ~isnumeric(ploSig)
                error('Invalid Input - must be an number')
            end
            if(plotSig > numel(availableSigs) || plotSig < 1)
                error('Invalid selection')
            end
        end
        if isempty(plotSig)
            error('Aborted: no signal selected')
        else
            plotSig = availableSigs(plotSig);
            plotSigChan = ismember(processChanAttbr,plotSig);
        end
    end
end

%%
plotSigChan = find(plotSigChan);
[~,fileNameOnly,~] = fileparts(level1Files{processChan(plotSigChan)}(1).name);
fileNameFields = strsplit(fileNameOnly,fileFieldSep);
% extract Campaign and Flight info for use in plots later
Campaign = fileNameFields{1};
FlightNo = fileNameFields{2};

ftimeutc = [level1Files{processChan(plotSigChan)}.utc];
numfiles=numel(level1Files{processChan(plotSigChan)});
fileLevel = fileNameFields{levelField};
if strncmp(fileLevel,'level2',numel('level2'))
    sardata = true;
else
    sardata = false;
end
datatag = fileLevel;
if numel(fileNameFields)>levelField
    for ffn = levelField+1:numel(fileNameFields)
        datatag = [datatag fileFieldSep fileNameFields{ffn}]; %#ok<AGROW>
    end
end
if numfiles>1
    filePeriod = mode(round(diff([level1Files{processChan(plotSigChan)}.utc])));
else
    if sardata
        load([dataPath,level1Files{processChan(plotSigChan)}.name],'echoTimeUTC');
        filePeriod = range(echoTimeUTC); clear echoTimeUTC;
    else
        load([dataPath,level1Files{processChan(plotSigChan)}.name],'echotimeutc');
        filePeriod = range(echotimeutc); clear echotimeutc;
    end
end

%% if quick plot is not enabled, allow user to select start and end plot times
if(~QuickPlotAll)
    firstfiletime = unix2matlabtime(ftimeutc(1));
    lastfiletime = unix2matlabtime(ftimeutc(numfiles) + filePeriod);
    AvailTime = {datestr(firstfiletime),datestr(lastfiletime)};
    defaultTime = AvailTime;
    prompt = {['Plot Start Time  (',AvailTime{1},')  :'],...
        ['Plot End Time    (',AvailTime{2},')  :']};
    dlg_title = 'Plot Time Input';
    if  usejava('desktop')
        answer = inputdlg(prompt,dlg_title,1,defaultTime);
    else
        answer{1} = input(['Enter Plot Start Time (''',AvailTime{1},'''):']);
        if ~ischar(answer{1})
            error('Aborted: Invalid Input')
        end
        answer{2} = input(['Enter Plot End Time (''',AvailTime{2},'''):']);
        if ~ischar(answer{2})
            error('Aborted: Invalid Input')
        end
    end
    if isempty(answer)
        error('Aborted: no range selected')
    else
        startplottime = answer{1};
        endplottime = answer{2};
    end
end

startplotutc=matlab2unixtime(datenum(startplottime));
endplotutc=matlab2unixtime(datenum(endplottime));

% This is a cheat to prevent NaNs in distbatch later. If the endplottime is
% exactly the start time of a file, file selector incorrectly picks the next
% file in the list as the last file to plot so the real last file bypasses
% the time-gating filter meant to remove data outside of plot time.
if any(ftimeutc == endplotutc)
    endplotutc = endplotutc-1;
end

%% Select files to plot
startplotfile=find(ftimeutc <= startplotutc,1,'last');
endplotfile=find(ftimeutc > endplotutc,1,'first');

if isempty(startplotfile)
    startplotfile=1;
    startplotutc = level1Files{processChan(plotSigChan)}(1).utc;
end
if isempty(endplotfile)
    endplotfile=numfiles;
    endplotutc = ftimeutc(numfiles) + filePeriod;
else
    endplotfile = endplotfile-1;
end

%% if there are no date to plot within user specified time, re-prompt user
% This section was not written originally to catch user input errors but it
% turned out to be usefult in catching erros in non Quick plot modes
if (endplotfile - startplotfile)< 1
    for ww = 1:3
        warning('Start and End times are not correct')
        firstfiletime = unix2matlabtime(ftimeutc(1));
        lastfiletime = unix2matlabtime(ftimeutc(numfiles) + filePeriod);
        AvailTime = {datestr(firstfiletime),datestr(lastfiletime)};
        defaultTime = {startplottime,endplottime};
        prompt = {['Plot Start Time  (',AvailTime{1},')  :'],...
            ['Plot End Time    (',AvailTime{2},')  :']};
        dlg_title = 'Plot Time Input';
        dlgoptions.window = 'modal';
        if  usejava('desktop')
            answer = inputdlg(prompt,dlg_title,1,defaultTime,dlgoptions);
        else
            answer{1} = input(['Enter Plot Start Time (''',defaultTime{1},'''):']);
            if ~ischar(answer{1})
                warning('Aborted: Invalid Input')
            end
            answer{2} = input(['Enter Plot End Time (''',defaultTime{2},'''):']);
            if ~ischar(answer{2})
                warning('Aborted: Invalid Input')
            end
        end
        if isempty(answer)
            error('Aborted: so range selected')
        else
            startplottime = answer{1};
            endplottime = answer{2};
        end
        startplotutc=matlab2unixtime(datenum(startplottime));
        endplotutc=matlab2unixtime(datenum(endplottime));
        
        if(startplotutc>=endplotutc)
            if usejava('desktop')
                wdlg = warndlg({'Can''t end the plot before it begins...';...
                    'Try Again!'});
                uiwait(wdlg);
            else
                warning('dicePlotTool:InvalidPlotTimes',['Can''t end the plot before it begins...\n'...
                    'Try Again!'])
            end
            if ww ==3
                error('Invalid plot period.');
            end
            continue;
        end
        
        startplotfile=find(ftimeutc <= startplotutc,1,'last');
        endplotfile=find(ftimeutc > endplotutc,1,'first');
        
        if isempty(startplotfile)
            startplotfile=1;
            startplotutc = ftimeutc(1);
        end
        if isempty(endplotfile)
            endplotfile=numfiles;
            endplotutc = ftimeutc(end) + filePeriod;
        else
            endplotfile = endplotfile -1;
        end
        if (endplotfile - startplotfile)>0
            break;
        else
            if ww ==3
                wdlg = warndlg({'There are no files available to plot for selected time period';...
                    'Check the source directory and run the program again'});
                uiwait(wdlg);
                error('No files to plot for specified period');
            end
        end
    end
end

%% Check if all available data are plotted and display a warning
if endplotfile - startplotfile + 1 < numel(ftimeutc)
    warning('dicePlotTool:notAllFilesPlotted',...
        'Not all available data are plotted. Change start/stop times to plot all available data');
end

%% Scan through data files and load Lat,Lon,Elev and surface for pre-plot preparation
tic
distlat = [];
distlon = [];
disthell = [];
distechotimeutc = [];
distsurf = [];
for dff = startplotfile:endplotfile
    if sardata
        load([dataPath,level1Files{processChan(plotSigChan)}(dff).name],'echoTimeUTC','Lat','Lon','FlightElev','SurfInds');
        if  exist('Lat','var') && exist('Lon','var') && exist('FlightElev','var')
            distlat = [distlat Lat]; clear Lat; %#ok<AGROW>
            distlon = [distlon Lon]; clear Lon; %#ok<AGROW>
            disthell = [disthell FlightElev]; clear FlightElev; %#ok<AGROW>
        else
            warning('dicePlotTool:noGPSData','No gps data available for one or more files');
            warning('off','dicePlotTool:noGPSData');
        end
        if exist('echoTimeUTC','var')
            distechotimeutc = [distechotimeutc echoTimeUTC]; clear echoTimeUTC; %#ok<AGROW>
        end
        if exist('SurfInds','var')
                        if(size(SurfInds,1)>size(SurfInds,2))
                SurfInds = SurfInds';
            end
            SurfInds(SurfInds<2) = NaN;
            distsurf = [distsurf min(SurfInds,[],1)]; clear SurfInds; %#ok<AGROW>
        end
    else
        load([dataPath,level1Files{processChan(plotSigChan)}(dff).name],'echotimeutc','lat','lon','hell','surfIndRad');
        if exist('lat','var') && exist('lon','var') && exist('hell','var')
            distlat = [distlat lat]; clear lat; %#ok<AGROW>
            distlon = [distlon lon]; clear lon; %#ok<AGROW>
            disthell = [disthell hell]; clear hell; %#ok<AGROW>            
        else
            warning('dicePlotTool:noGPSData','no GPS data available for one or more files');
            warning('off','dicePlotTool:noGPSData');
        end
        if exist('echotimeutc','var')
            distechotimeutc = [distechotimeutc echotimeutc]; clear echotimeutc; %#ok<AGROW>
        end
        if exist('surfIndRad','var')
                    if(size(surfIndRad,1)>size(surfIndRad,2))
                surfIndRad = surfIndRad';
            end
            surfIndRad(surfIndRad<2) = NaN;
            distsurf = [distsurf min(surfIndRad,[],1)]; clear surfIndRad; %#ok<AGROW>
        end
    end
end

% Find a first good surface
% min(distsurf(:,find(min(distsurf(:,1))>2),1,'first'))
firstGoodSurf = find(distsurf>2,1,'first');
if ~isempty(firstGoodSurf)
    firstGoodSurf = distsurf(firstGoodSurf);
else
    firstGoodSurf = 0;
end
clear distsurf 

inrangeInd = distechotimeutc >= startplotutc & distechotimeutc <= endplotutc;
distechotimeutc = distechotimeutc(inrangeInd);
if ~isempty(distlat) && ~isempty(distlon) && ~isempty(disthell)
    distlat = distlat(inrangeInd);
    distlon = distlon(inrangeInd);
    disthell = disthell(inrangeInd);
end

%% Deal with Nans found in position data
if any(isnan(distlat)) || any(isnan(distlon)) || any(isnan(disthell))
    maxNumNanVals = max([numel(find(isnan(distlat))),numel(find(isnan(distlat))),numel(find(isnan(distlat)))]);
    %if more than 20% of position data are Nan revert to plotting by
    %time
    if maxNumNanVals>0.2*numel(distechotimeutc)
        warning('dicePlotTool:tooManyNanPosData','Number of NaN in position data exceeds 20%%');
        if strcmpi('dist',plotXaxis)
            plotXaxis = 'time';
            plotPeriod = round(plotPeriod*1000/surveySpeed/60); % km*1000/m/s/60  km -> min
            tickSpacing = round(tickSpacing*1000/surveySpeed/60);
            warning('dicePlotTool:switchPlotXaxis',['Too many Nans in position data!\n',...
                '\tplot x-axis has been changed to time\n\tplot period has been changed to %d min\n',...
                '\ttickSpacing has been changed to %d min'],plotPeriod,tickSpacing);
        end
    else
        warning('dicePlotTool:someNansFound',['Some nans values found in position data!\n',...
            'Start and End plot times will be adjusted If Nans are found at the start and end of the period']);
        % find the first and last data points those are not NaN
        firstNotNan = find(~isnan(distlat + distlon + disthell),1,'first');
        lastNotNan = find(~isnan(distlat + distlon + disthell),1,'last');
        startplotutc = distechotimeutc(firstNotNan);
        endplotutc = distechotimeutc(lastNotNan);
        distlat = distlat(firstNotNan:lastNotNan);
        distlon = distlon(firstNotNan:lastNotNan);
        disthell = disthell(firstNotNan:lastNotNan);
        distechotimeutc = distechotimeutc(firstNotNan:lastNotNan);
        
        % find files within the time range after removing nan data points
        startplotfile=find(ftimeutc <= startplotutc,1,'last');
        endplotfile=find(ftimeutc > endplotutc,1,'first');
        
        if isempty(endplotfile)
            endplotfile = numfiles;
        else
            enplotfile = endplotfile -1;
        end
        % interpolate out any Nan values within within the position values
        if any(isnan(distlat))
            distlat(isnan(distlat)) = interp1(echotimeutc(~isnan(distlat),distlat(~isnan(distlat)),echotimeutc(isnan(distlat))));
        end
        if any(isnan(distlon))
            distlon(isnan(distlon)) = interpDegWithJumps(echotimeutc(~isnan(distlon),distlon(~isnan(distlon)),echotimeutc(isnan(distlon))));
        end
        if any(isnan(disthell))
            disthell(isnan(disthell)) = interp1(echotimeutc(~isnan(disthell),disthell(~isnan(disthell)),echotimeutc(isnan(disthell))));
        end
    end
end

if isempty(distlat) || isempty(distlon) || isempty(disthell) || isempty(distechotimeutc)
    displayAxes = {''};
    warning('dicePlotTool:noGPSDataForAllFiles','no GPS data available for any file!\n displayAxes has bee changed to ''time''')
    if strcmpi('dist',plotXaxis)
        warning('dicePlotTool:noGPSDataForAllFiles','no GPS data available for any file!\n plot x-axis will be changed to time')
        plotXaxis = 'time';
        plotPeriod = round(plotPeriod*1000/surveySpeed/60); % km*1000/m/s/60  km -> min
        tickSpacing = round(tickSpacing*1000/surveySpeed/60);
        warning('dicePlotTool:noGPSDataForAllFiles',['No GPS data available for any file!\n',...
            '\tplot x-axis has been changed to time\n\tplot period has been changed to %d min\n',...
            '\ttickSpacing has been changed to %d min'],plotPeriod,tickSpacing);
    end
else
    %%%%% This bit implements a fix for jumps in longitude
    distlon(distlon<0) = distlon(distlon<0)+360;
    dLonInd = find(abs(diff(distlon))>1);    % Any jump more than 1 deg
    if isempty(dLonInd)
        distlon(distlon>180) = distlon(distlon>180)-360;
    else
        dTimeInd = find(diff(distechotimeutc)>filePeriod/5);      % Any jump more than ~ 2s
        if ~isempty(dTimeInd)
            warning('dicePlotTool:dataGapsFound','Gaps found in data - review flight logs')
            for ddg = 1:numel(dTimeInd)
                fprintf('Data gaps found at: %s\n',datestr(unix2matlabtime(distechotimeutc(dTimeInd(ddg)))));
            end
        end
        dLonInd = dLonInd(~ismember(dLonInd,dTimeInd));     % find all the longitude indices that are not same as time indices
        if isempty(dLonInd)
            distlon(distlon>180) = distlon(distlon>180)-360;
        else
            warning('dicePlotTool:lonJumpFound',['%d jumps found in longitude for %s \n',...
                'Re-run diceGeoLocate for level1 data\n',...
                'Re-process SAR with neighboring files and replace the file in concern\n',...
                'Plotting will continue..'],numel(dLonInd),fileNameFields{2});
            fprintf('Longitude jump found at:\n');
            for ljj = 1: numel(dLonInd)
                fprintf('%s\n',datestr(unix2matlabtime(distechotimeutc(dLonInd(ljj)))))
            end
            dLonInd = unique([dLonInd(dLonInd ==1); dLonInd(dLonInd(1:end-1)+1 == dLonInd(2:end));dLonInd(dLonInd ==numel(distlon)-1)]);
            midInds = dLonInd(dLonInd > 1 & dLonInd<numel(distlon)-1);
            distlon(midInds+1) = 0.5*(distlon(midInds) + distlon(midInds+2));
            % Take care of the case when the jump is the first or last data point
            if any(dLonInd ==1)
                distlon(1) = interpDegWithJumps(distechotimeutc(2:3),distlon(2:3),distechotimeutc(1),'linear','extrap');
            end
            if any(dLonInd ==dLonInd<numel(distlon))
                distlon(end) = interpDegWithJumps(distechotimeutc(end-2:end-1),distlon(end-2:end-1),distechotimeutc(end),'linear','extrap');
            end
            distlon(distlon>180) = distlon(distlon>180)-360;
        end
    end
    %%%%%%%%
    [dx,dy,~] = geodetic2enu(distlat(2:end),distlon(2:end),disthell(2:end),...
        distlat(1:end-1),distlon(1:end-1),disthell(1:end-1),wgs84Ellipsoid,'degrees');
    dHorzDist = ([0 dx].^2 + [0 dy].^2).^0.5; % first dx,dy,dz
    distall = cumsum(dHorzDist)/1000;
    
    clear dx dy;
    dx = mean(dHorzDist)*1000; % used for diff plot
    clear dHorzDist;
end
%% This section determines if the plot direction needs switching based on 
% linePlotDir and tieLinePlotDir
% WE is assumed to be West antarctica to East antarctica
% SN is assumed to be from south pole to north pole

if autoPlotDirChange && (all(isnan(distlon)) || all(isnan(distlat)))
        autoPlotDirChange = false;
        warning('dicePlotTool:noLatLon','No Longitude or Latitudes available for automatic direciton detection');
end
% determine the direction of the flight West to East and South to North are
% positive directions. Direciton is established using only first 10% of the
% data in the first third of a line
if(autoPlotDirChange)
    tempLonforDir = distlon(round(numel(distlon)*0.25):round(numel(distlon)*0.35));
    
    % if there is a jump in longitude wrap to 360.
    % This assumes we are only crossing one meridian at a time in a line
    if any(abs(diff(tempLonforDir))>60)
        tempLonforDir = wrapTo360(tempLonforDir);
    end
    tempLatforDir = distlat(round(numel(distlat)*0.25):round(numel(distlat)*0.35)) + 90;  % this sets south pot to 0 and north to 180
    tempHeading = azimuth(tempLatforDir(1),tempLonforDir(1),tempLatforDir(end),tempLonforDir(end),wgs84Ellipsoid,'degrees');
    if(tempHeading<30 || tempHeading>330 || (tempHeading>150 && tempHeading<210)) % this is a tie line
        if strcmpi(tieLinePlotDir,'sn')
            if(tempHeading<30 || tempHeading>330)
                reversexaxis = false;
            else
                reversexaxis = true;
            end
        elseif strcmpi(tieLinePlotDir,'ns')
            if(tempHeading<30 || tempHeading>330)
                reversexaxis = true;
            else
                reversexaxis = false;
            end
        else
            warning('sirPlotTool:InvalidTieLineDir','Tie line plot direction %s invalid, use ''SN'' or ''NS'' only\nPlot direciton will not be changed.',tieLinePlotDir);
        end
    elseif (tempHeading>60 && tempHeading<120) || (tempHeading>240 && tempHeading<300)
        if strcmpi(linePlotDir,'we')
            if(tempHeading>60 && tempHeading<120)
                reversexaxis = true;
            else
                reversexaxis = false;
            end
        elseif strcmpi(linePlotDir,'ew')
            if(tempHeading>60 && tempHeading<120)
                reversexaxis = false;
            else
                reversexaxis = true;
            end
        else
            warning('sirPlotTool:InvalidLineDir','Line plot direction %s invalid, use ''SN'' or ''NS'' only\nPlot direciton will not be changed.',tieLinePlotDir);
        end
    else
        warning('sirPlotTool:InvalidHeading','Flight/Line heading %0.2f deg is outside of 30 deg margin for auto plot direction\n Manually set the ''reversexaxis'' variable',tempHeading);
    end
    clearvars tempLatforDir tempLonforDir
end

%% Set plotbydist Flag - this flag is used everywhere
if(strcmpi(plotXaxis,'dist'))
    plotbydist = true;
else
    plotbydist = false;
end

%% Open a txt file for logging figure info
if printFigure && (outputPng || outputPdf)
    if old_fileformat
        txtfilename = [txtoutputpath,fileNameFields{1},fileFieldSep,fileNameFields{2},fileFieldSep,datestr(unix2matlabtime(startplotutc),...
            [outFileDateFormat,fileFieldSep,outFileTimeFormat]),fileFieldSep,fileNameFields{3},fileFieldSep,datatag,'.txt'];
    else
        txtfilename = [txtoutputpath,fileNameFields{1},fileFieldSep,fileNameFields{2},fileFieldSep,datestr(unix2matlabtime(startplotutc),...
            [outFileDateFormat,fileFieldSep,outFileTimeFormat]),fileFieldSep,fileNameFields{5},fileFieldSep,datatag,'.txt'];
    end
    if exist(txtfilename,'file') == 2
        warning('dicePlotTool:txtFileExists','txt file \n %s \n Already Exists!\n new file will be written with current time suffix',txtfilename);
        txtfilename = [txtfilename(1:end-4),datestr(now,'_yyyymmdd_hhMMss'),'.txt'];
    end
    
    txtfid = fopen(txtfilename,'w');
    % cleans the fileID when the function exits.
    cleaner = onCleanup(@()fclose(txtfid));
    fprintf(txtfid,'FigureName, StartUTCDate,StartUTCTIme,StartUnixTime,StopUTCDate,StopUTCTIme,StopUnixTime,StartLatitude,StartLongitude,StartH-El,StopLatitude,StopLongitude,StopH-El\n');
end

%% save a copy of the plotting code in the output txt directory
if printFigure && (outputPdf || outputPng)
    thisScript = [mfilename,'.m'];
    % also save a copy of the plotter in the txt directory
    outscriptfile = [txtoutputpath,thisScript,datestr(now,'_yyyymmdd_hhMMss'),'.txt'];
    copyfile(thisScript,outscriptfile);
    fprintf('dicePlotTool: Copy of plotting script saved as %s\n',outscriptfile);
end

%% set number of x ticks based on x-axis data and plot period
if plotbydist
    numXticks = min(max(min(ceil(plotPeriod/10)+1,10),5),maxXticksNum);   %#ok<NASGU> % number of x-axis ticks
else
    numXticks = min(max(min(ceil(plotPeriod/5)+1,5),5),maxXticksNum);   %#ok<NASGU> % number of x-axis ticks
end

%% Plotting batch computation
if multiplePlots
    if plotbydist %#ok<UNRCH>
        numbatches = ceil(distall(end)/plotPeriod);
        if numbatches > (endplotfile-startplotfile-1)
            plotPeriod = round(filePeriod*2*surveySpeed/1000);
            warning('dicePlotTool:batchTooShort',['plotPeriod should be larger than a file size.\n',...
                'Changing plot period to %d'],plotPeriod);
        end
        batchstartutc(1) = startplotutc;
        if(numbatches == 1)
            batchendutc(1) = endplotutc;
        else
            batchMaxInd = find(distall<= plotPeriod,1,'last');
            batchendutc(1) = distechotimeutc(batchMaxInd);
            for bb = 2:numbatches - 1
                batchstartutc(bb) = distechotimeutc(batchMaxInd+1); %#ok<SAGROW>
                batchMaxInd = find(distall <= bb*plotPeriod,1,'last');
                batchendutc(bb) = distechotimeutc(batchMaxInd); %#ok<SAGROW>
            end
            batchstartutc(numbatches) = distechotimeutc(batchMaxInd+1);
            batchendutc(numbatches) = endplotutc;
        end
    else
        batchstartutc = startplotutc:plotPeriod*60:endplotutc;
        batchendutc = min([batchstartutc+plotPeriod*60-1],endplotutc); %#ok<NBRAK>
        numbatches = numel(batchstartutc);
    end
else
    % In single-plot mode, if images are larger than singleImageLimit, break them up reasonably
    if exist('ylims','var') && surfZero %#ok<UNRCH>
        ylimits = ylims;
    else
        if(sardata)
            load([dataPath,level1Files{processChan(plotSigChan)}(1).name],'twttTx');
            twtt = twttTx - twttTx(1); clear twttTx;
        else
            load([dataPath,level1Files{processChan(plotSigChan)}(1).name],'twtt');
        end
        ylimits = [-(twtt(end) - twtt(TxInd)), -twtt(1) - twtt(TxInd)]*cIce/2;
    end
    
    if ~plotbydist
        figdist = (distechotimeutc - distechotimeutc(1))*100/1000;  %using 100 m/s average
        tickAlignSpacing = tickSpacing*60*100/1000;
    else
        figdist = distall;
        tickAlignSpacing = tickSpacing;
    end
    if (range(ylimits)*range(figdist)/cutterH/cutterV) > singleImageLimit && breakUpImage
        numbatches = ceil((range(ylimits)*range(figdist)/cutterH/cutterV)/singleImageLimit);
        warning('dicePlotTool:imageTooBig','Maximum image size reached, Line will be printed into %d separate images',numbatches);
        maxImageLen = singleImageLimit*cutterH*cutterV/range(ylimits);
        if(minImageLen>=0.75*range(figdist))
            minImageLen = floor(maxImageLen/2);
        end
        lastBatchLen = max(minImageLen,range(figdist)-((numbatches-1)*maxImageLen));
        batchdist(numbatches) = lastBatchLen + mod(range(figdist)-lastBatchLen,(numbatches-1)*max(tickAlignSpacing,1));
        batchdist(1:numbatches-1) = (range(figdist)-batchdist(numbatches))/(numbatches - 1);
        batchdist = cumsum(batchdist);
        if (abs(batchdist(end) - range(figdist)))>0.001
            warning('dicePlotTool:lineBreakupFail','Distaces of individual batch do not add up to total range');
        end
        batchstartutc(1) = startplotutc;
        batchendutc(1) = distechotimeutc(find(figdist<=batchdist(1),1,'last'));
        for nnb = 2:numbatches
            batchstartutc(nnb) = distechotimeutc(find(figdist>batchdist(nnb-1),1,'first'));
            batchendutc(nnb) = distechotimeutc(find(figdist<=batchdist(nnb),1,'last'));
        end
        clearvars maxImageLen lastBatchLen tickAlignSpacing;
    else
        batchstartutc = startplotutc; %#ok<UNRCH>
        batchendutc = endplotutc;
        numbatches = 1;
    end
    numXticks = maxXticksNum;
end

for bb = 1:numbatches
    if bb ==1
        batchmin(bb) = startplotfile; %#ok<SAGROW>
    else
        batchmin(bb) = min(batchmax(bb-1) + 1,endplotfile); %#ok<SAGROW>
    end
    if bb == numbatches
        batchmax(bb) =  endplotfile; %#ok<SAGROW>
    else
        batchmax(bb) =  find(ftimeutc <= batchendutc(bb),1,'last'); %#ok<SAGROW>
    end
    if batchmax(bb) == endplotfile && bb < numbatches
        numbatches = bb;
        break;
    end
end

minfNum=batchmin;
maxfNum=batchmax;

%% Let the plottig begin
numExcessFrames = 0;
figHandle = -ones(numbatches,1);
for batch=1:numbatches
    for fnum = minfNum(batch):maxfNum(batch)
        if sardata
            load([dataPath,level1Files{processChan(plotSigChan)}(fnum).name],...
                'CG','twttTx','Lat','Lon','FlightElev','PlaneHeight','SurfInds','echoTimeUTC');
            radecho = 20*log10(abs(CG)); clear CG;
            twtt = twttTx - twttTx(1); clear twttTx;
            lat = Lat; clear Lat;
            lon = Lon; clear Lon;
            hell = FlightElev; clear FlightElev;
            planeHeght = PlaneHeight; clear PlaneHeight;
            surfIndRad = SurfInds; clear SurfInds;
            echotimeutc = echoTimeUTC; clear echoTimeUTC;
        else
            load([dataPath,level1Files{processChan(plotSigChan)}(fnum).name],...
                'radecho','twtt','echotimeutc','lat','lon','hell','planeHeight','surfIndRad');
            radecho = 20*log10(abs(radecho));
        end
        % Empty space for matching line by line with sirPlotTool
        %
        %
        %
        %
        %
        %
        %
        %
        %
        %
        %
        %
        %
        %
        %
        %
        %
        %
        %
        %
        %
        %
        %
        fprintf('Plotting File %d of %d of batch %d of %d\n',fnum-minfNum(batch)+1,maxfNum(batch)-minfNum(batch)+1,batch,numbatches)
        % Compatibility with older data
        if exist('hell','var') && exist('lat','var') && exist('lon','var')
            gpsexist = true;
        else
            warning('off','MATLAB:load:variableNotFound');
            gpsexist = false;
            %
            if elevcorrection
                warning('dicePlotTool:NoGpsData','No gps data, elevation will not be corrected');
                warning('off','dicePlotTool:NoGpsData')
                elevcorrection = false;
            end
        end
        if exist('surfIndRad','var')
            if(size(surfIndRad,1)>size(surfIndRad,2))
                surfIndRad = surfIndRad';
            end
            surfIndRad(surfIndRad<2) = NaN;
            if ~all(all(isnan(surfIndRad))) || firstGoodSurf>1
                surfexist = true;
            else
                surfexist = false;
                if plotLayers || ~strcmp(surfType,'none')
                    warning('dicePlotTool:NoSurfData','No surface data, surface will not be plotted');
                    warning('off','dicePlotTool:NoSurfData');
                end
            end
        else
            surfexist = false;
            if plotLayers || ~strcmp(surfType,'none')
                warning('dicePlotTool:NoSurfData','No surface data, surface will not be plotted');
                warning('off','dicePlotTool:NoSurfData');
            end
        end
        if exist ('planeHeight','var')
            heightexist = true;
            if iscolumn(planeHeight)
                planeHeight = planeHeight';
            end
        else
            heightexist = false;
        end
        % Empty space for matching line by line with sirPlotTool
        %
        %
        %
        %
        
        if  fnum == minfNum(batch)  % First file of a batch
            % clear last surface and time from previous files- These are
            % plotted with current surfaces to eliminate gaps in the
            % surface plots
            %             lastSurfRange = [];
            %             lastSurfTime = [];
            if batch > 1 && numExcessFrames > 0 % Add leftover echoes from last batch
                radecho = [excessradecho(:,1:numExcessFrames) radecho]; %#ok<AGROW>
                echotimeutc = [excessechotimeutc(1:numExcessFrames) echotimeutc]; %#ok<AGROW>
                if gpsexist
                    lat = [excesslat(1:numExcessFrames) lat]; %#ok<AGROW>
                    lon = [excesslon(1:numExcessFrames) lon]; %#ok<AGROW>
                    hell = [excesshell(1:numExcessFrames) hell]; %#ok<AGROW>
                end
                if surfexist
                    if isempty(excesssurfIndRad)
                        excesssurfIndRad = ones(size(surfIndRad,1),numel(echotimeutc))*NaN;
                    end
                    surfIndRad = [excesssurfIndRad(:,1:numExcessFrames) surfIndRad]; %#ok<AGROW>
                end
                %
                %
                %
                if heightexist
                    planeHeight = [excessplaneHeight(1:numExcessFrames) planeHeight]; %#ok<AGROW>
                end
                if ~isempty(echotimeutc)
                    batchfirstutc = echotimeutc(1);
                else
                    break;
                end
            else %Very first file, find echoes in plot range
                inTimeEchoes = echotimeutc>=batchstartutc(batch);
                radecho = radecho(:,inTimeEchoes);
                echotimeutc = echotimeutc(inTimeEchoes);
                if gpsexist
                    lat = lat(inTimeEchoes);
                    lon = lon(inTimeEchoes);
                    hell = hell(inTimeEchoes);
                end
                if surfexist
                    surfIndRad = surfIndRad(:,inTimeEchoes);
                end
                %
                %
                %
                if heightexist
                    planeHeight = planeHeight(inTimeEchoes);
                end
                if ~isempty(echotimeutc)
                    batchfirstutc = echotimeutc(1);
                else
                    warning('First time not set. Updating batch list')
                    minfNum(batch) = fnum+1;
                    break;
                end
            end
            % If surface not availabel use reference surface
            % specified. If surface is not a number, find the first
            % valid surface in the file- use reference surface if
            % not found - This is computed for every batch
            if surfexist
                refsurfInd = min(surfIndRad(:,1)); % Get surface from first echo
                if isnan(refsurfInd)    % if surface from first echo in NaN
                    refsurfInd = min(surfIndRad(find(~isnan(min(surfIndRad,[],1)),1,'first')),[],1); % find first non-Nan Surface from current file
                    refsurfInd(isnan(refsurfInd)) = [];
                    if isempty(refsurfInd) % if no good surface found in current file,
                        if(firstGoodSurf>1)
                            refsurfInd = firstGoodSurf; % use first good surface in entire flight/line, if available
                            if(surfZero)
                                warning('dicePlotTool:SurfaceInFirstFile','No valid surface found in the first file, using first good surface availabel in the batch');
                            end
                        else % no good surface available on the flight/line use the user specified surface
                            refsurfInd = find(twtt *cAir/2>=refsurf,1,'first');
                            if(surfZero)
                                warning('dicePlotTool:SurfaceInFirstFile','No valid surface found in the batch, using user supplied reference surface');
                            end
                        end
                        % Sir at this point need to convert index to range
                        % dice uses the index
                    end
                end
                % Sir does the index to range conversion here again
                %
            else
                if(surfZero)
                    warning('dicePlotTool:noSurfAvail','Radar surface not available\n User reference surface of %d m will be used',refsurf);
                    warning('off','dicePlotTool:noSurfAvail');
                end
                refsurfInd = find((twtt-(twtt(TxInd))) *cAir/2>=refsurf,1,'first');
            end
            % If reference lat lon elevation is not set, do it here.
            if  gpsexist
                reflat = lat(1);
                reflon = lon(1);
                reftime = echotimeutc(1);
                if ~exist('userRefElev','var')  % if user defined a reference Elevaton use that instead
                    refelev = hell(1);
                else
                    refelev = userRefElev;
                end
            end
        end
        
        if fnum == maxfNum(batch)
            inTimeEchoes = echotimeutc<=batchendutc(batch);
            if batch < numbatches   % last file of a batch, store any excess frames for next batch
                excessradecho = radecho(:,~inTimeEchoes);
                excessechotimeutc = echotimeutc(~inTimeEchoes);
                if gpsexist
                    excesslat = lat(~inTimeEchoes);
                    excesslon = lon(~inTimeEchoes);
                    excesshell = hell(~inTimeEchoes);
                else
                    excesslat = ones(size(excessechotimeutc))*NaN;
                    excesslon = ones(size(excessechotimeutc))*NaN;
                    excesshell = ones(size(excessechotimeutc))*NaN;
                end
                if surfexist
                    excesssurfIndRad = surfIndRad(:,~inTimeEchoes);
                else
                    excesssurfIndRad = [];
                end
                %
                %
                %
                %
                %
                if heightexist
                    excessplaneHeight = planeHeight(~inTimeEchoes);
                else
                    excessplaneHeight = ones(size(excessechotimeutc))*NaN;
                end
                
                numExcessFrames = numel(find(~inTimeEchoes));
            else % very last batch ,
                numExcessFrames = 0;
            end
            
            radecho = radecho(:,inTimeEchoes);
            echotimeutc = echotimeutc(inTimeEchoes);
            if gpsexist
                lat = lat(inTimeEchoes);
                lon = lon(inTimeEchoes);
                hell = hell(inTimeEchoes);
            end
            if surfexist
                surfIndRad = surfIndRad(:,inTimeEchoes);
            end
            %
            %
            %
            if heightexist
                planeHeight = planeHeight(:,inTimeEchoes);
            end
            if isempty(echotimeutc)
                break;
            end
        end
        % accumulate time, hell, lat and lon for the batch if required
        if gpsexist && (surfexist || plotLayers)
            allhell = [allhell hell]; %#ok<AGROW>
            alllat = [alllat lat]; %#ok<AGROW>
            alllon = [alllon lon]; %#ok<AGROW>
        end
        if surfexist || plotLayers
            allechotimeutc = [allechotimeutc echotimeutc];   %#ok<AGROW> % accumulate for layer plotting
        end
        %
        %
        if surfexist
            allSurfIndRad = [allSurfIndRad surfIndRad]; %#ok<AGROW>
        end
        
        %% Elevation Correction - correct for plane motion
        if elevcorrection && gpsexist
            % Perform interpolation about the reference elevation using elevation change since
            for kk=1:length(echotimeutc)
                elevoffset=hell(kk)-refelev;
                indoffset=elevoffset/((twtt(2)-twtt(1))*cAir/2);
                radecho(:,kk)=interp1([1:size(radecho,1)]',radecho(:,kk),[1:size(radecho,1)]'+indoffset,'linear'); %#ok<NBRAK>
            end
        end
        %% Set surface to zero
        if surfZero
            % Depth scale always stays the same relative to the reference surface  for the batch
            DepthScale=-(twtt - twtt(refsurfInd))*cIce/2;
        else
            DepthScale= -(twtt - twtt(TxInd))*cIce/2;
        end
        
        % Determine data within elevation range:
        if exist('ylims','var')
            if(surfZero)
                InRange= find(DepthScale<=ylims(1) & DepthScale>=ylims(2));
            else
                InRange = 1:numel(DepthScale); %#ok<UNRCH>
            end
        else
            InRange = 1:numel(DepthScale);
        end
        
        %% Plot
        if plotbydist
            xaxisData = distall(distechotimeutc>=echotimeutc(1) & distechotimeutc<=echotimeutc(end));
            if fnum == minfNum(batch)
                batchfirstxData = xaxisData(1);
            end
            xaxisData = xaxisData - batchfirstxData;
        else
            xaxisData = unix2matlabtime(echotimeutc);
        end
        
        if detrendy
            radecho = bsxfun(@minus, radecho, nanmean(radecho)); %#ok<UNRCH>
        end
        if x_smooth_win >1 || y_smooth_win >1
            if any(any(isnan(radecho)|isinf(radecho)))
                radecho(isnan(radecho)|isinf(radecho)) = mean(mean(radecho(~ isnan(radecho) & ~isinf(radecho))));
            end
            if y_smooth_win>1
                yb = (1/round(y_smooth_win))*ones(1,round(y_smooth_win));
                ya = 1;
                radecho = filtfilt(yb,ya,radecho);
            end
            if x_smooth_win >1
                xb = (1/round(x_smooth_win))*ones(1,round(x_smooth_win));
                xa = 1;
                radecho = filtfilt(xb,xa,radecho')';
            end
%             radecho = intuitive_lowpass2(radecho,x_smooth_win,y_smooth_win);
        end
        % Take a gradient image to make the layers pop-out
        if diffPlot
            dy = ((twtt(2)-twtt(1))*cIce/2);
            [TempX,TempY] = gradient(radecho,dx,dy);
            radecho = sign(TempY).*sqrt((TempX.^2+TempY.^2));
            clear TempX TempY;
            climdiff = mean(mean(radecho))+[clims(1)*std(reshape(radecho,[],1)),clims(2)*std(reshape(radecho,[],1))];
        end
        
        if ~ishghandle(figHandle(batch)) % if new batch creat a new figure
            figHandle(batch) = figure(batch);
        end
        if invertcaxis==1
            imagesc(xaxisData(1:cutterH:end),DepthScale(InRange(1:cutterV:end)),-radecho(InRange(1:cutterV:end),1:cutterH:end))
            if exist('clims','var')
                if diffPlot
                caxis([-climdiff(2:-1:1)]); %#ok<NBRAK>
                else
                caxis([-clims(2:-1:1)]); %#ok<NBRAK>
                end
            end
        else
            imagesc(xaxisData(1:cutterH:end),DepthScale(InRange(1:cutterV:end)),radecho(InRange(1:cutterV:end),1:cutterH:end));
            if exist('clims','var')
                if diffPlot
                    caxis(climdiff);
                else
                caxis(clims);
                end
            end
        end
        hold all
        clearvars('radecho','lat','lon','hell','planeHeight','surfIndRad')
        eval(['colormap(',cmap,')'])
        
        % if tickSpacig is 0 or larger than plot Period, a fixed number of
        % ticks is used instead
        if plotbydist
            thisPlotPeriod = xaxisData(end);
        else
            thisPlotPeriod = (echotimeutc(end) - batchfirstutc)/60;
        end
        
        if tickSpacing>0 && (tickSpacing <= thisPlotPeriod)
            if plotbydist
                xTicks = [0:tickSpacing:xaxisData(end)]; %#ok<NBRAK>
                if (xaxisData(end) - xTicks(end)) > 0.85*tickSpacing
                    xTicks = [xTicks xaxisData(end)]; %#ok<AGROW>
                end
            else
                 xTicks = [batchfirstutc:tickSpacing*60:echotimeutc(end)];
                 if (echotimeutc(end) - xTicks(end)) > 0.85*tickSpacing*60
                    xTicks = [xTicks echotimeutc(end)]; %#ok<AGROW>
                end
                % if plotting against time, convert to matlab time
                xTicks = unix2matlabtime(xTicks);
            end
            % update numXticks
            numXticks = numel(xTicks);
        else
            if plotbydist
                xTicks = linspace(0,xaxisData(end),numXticks);
            else
                xTicks = linspace(unix2matlabtime(batchfirstutc),unix2matlabtime(echotimeutc(end)),numXticks);
            end
        end
        
        if plotbydist
            xlim([0,xaxisData(end)]);
            set(gca,'xtick',xTicks);
            set(gca,'xticklabel',cellstr(num2str(xTicks','%.1f'))');
            xlabel({'Distance Flown, km'},'fontweight','bold');
        else
            xlim([unix2matlabtime(batchfirstutc),unix2matlabtime(echotimeutc(end))]);
            set(gca,'xtick',xTicks);
            datetick2('x','HH:MM:SS','keeplimits','keepticks');
            xlabel('Time of Day (UTC)','fontweight','bold');
        end
        
        ylabel('Depth in Ice, m [velocity in ice = 1.68e8 m/s]','fontweight','bold');
        
        if exist('ylims','var')
            if(surfZero)
                ylim([ylims(2),ylims(1)]);
                set(gca,'ydir','normal')
            else % create a ylims variable for scaling
                ylim([min(DepthScale(InRange)) max(DepthScale(InRange))]); %#ok<UNRCH>
                set(gca,'ydir','normal')
                ylims = [min(DepthScale(InRange)) max(DepthScale(InRange))];
                warning('dicePlotTool:yLimsIgnored',['Surface is not set to zero, ylims will be ignored\n',...
                    'set ''surfZero'' to ''true'' to enforce ylims']);
                warning('off','dicePlotTool:yLimsIgnored');
            end
        else
            ylims = [min(DepthScale(InRange)) max(DepthScale(InRange))];
            set(gca,'ydir','normal')
        end
        
        if reversexaxis==1
            set(gca, 'xdir','reverse')
        end
        title(plotSig)
        set(gca,'TickDir','out')
        % Draw figure:
        if contUpdateFig
            drawnow; %#ok<UNRCH>
        end
    end
    %% Plot Surfaces and Layers for the batch
    if surfexist && ~strcmpi(surfType,'none') && ~isempty(fnum)
        % remove any surface with Nan Values
        if any(any(isnan(allSurfIndRad)))
            warning('dicePlotTool:nanSurface','Surface(s) not available for some echoes')
            warning('off','dicePlotTool:nanSurface');
            allSurfIndRad(isnan(allSurfIndRad)) = 1; %#ok<SAGROW>
        end
        if surfZero
            surfRange = -(twtt(allSurfIndRad)-twtt(refsurfInd))*cAir/2;
        else
            surfRange = -(twtt(allSurfIndRad)-twtt(TxInd))*cAir/2;
        end
        
        surfRange(allSurfIndRad == 1) = NaN;
        if iscolumn(surfRange)
            surfRange = surfRange';
        end
        if elevcorrection
            surfRange = surfRange + (repmat(allhell,size(surfRange,1),1)-refelev);
        end
        if plotbydist
            if ~isempty(allechotimeutc)
                xaxisDataSurf = distall(distechotimeutc>=allechotimeutc(1) & distechotimeutc<=allechotimeutc(end));
                xaxisDataSurf = xaxisDataSurf - batchfirstxData;
            else
                xaxisDataSurf = [];
            end
        else
            xaxisDataSurf =  unix2matlabtime(allechotimeutc);
        end
        switch surfType
            case 'first'
                if size(surfRange,1) > 1
%                     if surfZero
%                         plot(xaxisDataSurf,min(surfRange,[],1)*cIce/cAir,surfLineColor{1});
%                     else
                        plot(xaxisDataSurf,max(surfRange,[],1)*cIce/cAir,surfLineColor{1});
%                     end
                    diceleg = legend(surfLegend{1});
                elseif size(surfRange,1) ==1
                    warning('dicePlotTool:singleSurfOnly','Only one surface is available (max power)')
                    warning('off','dicePlotTool:singleSurfOnly')
                    plot(xaxisDataSurf,surfRange(1,:)*cIce/cAir,surfLineColor{2});
                    diceleg = legend(surfLegend{2});
                else
                    warning('dicePlotTool:NoSurfAvail','No surface is available for one or more batches');
                    warning('off','dicePlotTool:NoSurfAvail');
                end
            case 'max'
                if size(surfRange,1) >=1
                    plot(xaxisDataSurf,surfRange(1,:)*cIce/cAir,surfLineColor{2});
                    diceleg = legend(surfLegend{2});
                else
                    warning('dicePlotTool:NoSurfAvail','No surface is available for one or more batches');
                    warning('off','dicePlotTool:NoSurfAvail');
                end
            case 'both'
                if size(surfRange,1) > 1
%                     if surfZero
                        plot(xaxisDataSurf,max(surfRange,[],1)*cIce/cAir,surfLineColor{1},...
                            xaxisDataSurf,surfRange(1,:)*cIce/cAir,surfLineColor{2});
%                     else
%                         plot(xaxisDataSurf,min(surfRange,[],1)*cIce/cAir,surfLineColor{1},...
%                             xaxisDataSurf,surfRange(1,:)*cIce/cAir,surfLineColor{2});
%                     end
                    diceleg = legend(surfLegend{1},surfLegend{2});
                elseif size(surfRange,1) ==1
                    warning('dicePlotTool:singleSurfOnly','Only one surface is available (max power)')
                    warning('off','dicePlotTool:singleSurfOnly')
                    plot(xaxisDataSurf,surfRange(1,:)*cIce/cAir,surfLineColor{2});
                    diceleg = legend(surfLegend{2});
                else
                    warning('dicePlotTool:NoSurfAvail','No surface is available for one or more batches');
                    warning('off','dicePlotTool:NoSurfAvail');
                end
            otherwise
                % do nothing
        end
        % clear the batch surface variable for next batch
    end

    % imported layers are plotted on a separate axis
    if plotLayers && ~layersLoaded && ~isempty(fnum)
        for pp = 1:numLayers
            if exist(layerDataPath{pp},'file')~=2
                warning('dicePlotTool:noLayerFile','Layer %d file %s does not exist',pp,layerDataPath{pp})
                [file,path] = uigetfile(pwd,num2str(pp,'Select a file for Layer %d'),'*.mat');
                layerDataPath{pp} = fullfile(path,file); %#ok<SAGROW>
                if isempty(path) || numel(path)<2
                    warning('dicePlotTool:lineNotPlotted','Invalid line file.Line %d will not be plotted\n set plotLayers = 0 to disable dialog box',pp);
                    layerData{pp} = NaN;
                    layerTime{pp} = NaN;
                    continue;
                end
            end
            % if there is a varaiable name collision lets take care of it here
            if exist(layerVar{pp},'var')
                tempExistingLayerVar = eval(layerVar{pp});
            end
            if exist(layerTimeVar{pp},'var')
                tempExistingLayerTimeVar = eval(layerTimeVar{pp});
            end
            
            load(layerDataPath{pp},layerTimeVar{pp},layerVar{pp});
            if ~exist(layerVar{pp},'var') || ~exist(layerTimeVar{pp},'var')
                warning('dicePlotTool:noLayerVars','Invalid Layer %d variables',pp)
                layerData{pp} = NaN;
                layerTime{pp} = NaN;
                if exist('tempExistingLayerVar','var')
                    eval([layerVar{pp} '= tempExistingLayerVar;']);
                    clear  tempExistingLayerVar;
                end
                if exist('tempExistingLayerTimeVar','var')
                    eval([layerVar{pp} '= tempExistingLayerTimeVar;']);
                    clear tempExistingLayerTimeVar;
                end
                continue;
            end
            layerTime{pp} = eval(layerTimeVar{pp});
            % For each case, layerData should converted to distane (scaled
            % by cIce/cAir) to the surface (positive above, negative below)
            % Some of this needs to tested and re-evaluated
            switch layerDataType{pp}    % % 'Index','RangeDist','DepthDist','RangeTime','DepthTime','Elev'
                case 'Index'    % index of radar echogram
                    layerData{pp} = (twtt(eval(layerVar{pp}))-twtt(refsurfInd))*cIce/2;
                case 'RangeDistIce'  % Distance from a/c scale by Ice/Air factor
                    layerData{pp} = eval(layerVar{pp}) - twtt(refsurfInd)*cIce/2;
                case 'RangeDistTrue'     % True Distace from a/c
                    layerData{pp} = (eval(layerVar{pp}) - twtt(refsurfInd)*cAir/2)*cIce/cAir;
                case 'DepthDist'    % Depth below the surface
                    if ~surfexist
                        warning('sirPlotTool:invalidLayerType','Surface Data not available, layer type %s is not usable',layerDataType{pp})
                        layerData{pp} = NaN;
                    else
                        layerData{pp} = -eval(layerVar{pp});
                    end
                case 'RangeTime'    % two way travel time from the a/c
                    layerData{pp} = (eval(layerVar{pp}) - twtt(refsurfInd))*cIce/2;
                case 'DepthTime'    % two way travel time below surface
                    layerData{pp} = -eval(layerVar{pp})*cIce/2;
                case 'Elev'
                    % get the surface range, then surface elevation
                    if any(any(isnan(allSurfIndRad)))
                        warning('dicePlotTool:nanSurface','Surface(s) not available for some echoes')
                        warning('off','dicePlotTool:nanSurface');
                        allSurfIndRad(isnan(allSurfIndRad)) = 1; %#ok<SAGROW>
                    end
                    tempsurfRange = (twtt(allSurfIndRad) - twtt(TxInd))*cAir/2;
                    tempsurfRange(allSurfIndRad == 1) = NaN;
                    if size(tempsurfRange,1)>size(tempsurfRange,2)
                        tempsurfRange = tempsurfRange';
                    end
                    tempsurfRange = min(tempsurfRange,[],1); % surface with max power
                    tempsurfElev =  allhell - tempsurfRange;
                    layerData{pp} = eval(layerVar{pp}) - interp1(allechotimeutc,tempsurfElev,layerTime{pp});
                    layerData{pp}(layerData{pp}>0) = layerData{pp}(layerData{pp}>0)*cIce/cAir;
            end
            if plotbydist
                xaxisDataLayer{pp} = interp1(distechotimeutc,distall,layerTime{pp});
            else
                xaxisDataLayer{pp} = unix2matlabtime(layerTime{pp});
            end
            if iscolumn(layerData{pp})
                layerData{pp} = layerData{pp}';
            end
            if iscolumn(layerTime{pp})
                layerTime{pp} = layerTime{pp}';
            end
            if elevcorrection && gpsexist
                layerData{pp} =  -(layerData{pp} - (interp1(allechotimeutc,allhell,layerTime{pp})- refelev));
            end
            %
            %
            %
            if exist('tempExistingLayerVar','var')
                eval([layerVar{pp} ' = tempExistingLayerVar;']);
                clear  tempExistingLayerVar;
            else
                clear(layerVar{pp});
            end
            if exist('tempExistingLayerTimeVar','var')
                eval([layerTimeVar{pp} ' = tempExistingLayerTimeVar;']);
                clear tempExistingLayerTimeVar;
            else
                clear(layerVar{pp});
            end
        end
        layersLoaded = true;
    end
    
    allSurfIndRad = [];
        
    % get ready for plotting layers if needed
    hold off;
    dicefig(batch) = gca; %#ok<SAGROW>
    dicefigSize = get(dicefig(batch),'Position');
    
    if plotLayers && layersLoaded && ~isempty(fnum)
%         layersAx(batch) = axes('Position',dicefigSize); %#ok<SAGROW>
        for pp = 1:numLayers
            if ~isempty(allechotimeutc)
                inbatchInd = layerTime{pp}>=allechotimeutc(1) & layerTime{pp} <= allechotimeutc(end);
            else
                continue;
            end
            if ~all(~inbatchInd)
                hold all;
                plot(dicefig(batch),xaxisDataLayer{pp}(inbatchInd),layerData{pp}(inbatchInd),layerLineSpec{pp});
                lastLegend = get(legend(gca),'String');
                lastLegend{numel(lastLegend)+1} = layerLegend{pp};
                diceleg = legend(lastLegend);
            end
        end
         set(layersAx(batch),'color','none','box','off',...
            'xtick',[],'ytick',[],'xlim',get(dicefig(batch),'xlim'),'ylim',get(dicefig(batch),'ylim'));
        set(layersAx(batch),'Tag','layerAxes','fontweight','bold');
        set(figure(batch),'CurrentAxes',layersAx(batch));
        axis off;
    end
    %
    %
    %
    %
    %
    %
    %
    %
    %
    %
    splitsigtype = strsplit(plotSig{1},'us');
    if ~isempty(fnum)
        if(~multiplePlots)
            if(isunix)
                titletext= {[Campaign,' ',FlightNo,' ',splitsigtype{2},' ',splitsigtype{1},'\mus ',...
                    datestr(unix2matlabtime(batchfirstutc),'yyyy-mm-dd (UTC)')];'';''};
            else
                titletext= {[Campaign,' ',FlightNo,' ',splitsigtype{2},' ',splitsigtype{1},'\mus ',...
                    datestr(unix2matlabtime(batchfirstutc),'yyyy-mm-dd (UTC)')];'';'';''};
            end
        else
            titletext= {[Campaign,' ',FlightNo,' ',splitsigtype{2},' ',splitsigtype{1},'\mus'];...
                datestr(unix2matlabtime(batchfirstutc),'yyyy-mm-dd (UTC)');'';''};
        end
        if exist('diceleg','var')
            set(diceleg,'Location',legendPos)
        end
        title(titletext,'fontweight','bold');
        
        if gpsexist
            distbatch = interp1(distechotimeutc,distall,allechotimeutc);
            distbatch = distbatch - distbatch(1);
        else
            distbatch = [];
        end
        if showScale
            if plotbydist %#ok<UNRCH>
                rangeXaxis = range(distbatch);
                if (tickSpacing >0)
                    xScale = tickSpacing;
                else
                    xScale = rangeXaxis/(numXticks - 1);
                end
                scaleAx(batch) = axes('Position',[dicefigSize(3)*(numXticks-3)/(numXticks-1)+dicefigSize(1) 0.15 dicefigSize(3)/(numXticks - 1) 1e-12]);
                set(scaleAx(batch),'units','normalized','color','none','xlim',[0 xScale],...
                    'xtick',[0 xScale/2 xScale],'xtickLabel',{'0','',num2str(xScale,'%.1f km')})
                set(scaleAx(batch),'Tag','dicefigXscale','fontweight','bold');
            else
                warning('dicePlotTool:xScaleVariableDist','X-Scale distance is variable, scale will not be shwon')
                warning('off','dicePlotTool:xScaleVariableDist');
            end
        end
        set(figure(batch),'CurrentAxes',dicefig(batch));
        if ismember('dist',displayAxes) || ismember('pos',displayAxes)
            [timeTicks,lonTicks,latTicks,distTicks] = diceTicks(alllat,alllon,allechotimeutc,distbatch,displayAxes,dicefig(batch),~multiplePlots);
        else
            timeTicks = [];lonTicks = [];latTicks = [];distTicks= [];
        end
        set(dicefig(batch),'fontweight','bold');
        if isunix && gpsexist
            if range(distbatch)>350
                set(dicefig(batch),'fontweight','bold','fontsize',5);
            else
                set(dicefig(batch),'fontweight','bold','fontsize',6);
            end
        end
        % pick first and last elements for figure info text file
        if gpsexist && (plotLayers || surfexist || plotbydist)
            batchhell = allhell([1,end]);
            batchechotime = allechotimeutc([1,end]);
            batchlat = alllat([1,end]);
            batchlon = alllon([1,end]);
        end
        if(batch<numbatches) % keep these in workspace for last batch or single plot for debugging
            allhell = [];
            alllat = [];
            alllon = [];
            allechotimeutc = [];
            %
        end
        %% Print the darn thing
        if printFigure && (outputPng || outputPdf)
            if old_fileformat
                if outputPng
                    % if multiplots is not used, scale the pagesize by max pagesize
                    if ~multiplePlots && maxLineLength>0 && maxLineDepth>0 && ~isempty(distbatch)
                        pngpagesize = maxPageSize./[maxLineLength maxLineDepth].*[range(distbatch) range(ylim)];
                        sizetag = [fileFieldSep num2str(pngpagesize(1)*100,'%.0fby') num2str(pngpagesize(2)*100,'%0.0f')];
                    else
                        sizetag = [];
                    end
                    pngfigname = [pngoutputpath,fileNameFields{1},fileFieldSep,fileNameFields{2},fileFieldSep,datestr(unix2matlabtime(batchstartutc(batch)),...
                        [outFileDateFormat,fileFieldSep,outFileTimeFormat]),fileFieldSep,fileNameFields{3},fileFieldSep,datatag,sizetag,'.png'];
                end
                if outputPdf
                    pdffigname = [pdfoutputpath,fileNameFields{1},fileFieldSep,fileNameFields{2},fileFieldSep,datestr(unix2matlabtime(batchstartutc(batch)),...
                        [outFileDateFormat,fileFieldSep,outFileTimeFormat]),fileFieldSep,fileNameFields{3},fileFieldSep,datatag,'.pdf']; %#ok<UNRCH>
                end
            else
                if outputPng
                    % if multiplots is not used, scale the pagesize by max pagesize
                    if ~multiplePlots && maxLineLength>0 && maxLineDepth>0 && ~isempty(distbatch)
                        pngpagesize = maxPageSize./[maxLineLength maxLineDepth].*[range(distbatch) range(ylim)];
                        sizetag = [fileFieldSep num2str(pngpagesize(1)*100,'%.0fby') num2str(pngpagesize(2)*100,'%0.0f')];
                    else
                        sizetag = [];
                    end
                    pngfigname = [pngoutputpath,fileNameFields{1},fileFieldSep,fileNameFields{2},fileFieldSep,datestr(unix2matlabtime(batchstartutc(batch)),...
                        [outFileDateFormat,fileFieldSep,outFileTimeFormat]),fileFieldSep,fileNameFields{5},fileFieldSep,datatag,sizetag,'.png'];
                end
                if outputPdf
                    pdffigname = [pdfoutputpath,fileNameFields{1},fileFieldSep,fileNameFields{2},fileFieldSep,datestr(unix2matlabtime(batchstartutc(batch)),...
                        [outFileDateFormat,fileFieldSep,outFileTimeFormat]),fileFieldSep,fileNameFields{5},fileFieldSep,datatag,'.pdf']; %#ok<UNRCH>
                end
            end
            % Check if figure exists - print will not overwrite existing figure
            if(outputPng)
                if exist(pngfigname,'file') == 2
                    warning('dicePlotTool:pngFileExists','png figure \n %s \n Already Exists!\n new file will be written with current time suffix',pngfigname);
                    pngfigname = [pngfigname(1:end-4),datestr(now,'_yyyymmdd_hhMMss'),'.png'];
                end
            end
            if(outputPdf)
                if exist(pdffigname,'file') == 2 %#ok<UNRCH>
                    warning('dicePlotTool:pdfFileExists','png figure \n %s \n Already Exists!\n new file will be written with current time suffix',pdffigname);
                    pdffigname = [pdffigname(1:end-4),datestr(now,'_yyyymmdd_hhMMss'),'.pdf'];
                end
            end
            % Save figure:
            fprintf('Saving figure ..\n')
            if outputPdf
                
                set(figHandle(batch),'PaperSize',pdfpagesize) %#ok<UNRCH>
                set(figHandle(batch),'PaperPosition',[0,0,pdfpagesize])
                print(figHandle(batch),'-dpdf',pdffigname,['-r',num2str(pdfresolution)],['-',lower(pdfrenderer)])
                fprintf('Saved as %s\n',pdffigname);
            end
            if outputPng
                set(figHandle(batch),'PaperSize',pngpagesize)
                set(figHandle(batch),'PaperPosition',[0,0,pngpagesize])
                print(figHandle(batch),'-dpng',pngfigname,['-r',num2str(pngresolution)],['-',lower(pngrenderer)]);
                fprintf('Saved as %s\n',pngfigname);
            end
            if exist('pngfigname','var')
                [~,figname,~] = fileparts(pngfigname);
            else
                [~,figname,~] = fileparts(pdffigname);
            end
            % save tick information to a text file
            if(~multiplePlots)
                ticksfilename = [txtoutputpath,filesep,figname,fileFieldSep,'Ticks.txt'];
                tickfid = fopen(ticksfilename,'w');
                % cleans the fileID when the function exits.
                tickFilecleaner = onCleanup(@()fclose(tickfid));
                fprintf(tickfid,'Time,Distance,Longitude,Latitude\n(UTC),(km),(deg),(deg)\n');
                for ttk = 1:numel(timeTicks)
                    fprintf(tickfid,'%s,%0.2f,%0.3f %c,%0.3f %c\n',...
                        datestr(unix2matlabtime(timeTicks(ttk)),'HH:MM:SS'),distTicks(ttk),...
                        abs(lonTicks(ttk)),lonEW(lonTicks(ttk)),abs(latTicks(ttk)),latNS(latTicks(ttk)));
                end
                clear tickFilecleaner;
            end
            try
                figInfo(batch,:) = {figname,datestr(unix2matlabtime(batchechotime(1)),'mm/dd/yyyy,HH:MM:SS'),batchechotime(1),...
                    datestr(unix2matlabtime(batchechotime(2)),'mm/dd/yyyy,HH:MM:SS'),batchechotime(2),...
                    batchlat(1),batchlon(1),batchhell(1),batchlat(2),batchlon(2),batchhell(2)}; %#ok<SAGROW>
            catch
                warning('dicePlotTool:noPosInfo','Position info is not Available');
                warning('off','dicePlotTool:noPosInfo');
                figInfo(batch,:) = {NaN,NaN,NaN,NaN,NaN,NaN,NaN,NaN,NaN}; %#ok<SAGROW>
            end
            fprintf(txtfid,'%s,%s,%.0f,%s,%.0f,%0.6f,%0.6f,%0.3f,%0.6f,%0.6f,%0.3f\n',figInfo{batch,:});
        end
        if closeFigures
            close(figure(batch)); %#ok<UNRCH>
        elseif ~multiplePlots && batch<numbatches
            warning('dicePlotTool:figuresClosed','All figures except the last one will be closed to avoid memory issued during printing');
            warning('off','dicePlotTool:figuresClosed')
            close(figure(batch));
        end
    end
    
end
clearvars cleaner;
warning('on','all')
toc
