% Surface Elevation Fixer
% This function replaces the erroneous Surf_Elev varaible in decimated and
% bedpicked data

% Note: The Surface Elevation calcuation during decimation (erroneously)
% didn't take the transmit time into account. This fix goes through each
% file and recomputes and replaces the existing Surf_Elev variable. This
% allows the code to be run on the same file more than once without
% causeing further issues.

function [fixed] = fixSurfElev(inputfile)
fileFieldSep = '_';

if nargin<1
    if usejava('desktop')
        [file,path] = uigetfile('*.mat','Select a dice level2a or Picks file to fix');
        inputfile = fullfile(path,file);
    else
        error('fixSurfElev:InvalidPath','Requires full level2a or Picks file as argument')
    end
end
TxInd = 318;
cAir = 299792458;
fixed = false;
[~,filenameonly,~] = fileparts(inputfile);
fileNameParts = strsplit(filenameonly,fileFieldSep);
if ~any([any(ismember(fileNameParts,'level2a')),any(ismember(fileNameParts,'Picks'))])
    return;
end
if any(ismember(fileNameParts,'Picks'))
    load(inputfile,'Surf_Elev','SurfTime','Time','FlightElev');
    try
        if Time(5)<0
            fprintf('fixSurfElev: %s surface does not require fixing\n',inputfile);
            fixed = true;
        else
            SurfTime = SurfTime - Time(TxInd);
            Time = Time - Time(TxInd);
            Surf_Elev = FlightElev - SurfTime*cAir/2;
            bakFile = [inputfile,'fixSurfElev_bak'];
            copyfile(inputfile,bakFile);
            save(inputfile,'Surf_Elev','SurfTime','Time','-append');
            delete(bakFile);
            fixed = true;
            fprintf('fixSurfElev: %s Surface Elevation fixed\n',inputfile);
        end
    catch errorid
        disp('Caught an Error!!')
        disp(errorid.identifier)
        disp(errorid.message)
        for jj = 1:numel(errorid.stack)
            fprintf('In Function %s, Line %d\n',errorid.stack(jj).name,errorid.stack(jj).line)
        end
        warning('on','all');
        return;
    end
elseif any(ismember(fileNameParts,'level2a'))
    load(inputfile,'Surf_Elev','Surface','Time','Elevation','layerInd');
    try
        if Time(5) < 0
            fprintf('fixSurfElev: %s surface does not require fixing\n',inputfile);
            fixed = true;
        else
            Time = Time - Time(TxInd);
            Surface = Time(min(layerInd,[],1));
            if iscolumn(Surface)
                Surface = Surface';
            end
            Surface(Surface> Time(end-2)) = NaN;
            Surf_Elev = Elevation - Surface*cAir/2;
            bakFile = [inputfile,'fixSurfElev_bak'];
            copyfile(inputfile,bakFile);
            save(inputfile,'Surf_Elev','Surface','Time','-append');
            delete(bakFile);
            fixed = true;
            fprintf('fixSurfElev: %s Surface Elevation fixed\n',inputfile);
        end
    catch errorid
        disp('Caught an Error!!')
        disp(errorid.identifier)
        disp(errorid.message)
        for jj = 1:numel(errorid.stack)
            fprintf('In Function %s, Line %d\n',errorid.stack(jj).name,errorid.stack(jj).line)
        end
        warning('on','all');
        return;
    end
end
end