% This script makes a reference signal from radar return from water
%Run ampmod on selected file before this. choose startind and stopind to
%include desired signal
%03/01/2015 - Changed variable name to match 
close all
run('ampmod')
saveRefs = 1;   % save the signals to a file?
refAmp = 1;     % Set amplitude of reference signal
refChan = 1;    % Select channel to create ref. sig from
filtType = 'lpf'; %  'bpf' or 'lpf'

% currently using transmit signal - change this range to use Rx from
% calibration flight

startind1us = 320;
stopind1us = 469;
startind10us = 336;
stopind10us = 800;  %1835;


if strcmp(filtType, 'bpf')
    [z,p,k]= butter(8,[7e6 69e6]/75e6,'bandpass');
else
    [z,p,k] = butter(8,(fc+bw/2*1.1)/(fs/2),'low');
end
dicelpf = zp2sos(z,p,k);

if(refChan == 1)
signal1us = filtfilt(dicelpf,1,sig1usch1);
signal10us = filtfilt(dicelpf,1,sig10usch1);
else
signal1us = filtfilt(dicelpf,1,sig1usch2);
signal10us = filtfilt(dicelpf,1,sig10usch2);
end

signal1us([1:startind1us-1,stopind1us+1:end]) = 0;
signal10us([1:startind10us-1,stopind10us+1:end]) = 0;

sig1usref = signal1us(startind1us:stopind1us);
sig10usref = signal10us(startind10us:stopind10us);

sigp1ref = refAmp*sig10usref./max(abs(sig10usref));
sigp2ref = refAmp*sig1usref./max(abs(sig1usref));

t10us = 0:1/fs:(10e-6-1/fs);
t1us = 0:1/fs:(1e-6-1/fs);

figure
ax(1) = subplot(2,1,1);
plot(sigp1ref)
grid
title('10us ref')
xlabel('Amplitude')
ylabel('Samples')
ax(2) = subplot(2,1,2);
plot(sigp2ref)
grid
title('1us ref')
xlabel('Amplitude')
ylabel('Samples')

L = length(signal1us);
NFFT = 2^nextpow2(L);   % Next power of 2 from length of y
fftsig = fft(double(sigp2ref),NFFT);
F1usSig = 2*abs(fftsig)/L;
fif = fs/2*linspace(0,1,NFFT/2+1)';
figure
plot(fif/1e6,10*log10(F1usSig(1:NFFT/2+1)))
title({'Amplitude Spectrum of IF 1us Sig';'Ch2'})
xlabel('Frequency (MHz)')
ylabel('|F_Tx_IF| (dB)')
grid on
% take inverse fft to produce time domain reference signal and scale it to
% make unit amplitude (max)
% sigp2ref = real(ifft(fftsig,150));
% sigp1ref = real(ifft(fftsig,1500));

L = length(signal10us);
NFFT = 2^nextpow2(L);   % Next power of 2 from length of y
fftsig = fft(double(sigp1ref),NFFT);
F10usSig = 2*abs(fftsig)/L;
fif = fs/2*linspace(0,1,NFFT/2+1)';
figure
plot(fif/1e6,10*log10(F10usSig(1:NFFT/2+1)))
title({'Amplitude Spectrum of IF 10us Sig';'Ch2'})
xlabel('Frequency (MHz)')
ylabel('|F_Tx_IF| (dB)')
grid on

if(saveRefs)
save('RefSignalsCalibrated','sigp1ref','sigp2ref');
end
