% This script allows re-printing of a dice figure if
% 1. if the plots needs different settings
% 2. plotter crashes during saving due to high dpi
% This ONLY works if multiplotPlots option is set to false and if the if workspace is still intact

% To use - type printPng on the matlab command line
% Following changes can be made before saving the new figure:
% 1. Change Title, ticks and annotations and text properties
% 2. Change Colorspec including the color limits
% 3. Change plot direction
% 4. Change plto Resolution
% 5. Change plot image size scaling

%% Change plot Resolution 
% pngresolution = 600;       % uncomment to use
%% Change plot page scaling
% maxPageSize = [120 40];   % uncomment to use
%% Change colorspec and or limits
% caxis([-40 60]); - uncomment to use, call caxis to see the current limits
% and adjust
%% Change plot x direction
% set(dicefig(batch), 'xdir','reverse'); -- uncomment to use, 'reverse' or
% 'normal'
%% Change Figure Title - uncomment to use
% titletext= {[Campaign,' ',FlightNo,' ',splitsigtype{2},' ',splitsigtype{1},'\mus ',...
%         datestr(unix2matlabtime(batchfirstutc),'yyyy-mm-dd (UTC)')];'';''};
%% Change Font size for all texts on the figure
% set(dicefig(batch),'fontsize',6); % uncomment to use
%% Change Figure x-ticks - need to play with diceTicks.m to change text parameters.
% this section will only call diceTicks.m to re-apply the ticks
% b = findobj(gca,'Type','text','Tag','textXlabels');
% delete(b)
% [timeTicks,lonTicks,latTicks,distTicks] = diceTicks(alllat,alllon,allechotimeutc,distbatch,displayAxes,dicefig(batch),~multiplePlots);
%% Re-print the image (automatically uses different name if necessary)
if printFigure && (outputPng || outputPdf)
    if old_fileformat
        if outputPng
            % if multiplots is not used, scale the pagesize by max pagesize
            if ~multiplePlots && maxLineLength>0 && maxLineDepth>0 && ~isempty(distbatch)
                pngpagesize = maxPageSize./[maxLineLength maxLineDepth].*[range(distbatch) range(ylim)];
                %                     datatag = [datatag fileFieldSep num2str(pngpagesize(1)*100,'%.0fby') num2str(pngpagesize(2)*100,'%0.0f')]; %#ok<AGROW>
            end
            pngfigname = [pngoutputpath,fileNameFields{1},fileFieldSep,fileNameFields{2},fileFieldSep,datestr(unix2matlabtime(batchstartutc(batch)),...
                [outFileDateFormat,fileFieldSep,outFileTimeFormat]),fileFieldSep,fileNameFields{3},fileFieldSep,datatag,'.png'];
        end
        if outputPdf
            pdffigname = [pdfoutputpath,fileNameFields{1},fileFieldSep,fileNameFields{2},fileFieldSep,datestr(unix2matlabtime(batchstartutc(batch)),...
                [outFileDateFormat,fileFieldSep,outFileTimeFormat]),fileFieldSep,fileNameFields{3},fileFieldSep,datatag,'.pdf'];
        end
    else
        if outputPng
            if ~multiplePlots && maxLineLength>0 && maxLineDepth>0 && ~isempty(distbatch)
                pngpagesize = maxPageSize./[maxLineLength maxLineDepth].*[range(distbatch) range(ylim)];
                %                     datatag = [datatag fileFieldSep num2str(pngpagesize(1)*100,'%.0fby') num2str(pngpagesize(2)*100,'%0.0f')]; %#ok<AGROW>
            end
            pngfigname = [pngoutputpath,fileNameFields{1},fileFieldSep,fileNameFields{2},fileFieldSep,datestr(unix2matlabtime(batchstartutc(batch)),...
                [outFileDateFormat,fileFieldSep,outFileTimeFormat]),fileFieldSep,fileNameFields{5},fileFieldSep,datatag,'.png'];
        end
        if outputPdf
            pdffigname = [pdfoutputpath,fileNameFields{1},fileFieldSep,fileNameFields{2},fileFieldSep,datestr(unix2matlabtime(batchstartutc(batch)),...
                [outFileDateFormat,fileFieldSep,outFileTimeFormat]),fileFieldSep,fileNameFields{5},fileFieldSep,datatag,'.pdf'];
        end
    end
    % Check if figure exists - print will not overwrite existing figure
    if(outputPng)
        if exist(pngfigname,'file') == 2
            warning('dicePlotTool:pngFileExists','png figure \n %s \n Already Exists!\n new file will be written with current time suffix',pngfigname);
            pngfigname = [pngfigname(1:end-4),datestr(now,'_yyyymmdd_hhMMss'),'.png'];
        end
    end
    if(outputPdf)
        if exist(pdffigname,'file') == 2
            warning('dicePlotTool:pdfFileExists','png figure \n %s \n Already Exists!\n new file will be written with current time suffix',pdffigname);
            pdffigname = [pdffigname(1:end-4),datestr(now,'_yyyymmdd_hhMMss'),'.pdf'];
        end
    end
    % Save figure:
    fprintf('Saving figure ..\n')
    if outputPdf
        set(figHandle(batch),'PaperSize',pdfpagesize)
        set(figHandle(batch),'PaperPosition',[0,0,pdfpagesize])
        print(figHandle(batch),'-dpdf',pdffigname,['-r',num2str(pdfresolution)],['-',lower(pdfrenderer)])
        fprintf('Saved as %s\n',pdffigname);
    end
    
    if outputPng
        set(figHandle(batch),'PaperSize',pngpagesize)
        set(figHandle(batch),'PaperPosition',[0,0,pngpagesize])
        print(figHandle(batch),'-dpng',pngfigname,['-r',num2str(pngresolution)],['-',lower(pngrenderer)]);
        fprintf('Saved as %s\n',pngfigname);
    end
    if exist('pngfigname','var')
        [~,figname,~] = fileparts(pngfigname);
    else
        [~,figname,~] = fileparts(pdffigname);
    end
    % save tick information to a text file
    if(~multiplePlots)
        txtoutputpath = [outputpath 'txt' filesep];
        if exist(txtoutputpath,'dir')==0
            mkdir(txtoutputpath);
        end
        ticksfilename = [txtoutputpath,filesep,figname,fileFieldSep,'Ticks.txt'];
        tickfid = fopen(ticksfilename,'w');
        % cleans the fileID when the function exits.
        tickFilecleaner = onCleanup(@()fclose(tickfid));
        fprintf(tickfid,'Time,Distance,Longitude,Latitude\n(UTC),(km),(deg),(deg)\n');
        for ttk = 1:numel(timeTicks)
            fprintf(tickfid,'%s,%0.2f,%0.3f %c,%0.3f %c\n',...
                datestr(unix2matlabtime(timeTicks(ttk)),'HH:MM:SS'),distTicks(ttk),...
                abs(lonTicks(ttk)),lonEW(lonTicks(ttk)),abs(latTicks(ttk)),latNS(latTicks(ttk)));
        end
        clear tickFilecleaner;
    end
    try
        figInfo(batch,:) = {figname,datestr(unix2matlabtime(batchechotime(1)),'mm/dd/yyyy,HH:MM:SS'),batchechotime(1),...
            datestr(unix2matlabtime(batchechotime(2)),'mm/dd/yyyy,HH:MM:SS'),batchechotime(2),...
            batchlat(1),batchlon(1),batchhell(1),batchlat(2),batchlon(2),batchhell(2)};
    catch
        warning('dicePlotTool:noPosInfo','Position info is not Available');
        warning('off','dicePlotTool:noPosInfo');
        figInfo(batch,:) = {NaN,NaN,NaN,NaN,NaN,NaN,NaN,NaN,NaN};
    end
end