%% Dice Analog Rx amplifier Power,SNR calculation etc...
% clear all; close all; clc;

InputSignaldB = -120
% signal power at input
Atten1Val = 20; % Attenuation value for first Attenuator
Atten2Val = 0; % Attenuation value for first Attenuator

% 80% coherence
NumHwStacks = 16;
NumSwStacks = 1;

pulseWidth = 10e-6;
bandwidth = 20e6; % account for tapering and iq separation
stackingGaindB = 10*log10(NumHwStacks*NumSwStacks); %% Gain on SNR only
CompressionGaindB = 10*log10(pulseWidth * bandwidth);
Earth_Temp = 290 ; %K
Sky_Temp = 290; %K -- little high but ok
ATemp = 0.5*Earth_Temp + 0.5* Sky_Temp; %% half the antenna is looking up
Kb = 1.38e-23 ;%JK^-1
Antenna_BW = 1032e6;
Antenna_Noise = 10*log10(Kb*ATemp*Antenna_BW*1000)  %% in dBm
InputSNRdB = InputSignaldB - Antenna_Noise
%% Switch
SWGain = -1.2;
SWNF = -SWGain;
SWPower = InputSignaldB + SWGain;
%% LNA
LNAGain = 36;     %dB's
LNANF = 1.3;     %dB's
LNAInmax = -8;  % Maximum allowable input at LNA input
LNAOutmax = 8; % Maximum possible output at 
if(SWPower >= LNAInmax)
    warning('LNA Input limit exceeded!');
end
LNAPower = SWPower + LNAGain;
if(LNAPower > LNAOutmax)
    warning('LNA Output Compressed!');
    LNAPower = LNAOutmax;
    LNAGain = LNAPower - SWPower;
end
%% BPF 
BPFGain=-1.2;   %dB's
BPFNF= - BPFGain;  %dB's
BPFPower = LNAPower + BPFGain;
%% Attenuator 1 
Att01Gain=-(2.2+ Atten1Val);   %dB's
Att01Inmax = 35;  % Maximum allowable input at LNA input
Att01Outmax = 24; % Maximum possible output at 
if(BPFPower >= Att01Inmax)
    warning('Attenuator1 Input limit exceeded!');
end
Att01Power = BPFPower + Att01Gain;
if(Att01Power > Att01Outmax)
    warning('Attenuator1 Output Compressed!');
    Att01Power = Att01Outmax;
    Att01Gain = Att01Power - BPFPower;
end
Att01NF= - Att01Gain;
%% Amp1
Amp01Gain=31;   %dB's
Amp01NF=2.5;    %dB's
Amp01Inmax = -3;  % Maximum allowable input at Amp1 (ZRL-400) input
Amp01Outmax = 25; % Maximum possible output at 
if(Att01Power >= Amp01Inmax)
    warning('Amp1 Input limit exceeded!');
end
Amp01Power = Att01Power + Amp01Gain;
if(Amp01Power > Amp01Outmax)
    warning('Amp1 Output Compressed!');
    Amp01Power = Amp01Outmax;
    Amp01Gain = Amp01Power - Att01Power;
end
%% Attenuator 2
Att02Gain=-(2.2+ Atten2Val);   %dB's
Att02Inmax = 35;  % Maximum allowable input at LNA input
Att02Outmax = 24; % Maximum possible output 
if(Amp01Power >= Att02Inmax)
    warning('Attenuator2 Input limit exceeded!');
end
Att02Power = Amp01Power + Att02Gain;
if(Att02Power > Att02Outmax)
    warning('Attenuator2 Output Compressed!');
    Att02Power = Att02Outmax;
    Att02Gain = Att02Power - Amp01Power;
end
Att02NF= - Att02Gain ;%+2;   %dB's
%% Amp2
Amp02Gain=31;   %dB's
Amp02NF=2.5;    %dB's
Amp02Inmax = -3;  % Maximum allowable input at Amp2 (ZRL-400) input
Amp02Outmax = 25; % Maximum possible output
if(Att02Power >= Amp02Inmax)
    warning('Amp2 Input limit exceeded!');
end
Amp02Power = Att02Power + Amp02Gain;
if(Amp02Power > Amp02Outmax)
    warning('Amp2 Output Compressed!');
    Amp02Power = Amp02Outmax;
    Amp02Gain = Amp02Power - Att02Power;
end
%% 
%%
SWGainW = 10^(SWGain/10);
SWNFW = 10^(SWNF/10);
LNAGainW=10^(LNAGain/10); %W
LNANFW=10^(LNANF/10);    %W
BPFGainW=10^(BPFGain/10);  %W
BPFNFW=10^(BPFNF/10);     %W
Att01GainW=10^(Att01Gain/10);    %W
Att01NFW=10^(Att01NF/10);   %W
Amp01GainW=10^(Amp01Gain/10);   %W
Amp01NFW=10^(Amp01NF/10);    %W
Att02GainW=10^(Att02Gain/10);   %W
Att02NFW=10^(Att02NF/10);      %W
Amp02GainW=10^(Amp02Gain/10);   %W
Amp02NFW=10^(Amp02NF/10);    %W


totalLinearGain=LNAGainW*BPFGainW*Att01GainW*Amp01GainW*Att02GainW*Amp02GainW;
totalGaindB = 10*log10(totalLinearGain)
%% Compute the noise figure of the system
NoiseFigure =  SWNFW + ...
               (LNANFW - 1)/SWGainW + ...
               (BPFNFW - 1)/(LNAGainW*SWGainW) + ...
               (Att01NFW - 1)/(LNAGainW*BPFGainW*SWGainW) + ...
               (Amp01NFW - 1)/(LNAGainW*BPFGainW*Att01GainW*SWGainW) + ...
               (Att02NFW - 1)/(LNAGainW*BPFGainW*Att01GainW*Amp01GainW*SWGainW) + ...
               (Amp02NFW - 1)/(LNAGainW*BPFGainW*Att01GainW*Amp01GainW * Att02GainW*SWGainW);
NoiseFiguredB = 10*log10(NoiseFigure)           % in dBs 
%% Outputs @ An. Receiver
OutputSNRdB = InputSNRdB - NoiseFiguredB        
OutputPowerdB = Amp02Power;
OutputNoisePower = OutputPowerdB - OutputSNRdB

%% Minimun detectable power at the digitizer input
Min_detPwrdB = 10*log10((2*1.125/4096)^2/50*1000);
if(OutputPowerdB <Min_detPwrdB)
    warning('Signal Power is not detectable!!');
end
% After FPGA Stacking
% Signal power goes up by factor of n^2 and noise power goes up by factor
% of n
OutputSigAfterHwStackingdB = OutputPowerdB + 10*log10(NumHwStacks^2)
OutputNoisePowerAfterStacking = OutputNoisePower + 10*log10(NumHwStacks)
% After Software stack and Compression
SNR_StackdB = OutputSNRdB + stackingGaindB
SNR_Compression = SNR_StackdB + CompressionGaindB

