function [] = PlotDiceQC(inputfolder,ProjDesc,chan,dFileStr,filenametimeinds)
%% Plot long echogram:

TxInd1us = 306+149;
TxInd10us = 306+449;
ProjDesc(ProjDesc == '_')= '-';
freport = fopen([inputfolder,ProjDesc,'QCReport.txt'],'w');
fprintf(freport,'DICE QC Report\n Generated: %s\n\n',datestr(now,'mm/dd/yyyy HH:MM:SS'));
fprintf(freport,'\t\tFile Name\t\t\t\t\t\t File Size (MBytes)\n');
fprintf('\t\tFile Name\t\t\t\t\t\t File Size (MBytes)\n');
for jj = 1:length(dFileStr)
    fprintf(freport,'%s\t\t\t%0.1f\n',dFileStr(jj).name,dFileStr(jj).bytes/1024/1024);
    fprintf('%s\t\t\t%0.1f\n',dFileStr(jj).name,dFileStr(jj).bytes/1024/1024);
end
fprintf(freport,'\n\n');
fprintf('\n\n');
for channum = 1:length(chan)
    switch(chan(channum))
        case 1
            filekey='*10us_Ch1*.mat';  % search key.  Should contain "*"
            chanNum = 'Ch1';
            pulseLen = ' 3\mus';
            TxInd = TxInd10us;
        case 2
            filekey='*10us_Ch2*.mat';  % search key.  Should contain "*"
            chanNum = 'Ch2';
            pulseLen = ' 3\mus';
            TxInd = TxInd10us;
        case 3
            filekey='*1us_Ch1*.mat';  % search key.  Should contain "*"
            chanNum = 'Ch1';
            pulseLen = ' 1\mus';
            TxInd = TxInd1us;
        otherwise
            filekey='*1us_Ch2*.mat';  % search key.  Should contain "*"
            %             figname=[inputfolder,ProjDesc,'Stack32_1usCh2_QC.png'];
            %             titletext=[ProjDesc(1:end-1), ', Ch2, 1\mus, stacking=64'];
            chanNum = 'Ch2';
            pulseLen = ' 1\mus';
            TxInd = TxInd1us;
    end
    
    % clims=[5,90];                    % dB
    ylims=[-50, 2500];                   % m
    cmap='gray';                       % valid colormap string
    invertcaxis=1;                     % 0 or 1
    c_air=3e8;                         % m/s
    c_ice=1.68e8;                      % m/s
    cutterH=1;                         % horizontal decimation
    pagesize=[15,8];                   % inches
    resolution=300;                    % dpi
    numAvg = 1;
    
    % Find input files:
    inputfiles=dir([inputfolder,filekey]);
    numfiles=length(inputfiles);
    if numfiles==0
        error('Could not locate input files.')
    end
    
    % Loop through and plot input files:
    lastind=0;
    isfirstfile=1;
    for ii=1:numfiles
        load([inputfolder,inputfiles(ii).name],'radecho','twtt','utc','numAvg','SurfPwr','BedPwr','NoisePwr','SurfInds','BedInds','numbatches')
        radecho = real(radecho) + 1i*imag(radecho);
        DepthScale=(twtt - twtt(TxInd))*c_ice/2;
        
        report.surfPwr(ii,chan(channum)) = SurfPwr;
        report.bedPwr(ii,chan(channum)) = BedPwr;
        report.noisePwr(ii,chan(channum)) = NoisePwr;
        report.utc(ii,chan(channum)) = utc(1);
        report.surfRange(ii,chan(channum)) = mean(DepthScale(SurfInds));
        report.bedRange(ii,chan(channum)) = mean(DepthScale(BedInds));
        report.fileName{ii,chan(channum)} = inputfiles(ii).name;
        
        % Determine data within elevation range:
        InRange= abs(DepthScale)>ylims(1)& abs(DepthScale)<ylims(2);
        
        %radecho(InRange,1:cutterH:end) = filter(ones(1,numAvg)/numAvg,1,abs(radecho(InRange,1:cutterH:end)),[],2);
        % plot data:
        figure(ii);
        subplot(4,1,channum);
        if invertcaxis==1
            imagesc(unix2matlabtime(utc),DepthScale(InRange),-20*log10(abs(radecho(InRange,1:cutterH:end))));
        else
            imagesc(unix2matlabtime(utc),DepthScale(InRange),20*log10(abs(radecho(InRange,1:cutterH:end))));
        end
        ylim(ylims);
        if(channum == 1)
            titletext={['DICE QC: ', ProjDesc(1:9),'-',inputfiles(ii).name(15:22),'-',inputfiles(ii).name(24:29)];[chanNum,',', pulseLen,', Stacking =',num2str(numAvg)]};
        else
            titletext= [chanNum,',', pulseLen];
        end
        title(titletext)
        xlabel('Flight Time, UTC')
        ylabel('Range in Ice, m')
        datetick2('x','HH:MM:SS','keeplimits')
        set(gca,'ydir','reverse')
        set(gca,'TickDir','out')
        eval(['colormap(',cmap,')'])
        
        figname = [inputfolder,ProjDesc,datestr(unix2matlabtime(utc(1)),'HHMM'),'_batch',num2str(ii),'_QC.pdf'];
        set(gcf,'PaperSize',pagesize)
        set(gcf,'PaperPosition',[0,0,pagesize])
        print('-dpdf',figname,['-r',num2str(resolution)])
    end
end
fprintf('Printing Report...\n')
for bb = 1:numbatches
    for cc = 1:length(chan)
        fprintf(freport,'UTC Time %s : File %s \n',datestr(unix2matlabtime(report.utc(bb,chan(cc))),'mm/dd/yyyy HH:MM:SS'),[report.fileName{bb,chan(cc)}]);
        %         fprintf(freport,'Surface Power  : %g dB\t\t Range (Ice Dist c = 1.68e8) : %g meters\n',report.surfPwr(bb,chan(cc)),report.surfRange(bb,chan(cc)));
        fprintf(freport,'Surface Power  : %0.1f dB\t\t Range\t Air: %0.1f m (%0.1f ft) \t Ice Equivalent (c = 1.68e8): %0.1fm\n',...
            report.surfPwr(bb,chan(cc)),report.surfRange(bb,chan(cc))*3/1.68,report.surfRange(bb,chan(cc))*3/1.68*3.28,report.surfRange(bb,chan(cc)));
        %         fprintf(freport,'Bed Power      : %g dB\t\t Range (Ice Dist c = 1.68e8) : %g meters\n',report.bedPwr(bb,chan(cc)),report.bedRange(bb,chan(cc)));
        fprintf(freport,'Bed Power  : %0.1f dB\t\t Range\t Air: %0.1f m (%0.1f ft) \t Ice Equivalent (c = 1.68e8): %0.1fm\n',...
            report.bedPwr(bb,chan(cc)),report.bedRange(bb,chan(cc))*3/1.68,report.bedRange(bb,chan(cc))*3/1.68*3.28,report.bedRange(bb,chan(cc)));
        fprintf(freport,'Noise Power    : %0.1f dB\n\n',report.noisePwr(bb,chan(cc)));
        fprintf('UTC Time %s : File %s \n',datestr(unix2matlabtime(report.utc(bb,chan(cc))),'mm/dd/yyyy HH:MM:SS'),[report.fileName{bb,chan(cc)}]);
        %         fprintf('Surface Power  : %g dB\t\t Range (Ice Dist c = 1.68e8) : %g meters\n',report.surfPwr(bb,chan(cc)),report.surfRange(bb,chan(cc)));
        fprintf('Surface Power  : %0.1f dB\t\t Range\t Air: %0.1f m (%0.1f ft) \t Ice Equivalent (c = 1.68e8): %0.1fm\n',...
            report.surfPwr(bb,chan(cc)),report.surfRange(bb,chan(cc))*3/1.68,report.surfRange(bb,chan(cc))*3/1.68*3.28,report.surfRange(bb,chan(cc)));
        %         fprintf('Bed Power      : %g dB\t\t Range (Ice Dist c = 1.68e8) : %g meters\n',report.bedPwr(bb,chan(cc)),report.bedRange(bb,chan(cc)));
        fprintf('Bed Power  : %0.1f dB\t\t Range\t Air: %0.1f m (%0.1f ft) \t Ice Equivalent (c = 1.68e8): %0.1fm\n',...
            report.bedPwr(bb,chan(cc)),report.bedRange(bb,chan(cc))*3/1.68,report.bedRange(bb,chan(cc))*3/1.68*3.28,report.bedRange(bb,chan(cc)));
        fprintf('Noise Power    : %0.1f dB\n\n',report.noisePwr(bb,chan(cc)));
    end
end

fclose(freport);

toc