% Tejendra Dhakal
% 02/01/2012
% LDEO, Columbia University

% This Script parses all the radar files in a User selected QC directory.
% By default both signal types are processed for both channels

% B'atch processing. The script now combines files into batches depending
% time separation between the files. >11s separation creates a new batch

% Saving data.  Scripts produces warning if there are data in processed
% directory. User should create new folders for each processing run

% Stacking.  Default stacking of 32 is used to reduce processing times
% Plots. Single plot per batch is generated - all channels are plotted into
% one using subpolot and saved to a pdf
%%
clc; clearvars;
%close all;
tic

%% Parameters:

% File Paths and Directories:

% reffile = 'C:\Users\Tej\Desktop\Icepod\DICE\Matlab\Chirp\RefSignalsIdeal_IQ.mat';
% reffile = 'C:\Users\Tej\Desktop\Icepod\DICE\Matlab\Chirp\RefSignalsCalib_IQ.mat';
%inputfolder='F:\DICE-flight-05\';                                    % folder containing input data (include trailing '/' or '\')
%inputfolder='/local/data/gravity/users/dhakal/Icepod/GL3/FLT_04_DICE/';    % folder containing input data (include trailing '/' or '\')
%outputfolder='F:\DICE-flight-05\Processed\';                           % folder to send output data (include trailing '/' or '\')
% outputfolder='/local/data/gravity/users/dhakal/Icepod/GL3/FLT_04_DICE/PulseCompressed/';    % folder to send output data (include trailing '/' or '\')
numTry = 2;
while(numTry)
    inputfolder = uigetdir([],'Select the directory containing DICE QC Data');                  % folder containing input data (include trailing '/' or '\')
    numTry = numTry -1;
    if(inputfolder == 0)
        if(numTry > 0)
            ed = errordlg({'Stop wasting my flops!';'Select the QC data directory or else..'},'Path Error');
            uiwait(ed,60);
        else
            errordlg({'Have it your way..';'I''d rather spend my flops watching Mr. Robot anyway'});
            uiwait(ed,5);
            error('Invalid Path');
        end
    else
        break;
    end
end

while(1)
    surfRange = inputdlg('Enter the survey elevation (ft):','Elevation Input)',1,{'2500'});
    surfRange = str2double(surfRange);
    if isnan(surfRange)
        ed = errordlg({'Invalid Input!';'Try Again.'});
        uiwait(ed,60);
    elseif surfRange < 300
        ed = errordlg({'I doubt they flew this low; try again!'});
        uiwait(ed,60);
    else
        fprintf('Using %d ft for survey elevatioon\n',surfRange)
        break;
    end
end

inputfolder = [inputfolder,filesep];
outputfolder=[inputfolder,'PulseCompressed_QC',filesep];                           % folder to send output data (include trailing '/' or '\')

% Radar Settings:
PRI = 160e-6;                      % Pulse Repitition Interval (s)
sampWin = 80e-6;
datalen10us = 12000;               % Number of samples in 10 us data (integer)
datalen1us = 12000;                % Number of samples in 1 us data (integer)
numHWStacks = 16;                  % Number of hardware stacks
fs = 150e6;                        % Sampling Frequency (Hz)
pw1 = 3e-6;                       % pulse width for 1st plse (10 us)
pw2 = 1e-6;                        % pulse width for 2nd pulse( 1us)
bw = 60e6;                         % Bandwidth
fc = 38e6;                         % Center Frequency
startFreq = 8e6;                   % start Frequency for Chirp
endFreq = 68e6;                    % end Frequency for Chrip
numbits=16;                        % Number of digitizer bits (integer)
nummetadatasamples=16;             % Number of samples in the data file that are metadata (integer)
filenamedateinds=[12:19];          % Indices of datestamp within input file names (two integers)
filenametimeinds=[21:26];          % Indices of timestamp within input file names (two integers)
projdescInd = [1:19];
FlightNoInd = [6:9];

% Processing Settings:
doCompression = 1;                  % if disabled raw datawill be saved instead
IdealRef = 1;                       % Choose between Ideal and calibrated chirp
Campaign = 'RS01';

processChan = [1 2 3 4];               % 1 = 10us CH1; 2 = 10us CH2; 3 = 1us CH1; 4 = 1us CH2, can be vector to process multiple channels
filesperbatch = 10;                % Number of files compiled into each batch (integer)
numAvg = 32;                        % Software Stacking Length for this processing (integer)
fileLenTime = 10;                  % File length time (s)
usefilter=1;                       % Use filter loaded above?  (0 or  1)
usenotchfilter = 0;                % Use notch filters to take  out specific frequencies- change the frequencies in the code as required
tapertype='tukey';                % What taper (if any) to use on the reference function ('tukey', 'kaiser', 'both', or 'none')
taperparam=[0.3 5];                % Adjustable parameter to go with the taper.  Meaning varies by taper type.  For 'both', the first element is for tukey and the second is for chebyshev
overwriteoutput=0;                 % Controls whether or not to overwrite output files on disc (0 or 1)
replacezeros=1;                    % Controls whether to replace data with zeros in a discrete band, specified below (0 or 1)
process_metadata = 0;              % disabling this might speed things up a bit.

winStartInd1us = [1, 720];         % Starting index of zero replacement (integer) (1 for 10us, 1 for 1us)
% winStopInd1us = [280, 1200];       % Ending index of zero replacement (integer)  (270 for 10us, 270 for 1us)
winStopInd1us = [280, 1100];       % Ending index of zero replacement (integer)  (270 for 10us, 270 for 1us)
winStartInd10us = [1, 2710];       % Starting index of zero replacement (integer) (2712 for 10us, 630 for 1us)
winStopInd10us = [280, 3800];      % Ending index of zero replacement (integer)  (3172 for 10us, 990 for 1us)

winSizeSurfFilt = 100;

TxInd1us = 306+149;
TxInd10us = 306+449;

% surfbreakrange10 = [700:800] + TxInd10us;
surfbreakrange10 = floor(surfRange/3.28) + TxInd10us + [-100:100];  % Range is about 1 Sample/m in Air.
surfbreakrange1 = floor(surfRange/3.28) + TxInd1us + [-100:100];    % index

bedRange10 = surfbreakrange10(1)+200 : surfbreakrange10(1)+2200;    % Range is about 1 Sample/0.45m in Ice c = 1.68.
bedRange1 = surfbreakrange1(1)+200 : surfbreakrange1(1)+2200;

surfbreakrange = [surfbreakrange10',surfbreakrange10',surfbreakrange1',surfbreakrange1'];
bedRange = [bedRange10',bedRange10',bedRange1',bedRange1'];

NoiseInds= 11000:11500;      % index


% Notch filter setting:
BW = 1;                            % Bandwidth
Apass = 1;                         % Bandwidth Attenuation
Fs = 150;                          % Sampling Frequency
%notchList = [11.75, 35.25, 58.75, 8.99, 17, 20.74, 23.5, 30, 32.49, 35.26, 40.5, 44.24, 49.49, 56, 65.25, 67.75];

% % Moving average:
% smoothfilter = 0;               % Use running average? (0 or 1)
% smoothwind = 32;                % Running average window of echos
% smoothingcoef_b = ones(1, smoothwind)/smoothwind;



% Load reference signals and filter function:
if(exist('reffile','var') && IdealRef ==0)
    load(reffile)
    Nref1us = length(sig1usref);
    Nref10us = length(sig10usref);
    sig1usref = sig1usref/max(sig1usref);
    sig10usref = sig10usref/max(sig10usref);
    t1 = linspace(0,Nref10us-1,Nref10us)'/fs;
    t2 = linspace(0,Nref1us-1,Nref1us)'/fs;
else
    %% Reference signals
    t1 = [0:1/fs:pw1-1/fs]';
    t2 = [0:1/fs:pw2-1/fs]';
    % compute chirp phase
    freq1 = startFreq + bw/pw1*t1/2;
    freq2 = startFreq + bw/pw2*t2/2;
    fprintf('Using Ideal Reference\n');
    sig10usref = exp(1i*2*pi*freq1.*t1);
    sig1usref = exp(1i*2*pi*freq2.*t2);
    % Figure out size of reference signals:
    Nref1us = length(sig1usref);
    Nref10us = length(sig10usref);
end
% Define filter
[bb_b, bb_a] = butter(8,[7 69]/75,'bandpass');


% Apply taper to reference signal:
if strcmp(tapertype,'tukey')
    sig1usref = sig1usref.*tukeywin(Nref1us,taperparam(1));
    sig10usref = sig10usref.*tukeywin(Nref10us,taperparam(1));
elseif strcmp(tapertype,'kaiser')
    sig1usref = sig1usref.*kaiser(Nref1us,taperparam(2));
    sig10usref = sig10usref.*kaiser(Nref10us,taperparam(2));
elseif strcmp(tapertype,'both')
    sig1usref = sig1usref.*tukeywin(Nref1us,taperparam(1)).*kaiser(Nref1us,taperparam(2));
    sig10usref = sig10usref.*tukeywin(Nref10us,taperparam(1)).*kaiser(Nref10us,taperparam(2));
elseif strcmp(tapertype,'none')==0
    error('Parameter "tapertype" must be a string equal to "tukey", "cheb", "both", or "none".')
end

% Create complex baseband ref;
sig10usref =  cos(2*pi*fc*t1).*sig10usref - 1i*sin(2*pi*fc*t1).*sig10usref;
sig1usref =     cos(2*pi*fc*t2).*sig1usref - 1i*sin(2*pi*fc*t2).*sig1usref;

% Filter reference function:
if usefilter==1
    sig1usref=filtfilt(bb_b,bb_a,sig1usref);
    sig10usref=filtfilt(bb_b,bb_a,sig10usref);
end
Nt1 = length(sig10usref) + datalen10us -1;
Nt2 = length(sig1usref) + datalen1us -1;

% Zero-pad reference function to same length as data:
sig1usref=[sig1usref;zeros(datalen1us-1,1)];
sig10usref=[sig10usref;zeros(datalen10us-1,1)];

% Fourier Transform Reference Function:
sig10usref = conj(fft(sig10usref,Nt1));
sig1usref = conj(fft(sig1usref,Nt2));

%% Check files and directories:

% Find input files:
dFileStr = dir([inputfolder,'*.dice']);
numDataFiles = length(dFileStr);
if numDataFiles==0
    ed = errordlg({'Data Files Not Found!';'Please Check the QC Data Dir:';inputfolder});
    uiwait(ed,15);
    exit;
end

% Sort data files by timestamp:
for ii = 1:numDataFiles
    dFileStr(ii).utc=86400*(datenum([dFileStr(ii).name(filenamedateinds),dFileStr(ii).name(filenametimeinds)],'yyyymmddHHMMSS')...
        -datenum('01-01-1970 00:00:00','dd-mm-yyyy HH:MM:SS'));
end

[sv,sx] = sort([dFileStr.utc]);
dFileStr = dFileStr(sx);
fTime = [dFileStr.utc];
dTime = diff(fTime);
batchind = find(dTime >22);
if(isempty(batchind))
    batchind = 1;
    batchmin = 1;
    batchmax = numel(fTime);
    numbatches = 1;
else
    numbatches = numel(batchind)+1;
    batchmin(1) = 1;
    batchmax(1) = batchind(1);
    batchmin(numbatches) = batchind(end)+1;
    batchmax(numbatches) = numDataFiles;
    
    for bb=2:numbatches-1
        batchmin(bb) = batchind(bb-1)+1;
        batchmax(bb) = batchind(bb);
    end
end
ProjDesc = dFileStr(1).name(projdescInd);
FlightNo = dFileStr(1).name(FlightNoInd);

% Make output folder, if necessary:
if exist(outputfolder,'dir')==0
    mkdir(outputfolder)
end
%% Frame Computations

% Compute number of batches to process:
% Compute frame size and frame length:
frameSize = (datalen10us*2 + datalen1us*2+80)*2;
frameLen = frameSize/2;
maxNumFrames = ceil(max([dFileStr.bytes])/frameSize)+numAvg+1;        % Maximum data traces in a file (integer)
%% Check Channels to Process:

% Pre-allocate memory:
numChannels=length(processChan);
echotypetmp=cell(numChannels,1);
ref=cell(numChannels,1);
Nt=zeros(numChannels,1);
Nref=zeros(numChannels,1);
winStartInd=zeros(numChannels,size(winStartInd10us,2));
winStopInd=zeros(numChannels,size(winStopInd10us,2));
dataind=cell(numChannels,1);
metadataind=cell(numChannels,1);
lastTime = zeros(numChannels,1);

%% Produce Notch filters to be used

% for flist = 1:size(notchList,2)
%     [b, a] = iirnotch(notchList(flist)/(Fs/2), BW/(Fs/2), Apass);
%     Hd(flist) = dfilt.df2(b,a);
% end

% New filters:

Fnotch = 10;  % Notch Frequency
[b, a] = iirnotch(Fnotch/(Fs/2), BW/(Fs/2), Apass);
Hd1     = dfilt.df2(1, 1);

Fnotch = 20.4;
[b, a] = iirnotch(Fnotch/(Fs/2), BW/(Fs/2), Apass);
Hd2     = dfilt.df2(1, 1);

Fnotch = 30;
[b, a] = iirnotch(Fnotch/(Fs/2), BW/(Fs/2), Apass);
Hd3     = dfilt.df2(b, a);


Fnotch = 40;
[b, a] = iirnotch(Fnotch/(Fs/2), BW/(Fs/2), Apass);
Hd4     = dfilt.df2(1, 1);

Fnotch = 50;
[b, a] = iirnotch(Fnotch/(Fs/2), BW/(Fs/2), Apass);
Hd5     = dfilt.df2(b, a);

Fnotch = 60;
[b, a] = iirnotch(Fnotch/(Fs/2), BW/(Fs/2), Apass);
Hd6     = dfilt.df2(1, 1);

% Loop through channels to process:
for channel=1:numChannels
    
    % Check channel:
    switch processChan(channel)
        case 1
            % Assign echo type:
            echotypetmp{channel} = '10us_Ch1';
            % Assign and expand reference function:
            ref{channel} = repmat(sig10usref,[1,maxNumFrames]);
            % Assign lengths:
            Nt(channel) = datalen10us;
            Nref(channel) = Nref10us;
            NFFT(channel) = Nt1;
            truncLen(channel) = NFFT(channel);
            % Assign indices of zero replacement interval:
            winStartInd(channel,:)=winStartInd10us;
            winStopInd(channel,:)=winStopInd10us;
            % Assign indices of this channel within the input file:
            dataind{channel} = repmat(logical([zeros(2,1);ones(datalen10us,1);zeros(nummetadatasamples,1);zeros(2,1);...
                zeros(datalen10us + 20 + (datalen1us + 20)*2, 1)]),floor(maxNumFrames),1);
            metadataind{channel} = repmat(logical([zeros(2,1);zeros(datalen10us,1);ones(nummetadatasamples,1);zeros(2,1);...
                zeros(datalen10us + 20 + (datalen1us + 20)*2, 1)]),floor(maxNumFrames),1);
        case 2
            % Assign echo type:
            echotypetmp{channel} = '10us_Ch2';
            % Assign and expand reference function:
            ref{channel} = repmat(sig10usref,[1,maxNumFrames]);
            % Assign lengths:
            Nt(channel) = datalen10us;
            Nref(channel) = Nref10us;
            NFFT(channel) = Nt1;
            truncLen(channel) = NFFT(channel);
            % Assign indices of zero replacement interval:
            winStartInd(channel,:)=winStartInd10us;
            winStopInd(channel,:)=winStopInd10us;
            % Assign indices of this channel within the input file:
            dataind{channel} = repmat(logical([zeros(datalen10us + 20,1);zeros(2,1);...
                ones(datalen10us,1);zeros(nummetadatasamples,1);zeros(2,1);...
                zeros((datalen1us + 20)*2, 1)]),floor(maxNumFrames),1);
            metadataind{channel} = repmat(logical([zeros(datalen10us + 20,1);zeros(2,1);...
                zeros(datalen10us,1);ones(nummetadatasamples,1);zeros(2,1);...
                zeros((datalen1us + 20)*2, 1)]),floor(maxNumFrames),1);
        case 3
            % Assign echo type:
            echotypetmp{channel} = '1us_Ch1';
            % Assign and expand reference function:
            ref{channel} = repmat(sig1usref,[1,maxNumFrames]);
            % Assign lengths:
            Nt(channel) = datalen1us;
            Nref(channel) = Nref1us;
            NFFT(channel) = Nt2;
            truncLen(channel) = NFFT(channel);
            % Assign indices of zero replacement interval:
            winStartInd(channel,:)=winStartInd1us;
            winStopInd(channel,:)=winStopInd1us;
            % Assign indices of this channel within the input file:
            dataind{channel} = repmat(logical([zeros((datalen10us + 20)*2,1);zeros(2,1);...
                ones(datalen1us,1);zeros(nummetadatasamples,1);zeros(2,1);...
                zeros((datalen1us + 20), 1)]),floor(maxNumFrames),1);
            metadataind{channel} = repmat(logical([zeros((datalen10us + 20)*2,1);zeros(2,1);...
                zeros(datalen1us,1);ones(nummetadatasamples,1);zeros(2,1);...
                zeros((datalen1us + 20), 1)]),floor(maxNumFrames),1);
        otherwise
            % Assign echo type:
            echotypetmp{channel} = '1us_Ch2';
            % Assign and expand reference function:
            ref{channel} = repmat(sig1usref,[1,maxNumFrames]);
            % Assign lengths:
            Nt(channel) = datalen1us;
            Nref(channel) = Nref1us;
            NFFT(channel) = Nt2;
            truncLen(channel) = NFFT(channel);
            % Assign indices of zero replacement interval:
            winStartInd(channel,:)=winStartInd1us;
            winStopInd(channel,:)=winStopInd1us;
            % Assign indices of this channel within the input file:
            dataind{channel} = repmat(logical([zeros((datalen10us + 20)*2 + datalen1us + 20, 1);...
                zeros(2,1);ones(datalen10us,1);zeros(nummetadatasamples,1);zeros(2,1)]),floor(maxNumFrames),1);
            metadataind{channel} = repmat(logical([zeros((datalen10us + 20)*2 + datalen1us + 20, 1);...
                zeros(2,1);zeros(datalen1us,1);ones(nummetadatasamples,1);zeros(2,1)]),floor(maxNumFrames),1);
    end
    
end

%% Miscellaneous Computations:

% Compute number of batches to process:
% numbatches=ceil((endfile-startfile+1)/filesperbatch);
filesperbatch = max(batchmax-batchmin)+1;
% Compute denominator for data normalization:
denom=2^(numbits-1);

%% Process Data:

% Pre-allocate memory:
tmpfile=zeros(max([dFileStr.bytes]/2)+frameSize,1);
numExcessFrames=zeros(numChannels,1);
radechotmp=cell(numChannels,1);
utctmp=cell(numChannels,1);
metadatatmp=cell(numChannels,1);
datastack=cell(numChannels,1);
tmpdata=cell(numChannels,1);
excessdata=cell(numChannels,1);
excessmetadata=cell(numChannels,1);
for channel=1:numChannels
    radechotmp{channel}=zeros(NFFT(channel),ceil(maxNumFrames/numAvg)*filesperbatch);
    utctmp{channel}=zeros(1,ceil(maxNumFrames/numAvg)*filesperbatch);
    metadatatmp{channel}=zeros(nummetadatasamples,ceil(maxNumFrames/numAvg)*filesperbatch,numAvg);
    datastack{channel}=zeros(NFFT(channel),ceil(maxNumFrames/numAvg));
    tmpdata{channel}=zeros(NFFT(channel),ceil(maxNumFrames/numAvg));
    excessdata{channel}=zeros(Nt(channel),numAvg);
    excessmetadata{channel}=zeros(nummetadatasamples,numAvg);
end

% Generate IQ-demod matrix
tdata = [0:1/fs:sampWin-1/fs]';
iqDemod = repmat(cos(2*pi*fc*tdata)-1i*sin(2*pi*fc*tdata),1,maxNumFrames);

% Initialize batch start and end files:
minfNum=batchmin;
maxfNum=batchmax;

lastTime(:) = fTime(1);
% Loop through batches:
for batch=1:numbatches
    
    % Communicate with human:
    fprintf('Processing batch %d of %d\n',batch,numbatches)
    
    % Reset echo counter:
    numechos = ones(numChannels,1);
    
    % Loop through files:
    for ii = minfNum(batch):maxfNum(batch)
        
        % Check file size:
        if mod([dFileStr(ii).bytes],frameSize)>0 || dFileStr(ii).bytes == 0
            warning('Warning: Skipping Truncated File %s\n',dFileStr(ii).name)
            if(ii < maxfNum(batch))
                lastTime(:) = fTime(ii+1); % reset the start time
                numExcessFrames(:) = 0;     % reset excess frame count
            end
            continue
        end
        lastTime(:) = fTime(ii); % reset the start time
        
        % Read file:
        % Counters:
        % numFramesInFile = number of frames in current file
        % numAllFrames = Frames in files plus the excess frames from previous file
        % numReadFrames = number of frames to read from current data
        % numRemainingFrames = number of frames to be read from current data into excess Frame
        % numStackedFrames = number of frames to process after stacking
        fid1 = fopen([inputfolder,dFileStr(ii).name],'r');
        numFramesInFile = dFileStr(ii).bytes/frameSize;
        numFramesToStack = (numFramesInFile + numExcessFrames(1)) - mod((numFramesInFile + numExcessFrames(1)),numAvg);
        numRemainingFrames = numFramesInFile- (numFramesToStack - numExcessFrames(1));
        numStackedFrames = numFramesToStack/numAvg;
        tmpfile(1:numFramesInFile*frameSize/2) = fread(fid1,numFramesInFile*frameSize/2,'int16');
        fclose(fid1);
        
        % Check number of frames:
        if numFramesInFile ==0
            warning('Warning: Skipping File, NumFrames is zero');
            continue
        end
        
        % Communicate with human:
        fprintf('Processing file %d of %d\n',ii-minfNum(batch)+1, maxfNum(batch)-minfNum(batch)+1)
        
        % Loop through channels:
        for channel=1:numChannels
            
            % Read unstacked data:
            if numExcessFrames(channel)>0
                dataprestack=[excessdata{channel}(:,1:numExcessFrames(channel)),reshape(tmpfile(dataind{channel}(1:frameLen*numFramesInFile))/denom,Nt(channel),numFramesInFile)];
                if(process_metadata == 1)
                    metadataprestack=[excessmetadata{channel}(:,1:numExcessFrames(channel)),reshape(tmpfile(metadataind{channel}(1:frameLen*numFramesInFile)),nummetadatasamples,numFramesInFile)];
                end
            else
                dataprestack=   reshape(tmpfile(dataind{channel}(1:frameLen*numFramesInFile))/denom,Nt(channel),numFramesInFile);
                if(process_metadata == 1)
                    metadataprestack = reshape(tmpfile(metadataind{channel}(1:frameLen*numFramesInFile)),nummetadatasamples,numFramesInFile);
                end
            end
            
            % Save excess data to be left after stacking
            if numRemainingFrames>0
                excessdata{channel}(:,1:numRemainingFrames)=dataprestack(1:Nt(channel),numAvg*numStackedFrames+1:end);
                if(process_metadata == 1)
                    excessmetadata{channel}(:,1:numRemainingFrames)=metadataprestack(1:nummetadatasamples,numAvg*numStackedFrames+1:end);
                end
            end
            numExcessFrames(channel)=numRemainingFrames;
            
            %% Stack Data and metadata
            % Stack data (sum or mean)
            %             datastack{channel}(1:Nt(channel),1:numStackedFrames) = reshape(mean(reshape(dataprestack(1:Nt(channel),1:numAvg*numStackedFrames)',numAvg,[]),1)',numStackedFrames,[])';
            datastack{channel}(1:Nt(channel),1:numStackedFrames) = reshape(mean(reshape(dataprestack(1:Nt(channel),1:numAvg*numStackedFrames),Nt(channel),numAvg,numStackedFrames),2),Nt(channel),numStackedFrames);
            % Stack metadata:
            if(process_metadata == 1)
                metadatatmp{channel}(:,numechos(channel):numechos(channel)+numStackedFrames-1,:) = permute(reshape(metadataprestack(:,1:numAvg*numStackedFrames),nummetadatasamples,numAvg,numStackedFrames),[1,3,2]);
            end
            %% Use Notch Filter if needed to take out specifit spurious peaks
            if usenotchfilter==1
                %                 datastack{channel}(:,1:numStackedFrames) = filter(Hd16, filter(Hd15, filter(Hd14,filter(Hd13,filter(Hd12,filter(Hd11,filter(Hd10,filter(Hd9,filter(Hd8,...
                %                     filter(Hd7,filter(Hd6,filter(Hd5,filter(Hd4,filter(Hd3,filter(Hd2, filter(Hd1, filtfilt(bbfilter,1,datastack{channel}(:,1:numStackedFrames))))))))))))))))));
                datastack{channel}(1:Nt(channel),1:numStackedFrames) = filter(Hd6,filter(Hd5,filter(Hd4,filter(Hd3,filter(Hd2,filter(Hd1,datastack{channel}(1:Nt(channel),1:numStackedFrames)))))));
                %                     datastack{channel}(:,1:numStackedFrames) = filter(Hd2,filter(Hd1,datastack{channel}(:,1:numStackedFrames)));
            end
            %% demodulate and filter
            if(doCompression ==1)
                datastack{channel}(1:Nt(channel),1:numStackedFrames) = datastack{channel}(1:Nt(channel),1:numStackedFrames).*iqDemod(1:Nt(channel),1:numStackedFrames);
            end
            
            %             if smoothfilter == 1
            %                 datastack{channel}(:,1:numStackedFrames) = (filter(smoothingcoef_b, 1, datastack{channel}(:,1:numStackedFrames)'))';
            %                   %datastack{channel}(:,1:numStackedFrames) = filter(smoothingcoef_b, 1, datastack{channel}(:,1:numStackedFrames), [], 2);
            %             end
            
            
            if usefilter == 1
                datastack{channel}(1:Nt(channel),1:numStackedFrames) = filtfilt(bb_b,bb_a,datastack{channel}(1:Nt(channel),1:numStackedFrames));
                %                 datastack{channel}(1:Nt(channel),1:numStackedFrames) = dicebpf(datastack{channel}(1:Nt(channel),1:numStackedFrames));
            end
            % Replace one section with zeros:
            if replacezeros==1
                for rind = 1:size(winStartInd,2);
                    datastack{channel}(winStartInd(channel,rind):winStopInd(channel,rind),1:numStackedFrames)= rand(winStopInd(channel,rind)-winStartInd(channel,rind)+1, numStackedFrames)*rms(datastack{channel}(NoiseInds));
                    %datastack{channel}(winStartInd(channel,rind):winStopInd(channel,rind),1:numStackedFrames)= datastack{channel}(end-(winStopInd(channel,rind)-winStartInd(channel,rind)+5):end-5, 1:numStackedFrames);
                end
            end
            
            % Perform pulse compression:
            if(doCompression ==1)
                tmpdata{channel}(1:NFFT(channel),1:numStackedFrames) = ifft(fft([zeros(Nref(channel)-1,numStackedFrames);datastack{channel}(1:Nt(channel),1:numStackedFrames)],NFFT(channel)).*ref{channel}(1:NFFT(channel),1:numStackedFrames),NFFT(channel));
                datastack{channel}(1:truncLen(channel),1:numStackedFrames) = real(tmpdata{channel}(1:truncLen(channel),1:numStackedFrames))+ 1j*imag(tmpdata{channel}(1:truncLen(channel),1:numStackedFrames));
                % Assign this file's output matrix:
                radechotmp{channel}(1:truncLen(channel),numechos(channel):numechos(channel)+numStackedFrames-1) = datastack{channel}(1:truncLen(channel),1:numStackedFrames);
            else
                radechotmp{channel}(1:Nt(channel),numechos(channel):numechos(channel)+numStackedFrames-1) = datastack{channel}(1:Nt(channel),1:numStackedFrames);
            end
            
            % Compute utc time vector:
            %             utctmp{channel}(numechos(channel):numechos(channel)+numStackedFrames-1)=fTime(ii)+linspace(0,fileLenTime-PRI*64*numAvg,numStackedFrames);
            utctmp{channel}(numechos(channel):numechos(channel)+numStackedFrames-1)= lastTime(channel)+(0:numStackedFrames-1)*PRI*64*numAvg;
            %             lastTime(channel) = utctmp{channel}(numechos(channel)+numStackedFrames-1)+PRI*64*numAvg;
            % Increment number of echos:
            numechos(channel) = numechos(channel) + numStackedFrames;
        end
        
    end
    
    % Loop through channels:
    for channel=1:numChannels
        % Create final output variables for this batch:
        if(doCompression ==1)
            radecho = radechotmp{channel}(1:truncLen(channel),1:numechos(channel)-1);
            [~,SurfInds]=max(filter(ones(1,winSizeSurfFilt)/winSizeSurfFilt,1,abs(radecho(surfbreakrange(:,channel),:)),[],2));
            [~,BedInds]=max(filter(ones(1,winSizeSurfFilt)/winSizeSurfFilt,1,abs(radecho(bedRange(:,channel),:)),[],2));
        else
            radecho = radechotmp{channel}(1:Nt(channel),1:numechos(channel)-1);
            SurfInds = NaN;
            BedInds = NaN;
        end
        SurfInds = surfbreakrange(1,channel) + SurfInds -1;
        BedInds = bedRange(1,channel) + BedInds -1;
        
        SurfPwr = 20*log10(mean(mean(abs(radecho(SurfInds-5:SurfInds+5,:)))));
        BedPwr = 20*log10(mean(mean(abs(radecho(BedInds-5:BedInds+5,:)))));
        NoisePwr = 20*log10(mean(mean(abs(radecho(NoiseInds,:)))));
        if(process_metadata == 1)
            metadata = metadatatmp{channel}(:,1:numechos(channel)-1,:);
        end
        utc = utctmp{channel}(1:numechos(channel)-1);
        filesinbatch=dFileStr(minfNum(batch):maxfNum(batch));
        
        % Assign this echo type:
        echotype=echotypetmp{channel};
        
        % Compute two-way travel time vector:
        twtt = linspace(0,Nt(channel)/fs,Nt(channel))';
        
        % Create output batch name:
        outputname = [FlightNo,'_',echotype,'_',dFileStr(minfNum(batch)).name(filenamedateinds),'_',datestr(utc(1)/86400,'HHMMSS'),'_',num2str(batch),'.mat'];
        % Save output file:
        if exist([outputfolder,outputname],'file')==0 || overwriteoutput==1
            fprintf('Saving Processed Data\n %s\n',outputname)
            if(process_metadata == 1)
                save([outputfolder,outputname],'FlightNo','twtt','radecho','utc','metadata','echotype','filesinbatch','numAvg');
            else
                save([outputfolder,outputname],'FlightNo','twtt','radecho','utc','echotype','filesinbatch','numAvg','SurfPwr','BedPwr','NoisePwr','SurfInds','BedInds','numbatches');
            end
        else
            warning(['Warning: Output File Already Exists, Batch Not Saved, Output File: ',outputname])
        end
    end
    
end
% Final human communication:
disp('Done Processing!')
disp('Generating Plots...')
PlotDiceQC(outputfolder,ProjDesc,processChan,dFileStr,filenametimeinds);