function [] = diceDecimateLevel1(newConfig)
% Parameters: newConfig is the name of the script that initializes all required variables
% if no argument is passed "diceProcessConfig.m" is run by default
% This code reads the sar Processed data, decimates and concatenates the
% files for bed picking etc.
% Processsing.
%   - Decimates the pulseCompressed data by a given factor
%   - Concatenates files to given period
%   - options for averaging and smoothing available before decimation
% Initialize the settings and variables
fprintf('************** diceDecimateLevel1.m ****************\n')

if nargin <1
    configScript = 'diceProcessConfig.m';
else
    configScript = newConfig;
end
run(configScript);
tic

maxnumTraces = ceil(fileLenTime/PRI/4/numAvg*1.1*filesperbatch)+bedPickTracesPerFile;
tmpecho = zeros(max(truncLenp1,truncLenp2),maxnumTraces);
tmpLat = zeros(1,maxnumTraces);
tmpLon =  zeros(1,maxnumTraces);
tmpSurfInds =  zeros(numSurfPicks,maxnumTraces);
tmpPlaneHeight =  zeros(1,maxnumTraces);
tmpFlightElev =  zeros(1,maxnumTraces);

tmputc = zeros(1,maxnumTraces);
prodLevel.name = 'level1a';
prodLevel.desc = 'bedPickDecimated';
truncLengths = {truncLenp1;truncLenp1;truncLenp2;truncLenp2};
if strcmp(bpSubsampleOption,'average')
    sigOpt = 1;
elseif strcmp(bpSubsampleOption,'decimate')
    sigOpt = 2;
elseif strcmp(bpSubsampleOption,'smooth')
    sigOpt = 3;
    bedpickFilt = ones(1,bedPickDecimate)/bedPickDecimate;
else
    error('diceDecimate:InvalidSubSampleOptions','Invalid "bpSubsampleOption" \n Check your setting')
end
%% Work
for ff = 1:numel(uniqueFlightNum)
    %% Check files and directories:
    FlightNo = uniqueFlight{ff};
%     flightDir = [level1apath,diceDir,filesep,stackDir,filesep,FlightNo,filesep,dataDir,filesep]; % output dir
    flightDir = [datarootpath,filesep,diceDir,filesep,level1aDiceDir,filesep,stackDir,filesep,FlightNo,filesep,dataDir,filesep];
%     dataPath = [level1path,diceDir,filesep,stackDir,filesep,FlightNo,filesep,dataDir,filesep];   % input dir
    dataPath = [datarootpath,filesep,diceDir,filesep,level1DiceDir,filesep,stackDir,filesep,FlightNo,filesep,dataDir,filesep];
    if exist(dataPath,'dir')==0
        warning('diceDecimateLevel1:noLevel1Dir','Level1 Flight %s Directory does not exists!',FlightNo);
        continue;
    end
    % Make output folder, if necessary:
    if exist(flightDir,'dir')==0
        mkdir(flightDir)
    elseif overwriteoutput == 1
        warning('diceDecimateLevel1:FlightDirExists','Flight %s Directory already exists!\n All existing files will be deleted!',FlightNo);
        delete([flightDir,'*',FlightNo,'*.mat']);
        delete([flightDir,'diceProcessConfig*.m']);
    else
        warning('diceDecimateLevel1:FlightDirExists','Flight %s Directory already exists - Flight will not be processed!\n Set "overwriteoutput = 1" to force processing',FlightNo);
        continue;
    end
    [~,configFilename,~] = fileparts(configScript);
    copyfile([configFilename,'.mat'],[flightDir,configFilename,'.mat'])
    fprintf('diceDecimateLevel1: Processing Flight %s\n',FlightNo)
    %% Find the files for each line process.
    for chan_num = 1:numel(processChan)
        level1Files=dir([dataPath,'*',processChanAttbr{processChan(chan_num)},'*.mat']);
        numfiles=numel(level1Files);
        if(numfiles == 0)
            warning(['decDecimateLevel1:NoFiles','no',processChanAttbr{processChan(chan_num)},'Files'],'No level0 %s files found!\nMoving on!',processChanAttbr{processChan(chan_num)})
            continue;
        else
            for ii = 1:numfiles
                fileNameFields = strsplit(level1Files(ii).name,fileFieldSep);
                try
                    old_fileformat = false;
                    level1Files(ii).utc = matlab2unixtime(datenum([fileNameFields{dateField},fileNameFields{timeField}],[outFileDateFormat,outFileTimeFormat]));
                catch
                    old_fileformat = true;
                    level1Files(ii).utc = matlab2unixtime(datenum([fileNameFields{dateField+1},fileNameFields{timeField+1}],[outFileDateFormat,outFileTimeFormat]));
                    warning('diceDecimateLevel1:oldFileNameFormat','File names are in old format, run dicePulseCompress to rename these files')
                    warning('off','diceDecimateLevel1:oldFileNameFormat');
                end
            end
            [~,sx] = sort([level1Files.utc]);
            level1Files = level1Files(sx);
        end
        fTime = [level1Files.utc];
        dTime = diff(fTime);
        batchind = find(dTime >filesperbatch*fileLenTime*1.1);
        numbatches = numel(batchind)+1;
        batchmin(1) = 1;
        if isempty(batchind)
            batchmax(1) = numfiles;
        else
            batchmax(1) = batchind(1);
            batchmin(numbatches) = batchind(end)+1;
            batchmax(numbatches) = numfiles;
        end
        for bb=2:numbatches-1
            batchmin(bb) = batchind(bb-1)+1;
            batchmax(bb) = batchind(bb);
        end
        if numbatches>1
            warning('diceDecimateLevel1:FilesNotContiguous','Files are not contiguous!');
        end
        minfNum=batchmin;
        maxfNum=batchmax;
        
        load([dataPath,level1Files(1).name],'twtt','numAvg')
        
        
        
        for batch=1:numbatches
            numRemainingTraces = 0;
            for ii = minfNum(batch):maxfNum(batch)
                load([dataPath,level1Files(ii).name],'radecho','echotimeutc','Campaign','echotype','FlightNo','lat','lon','hell','surfIndRad','planeHeight')
                fprintf('diceDecimateLevel1: Processing %s File %d of %d\n',processChanAttbr{processChan(chan_num)},ii,numfiles);
                numTraces = size(radecho,2);
                if(numRemainingTraces > 0)
                    tmpecho(1:truncLengths{processChan(chan_num)},1:numRemainingTraces+numTraces) = [tmpecho(1:truncLengths{processChan(chan_num)},1:numRemainingTraces),radecho];
                    tmputc(1:numRemainingTraces+numTraces) = [tmputc(1:numRemainingTraces),echotimeutc];
                    tmpLat(1:numRemainingTraces+numTraces) = [tmpLat(1:numRemainingTraces),lat];
                    tmpLon(1:numRemainingTraces+numTraces) = [tmpLon(1:numRemainingTraces),lon];
                    tmpSurfInds(:,1:numRemainingTraces+numTraces) = [tmpSurfInds(:,1:numRemainingTraces),surfIndRad];
                    tmpPlaneHeight(1:numRemainingTraces+numTraces) = [tmpPlaneHeight(1:numRemainingTraces),planeHeight];
                    tmpFlightElev(1:numRemainingTraces+numTraces) = [tmpFlightElev(1:numRemainingTraces),hell];
                    
                    numRemainingTraces = numRemainingTraces + numTraces;
                else
                    tmpecho(1:truncLengths{processChan(chan_num)},1:numTraces) = radecho;
                    tmputc(1:numTraces) = echotimeutc;
                    tmpLat(1:numTraces) = lat;
                    tmpLon(1:numTraces) = lon;
                    tmpSurfInds(:,1:numTraces) = surfIndRad;
                    tmpPlaneHeight(1:numTraces) = planeHeight;
                    tmpFlightElev(1:numTraces) = hell;
                    
                    numRemainingTraces = numTraces;
                end
                while(numRemainingTraces >= bedPickTracesPerFile)
                    if(sigOpt ==1)
                        radecho = reshape(mean(reshape(tmpecho(1:truncLengths{processChan(chan_num)},1:bedPickTracesPerFile),...
                            truncLengths{processChan(chan_num)},bedPickDecimate,bedPickTracesPerFile/bedPickDecimate),2),...
                            truncLengths{processChan(chan_num)},bedPickTracesPerFile/bedPickDecimate);
                        echotimeutc = mean(reshape(tmputc(1:bedPickTracesPerFile),bedPickDecimate,bedPickTracesPerFile/bedPickDecimate),1);
                        lat = mean(reshape(tmpLat(1:bedPickTracesPerFile),bedPickDecimate,bedPickTracesPerFile/bedPickDecimate),1);
                        lon = mean(reshape(tmpLon(1:bedPickTracesPerFile),bedPickDecimate,bedPickTracesPerFile/bedPickDecimate),1);
                        hell = mean(reshape(tmpFlightElev(1:bedPickTracesPerFile),bedPickDecimate,bedPickTracesPerFile/bedPickDecimate),1);
                        surfIndRad = reshape(round(mean(reshape(tmpSurfInds(1:numSurfPicks,1:bedPickTracesPerFile),...
                            numSurfPicks,bedPickDecimate,bedPickTracesPerFile/bedPickDecimate),2)),numSurfPicks,bedPickTracesPerFile/bedPickDecimate);
                        planeHeight = mean(reshape(tmpPlaneHeight(1:bedPickTracesPerFile),bedPickDecimate,bedPickTracesPerFile/bedPickDecimate),1);
                    elseif (sigOpt ==2)
                        radecho = tmpecho(1:truncLengths{processChan(chan_num)},1:bedPickDecimate:bedPickTracesPerFile);
                        echotimeutc = tmputc(1:bedPickDecimate:bedPickTracesPerFile);
                        lat = tmpLat(1:bedPickDecimate:bedPickTracesPerFile);
                        lon = tmpLon(1:bedPickDecimate:bedPickTracesPerFile);
                        hell = tmpFlightElev(1:bedPickDecimate:bedPickTracesPerFile);
                        surfIndRad = tmpSurfInds(:,1:bedPickDecimate:bedPickTracesPerFile);
                        planeHeight = tmpPlaneHeight(1:bedPickDecimate:bedPickTracesPerFile);
                    elseif (sigOpt ==3)
                        radecho = filter(bedpickFilt,1,tmpecho(1:truncLengths{processChan(chan_num)},1:bedPickTracesPerFile),[],2);
                        radecho = radecho(:,1:bedPickDecimate:end);
                        echotimeutc = tmputc(1:bedPickDecimate:bedPickTracesPerFile);
                        lat = tmpLat(1:bedPickDecimate:bedPickTracesPerFile);
                        lon = tmpLon(1:bedPickDecimate:bedPickTracesPerFile);
                        hell = tmpFlightElev(1:bedPickDecimate:bedPickTracesPerFile);
                        surfIndRad = tmpSurfInds(:,1:bedPickDecimate:bedPickTracesPerFile);
                        planeHeight = tmpPlaneHeight(1:bedPickDecimate:bedPickTracesPerFile);
                    end
                    
                    numRemainingTraces = numRemainingTraces - bedPickTracesPerFile;
                    
                    tmpecho(1:truncLengths{processChan(chan_num)},1:numRemainingTraces) = tmpecho(1:truncLengths{processChan(chan_num)},bedPickTracesPerFile+1:numRemainingTraces+bedPickTracesPerFile);
                    tmputc(1:numRemainingTraces) = tmputc(bedPickTracesPerFile+1:numRemainingTraces+bedPickTracesPerFile);
                    tmpLat(1:numRemainingTraces) = tmpLat(bedPickTracesPerFile+1:numRemainingTraces+bedPickTracesPerFile);
                    tmpLon(1:numRemainingTraces) = tmpLon(bedPickTracesPerFile+1:numRemainingTraces+bedPickTracesPerFile);
                    tmpFlightElev(1:numRemainingTraces) = tmpFlightElev(bedPickTracesPerFile+1:numRemainingTraces+bedPickTracesPerFile);
                    tmpSurfInds(:,1:numRemainingTraces) = tmpSurfInds(:,bedPickTracesPerFile+1:numRemainingTraces+bedPickTracesPerFile);
                    tmpPlaneHeight(1:numRemainingTraces) = tmpPlaneHeight(bedPickTracesPerFile+1:numRemainingTraces+bedPickTracesPerFile);
                    if old_fileformat
                        save([flightDir,fileNameFields{1},fileFieldSep,fileNameFields{2},fileFieldSep,datestr(unix2matlabtime(echotimeutc(1)),...
                            [outFileDateFormat,fileFieldSep,outFileTimeFormat]),fileFieldSep,fileNameFields{3},fileFieldSep,prodLevel.name,fileFieldSep,Sensor,'.mat'],...
                            'radecho','echotimeutc','Campaign','echotype','FlightNo','numAvg','twtt','lat','lon','hell','surfIndRad','planeHeight','prodLevel');
                    else
                        save([flightDir,fileNameFields{1},fileFieldSep,fileNameFields{2},fileFieldSep,datestr(unix2matlabtime(echotimeutc(1)),...
                            [outFileDateFormat,fileFieldSep,outFileTimeFormat]),fileFieldSep,fileNameFields{5},fileFieldSep,prodLevel.name,fileFieldSep,Sensor,'.mat'],...
                            'radecho','echotimeutc','Campaign','echotype','FlightNo','numAvg','twtt','lat','lon','hell','surfIndRad','planeHeight','prodLevel');
                    end
                end
            end
            if(numRemainingTraces > prfPreSar*60) % if there is more than a minute of data at the end of the batch save it
                numRemainingTraces = numRemainingTraces - mod(numRemainingTraces,bedPickDecimate);
                if(sigOpt ==1)
                    radecho = reshape(mean(reshape(tmpecho(1:truncLengths{processChan(chan_num)},1:numRemainingTraces),...
                        truncLengths{processChan(chan_num)},bedPickDecimate,numRemainingTraces/bedPickDecimate),2),...
                        truncLengths{processChan(chan_num)},numRemainingTraces/bedPickDecimate);
                    echotimeutc = mean(reshape(tmputc(1:numRemainingTraces),bedPickDecimate,numRemainingTraces/bedPickDecimate),1);
                    lat = mean(reshape(tmpLat(1:numRemainingTraces),bedPickDecimate,numRemainingTraces/bedPickDecimate),1);
                    lon = mean(reshape(tmpLon(1:numRemainingTraces),bedPickDecimate,numRemainingTraces/bedPickDecimate),1);
                    hell = mean(reshape(tmpFlightElev(1:numRemainingTraces),bedPickDecimate,numRemainingTraces/bedPickDecimate),1);
                    surfIndRad = reshape(round(mean(reshape(tmpSurfInds(1:numSurfPicks,1:numRemainingTraces),...
                        numSurfPicks,bedPickDecimate,numRemainingTraces/bedPickDecimate),2)),numSurfPicks,numRemainingTraces/bedPickDecimate);
                    planeHeight = mean(reshape(tmpPlaneHeight(1:numRemainingTraces),bedPickDecimate,numRemainingTraces/bedPickDecimate),1);
                elseif (sigOpt ==2)
                    radecho = tmpecho(1:truncLengths{processChan(chan_num)},1:bedPickDecimate:numRemainingTraces);
                    echotimeutc = tmputc(1:bedPickDecimate:numRemainingTraces);
                    lat = tmpLat(1:bedPickDecimate:numRemainingTraces);
                    lon = tmpLon(1:bedPickDecimate:numRemainingTraces);
                    hell = tmpFlightElev(1:bedPickDecimate:numRemainingTraces);
                    surfIndRad = tmpSurfInds(:,1:bedPickDecimate:numRemainingTraces);
                    planeHeight = tmpPlaneHeight(1:bedPickDecimate:numRemainingTraces);
                elseif (sigOpt ==3)
                    radecho = filter(bedpickFilt,1,tmpecho(1:truncLengths{processChan(chan_num)},1:numRemainingTraces),[],2);
                    radecho = radecho(:,1:bedPickDecimate:end);
                    echotimeutc = tmputc(1:bedPickDecimate:numRemainingTraces);
                    lat = tmpLat(1:bedPickDecimate:numRemainingTraces);
                    lon = tmpLon(1:bedPickDecimate:numRemainingTraces);
                    hell = tmpFlightElev(1:bedPickDecimate:numRemainingTraces);
                    surfIndRad = tmpSurfInds(:,1:bedPickDecimate:numRemainingTraces);
                    planeHeight = tmpPlaneHeight(1:bedPickDecimate:numRemainingTraces);
                end
                if old_fileformat
                    save([flightDir,fileNameFields{1},fileFieldSep,fileNameFields{2},fileFieldSep,datestr(unix2matlabtime(echotimeutc(1)),...
                        [outFileDateFormat,fileFieldSep,outFileTimeFormat]),fileFieldSep,fileNameFields{3},fileFieldSep,prodLevel.name,fileFieldSep,Sensor,'.mat'],...
                        'radecho','echotimeutc','Campaign','echotype','FlightNo','numAvg','twtt','lat','lon','hell','surfIndRad','planeHeight','prodLevel');
                else
                    save([flightDir,fileNameFields{1},fileFieldSep,fileNameFields{2},fileFieldSep,datestr(unix2matlabtime(echotimeutc(1)),...
                        [outFileDateFormat,fileFieldSep,outFileTimeFormat]),fileFieldSep,fileNameFields{5},fileFieldSep,prodLevel.name,fileFieldSep,Sensor,'mat'],...
                        'radecho','echotimeutc','Campaign','echotype','FlightNo','numAvg','twtt','lat','lon','hell','surfIndRad','planeHeight','prodLevel');
                end
            end
        end
    end
end
fprintf('diceDecimateLevel1: Done!\n')
toc
end