function [] = diceDecimateLevel2(newConfig,logFile)
% Parameters: newConfig is the name of the script that initializes all required variables
% if no argument is passed "diceProcessConfig.m" is run by default
% This code reads the sar Processed data, decimates and concatenates the
% files for bed picking etc.
% Processsing.
%   - Decimates the sar processed data by a given factor
%   - Concatenates files to given period
%   - options for averaging and smoothing available before decimation
% Initialize the settings and variables
try
fprintf('************** diceDecimateLevel2.m ****************\n')
if nargin <1
    configScript = 'diceProcessConfig.m';
    logFile = 'diceProcessLog.log';
    diary off;
    if exist(logFile,'file')
        warning('diceDecimateLevel2:existingLogFile','A logfile %s already exists!\n, Please cleanup the logs from %s for future processing',logFile,pwd);
        [~,logFileNameOnly,logFileExt] = fileparts(logFile);
        logFile = [logFileNameOnly,datestr(now,'_yyyymmdd_hhMMss'),logFileExt];
    end
    diary(logFile);
    localDiary = true;
elseif nargin <2
    configScript = newConfig;
    logFile = 'diceProcessLog.log';
    diary off;
    if exist(logFile,'file')
        warning('diceDecimateLevel2:existingLogFile','A logfile %s already exists!\n, Please cleanup the logs from %s for future processing',logFile,pwd);
        [~,logFileNameOnly,logFileExt] = fileparts(logFile);
        logFile = [logFileNameOnly,datestr(now,'_yyyymmdd_hhMMss'),logFileExt];
    end
    diary(logFile);
    localDiary = true;
else
    configScript = newConfig;
    localDiary = false;
end

run(configScript);
tic

maxnumTraces = ceil(fileLenTime/PRI/4/numAvg*1.1*filesperbatch)+bedPickTracesPerFile;
tmpecho = zeros(max(truncLenp1,truncLenp2),maxnumTraces);
tmpLat = zeros(1,maxnumTraces);
tmpLon =  zeros(1,maxnumTraces);
tmpSurfInds =  zeros(numSurfPicks,maxnumTraces);
tmpPlaneHeight =  zeros(1,maxnumTraces);
tmpFlightElev =  zeros(1,maxnumTraces);

tmputc = zeros(1,maxnumTraces);

truncLengths = {truncLenp1;truncLenp1;truncLenp2;truncLenp2};
if strcmp(bpSubsampleOption,'average')
    sigOpt = 1;
elseif strcmp(bpSubsampleOption,'decimate')
    sigOpt = 2;
elseif strcmp(bpSubsampleOption,'smooth')
    sigOpt = 3;
    bedpickFilt = ones(1,bedPickDecimate)/bedPickDecimate;
else
    error('diceDecimateLevel2:InvalidSubSampleOptions','Invalid "bpSubsampleOption" \n Check your setting')
end
%% Work
outfileCount = 0;
for ll = 1:numel(LineNum)   %% process each line separately
    %% Check files and directories:
    FlightNo = Flight{ll};
    bedPickDecimatedTime = now;
    configFileCopied = false;
    sarDataPath = [sensorRootPath,filesep,process2Dir,filesep,stackDir,filesep,LineNum{ll},filesep,dataDir,filesep];
    bedPickDecimatedPath = [sensorRootPath,filesep,process2Dir,filesep,stackDir,filesep,LineNum{ll},filesep,decimatedDir,filesep,dataDir,filesep];
    processLogPath = [sensorRootPath,filesep,process2Dir,filesep,stackDir,filesep,LineNum{ll},filesep,decimatedDir,filesep,logDir,filesep];

    if exist(sarDataPath,'dir')==0
        warning('diceDecimateLevel2:noLevel2Dir','Level2 Line %s Directory does not exists!',LineNum{ll});
        continue;
    else
        anyFiles=dir([sarDataPath,'*',LineNum{ll},'*',Sensor,'*.mat']);
        if isempty(anyFiles)
            warning('diceDecimateLevel2:noLevel2Files','Level2 Line %s Directory has no data to decimate!',LineNum{ll});
            continue;
        end
    end
        
    % Make output folder, if necessary:
    if exist(bedPickDecimatedPath,'dir')==0
        mkdir(bedPickDecimatedPath)
    elseif overwriteoutput == 1
        warning('diceDecimateLevel2:LineDirExists','Line %s Directory already exists!\n All existing files will be deleted!',LineNum{ll});
        delete([bedPickDecimatedPath,'*',LineNum{ll},'*.mat']);
        delete([bedPickDecimatedPath,'diceProcessConfig*.m']);
    else
        anyFiles=dir([bedPickDecimatedPath,'*',CampaignYear,'*',Sensor,'*.mat']);
        if ~isempty(anyFiles)
            warning('diceDecimateLevel2:LineDirExists','Line %s Directory already exists - Line will not be processed!\n Set "overwriteoutput = 1" or delete existing data to re-process',LineNum{ll});
            continue;
        end
    end

    fprintf('diceDecimateLevel2: Processing Line %s\n',LineNum{ll})
    %% Find the files for each line process.
    for chan_num = 1:numel(processChan)
        level2Files=dir([sarDataPath,'*',processChanAttbr{processChan(chan_num)},'*.mat']);
        numfiles=numel(level2Files);
        if(numfiles == 0)
            warning(['decDecimateLevel2:NoFiles','no',processChanAttbr{processChan(chan_num)},'Files'],'No level2 %s files found!\nMoving on!',processChanAttbr{processChan(chan_num)})
            continue;
        else
            for ii = 1:numfiles
                fileNameFields = strsplit(level2Files(ii).name,fileFieldSep);
                try
                    old_fileformat = false;
                    level2Files(ii).utc = matlab2unixtime(datenum([fileNameFields{dateField},fileNameFields{timeField}],[outFileDateFormat,outFileTimeFormat]));
                catch
                    old_fileformat = true;
                    level2Files(ii).utc = matlab2unixtime(datenum([fileNameFields{dateField+1},fileNameFields{timeField+1}],[outFileDateFormat,outFileTimeFormat]));
                    warning('diceDecimateLevel2:oldFileNameFormat','File names are in old format')
                    warning('off','diceDecimateLevel2:oldFileNameFormat');
                end
            end
            [~,sx] = sort([level2Files.utc]);
            level2Files = level2Files(sx);
        end
        fTime = [level2Files.utc];
        dTime = diff(fTime);
        batchind = find(dTime >sarTracesPerFile/prfPreSar*1.1);
        numbatches = numel(batchind)+1;
        batchmin(1) = 1;
        if isempty(batchind)
            batchmax(1) = numfiles;
        else
            batchmax(1) = batchind(1);
            batchmin(numbatches) = batchind(end)+1;
            batchmax(numbatches) = numfiles;
        end
        for bb=2:numbatches-1
            batchmin(bb) = batchind(bb-1)+1;
            batchmax(bb) = batchind(bb);
        end
        if numbatches>1
            warning('diceDecimateLevel2:FilesNotContiguous','Files are not contiguous!');
        end
        minfNum=batchmin;
        maxfNum=batchmax;
        
        load([sarDataPath,level2Files(minfNum(1)).name],'TxInd','twttTx','cAir','cIce','fs','nIce','tSamp')
        echotype = processChanAttbr{processChan(chan_num)};
        Time = twttTx;
        
        for batch=1:numbatches
            numRemainingTraces = 0;
            for ii = minfNum(batch):maxfNum(batch)
                load([sarDataPath,level2Files(ii).name],'CG','Lat','Lon','FlightElev','twttTx','PlaneHeight','SurfInds','echoTimeUTC','prodLevel')
                fprintf('diceDecimateLevel2: Processing %s File %d of %d\n',processChanAttbr{processChan(chan_num)},ii,numfiles);
                CGnumTraces = size(CG,2);
                if(numRemainingTraces > 0)
                    tmpecho(1:truncLengths{processChan(chan_num)},1:numRemainingTraces+CGnumTraces) = [tmpecho(1:truncLengths{processChan(chan_num)},1:numRemainingTraces),CG];
                    tmputc(1:numRemainingTraces+CGnumTraces) = [tmputc(1:numRemainingTraces),echoTimeUTC];
                    tmpLat(1:numRemainingTraces+CGnumTraces) = [tmpLat(1:numRemainingTraces),Lat];
                    tmpLon(1:numRemainingTraces+CGnumTraces) = [tmpLon(1:numRemainingTraces),Lon];
                    tmpSurfInds(:,1:numRemainingTraces+CGnumTraces) = [tmpSurfInds(:,1:numRemainingTraces),SurfInds];
                    tmpPlaneHeight(1:numRemainingTraces+CGnumTraces) = [tmpPlaneHeight(1:numRemainingTraces),PlaneHeight];
                    tmpFlightElev(1:numRemainingTraces+CGnumTraces) = [tmpFlightElev(1:numRemainingTraces),FlightElev];
                    
                    numRemainingTraces = numRemainingTraces + CGnumTraces;
                else
                    tmpecho(1:truncLengths{processChan(chan_num)},1:CGnumTraces) = CG;
                    tmputc(1:CGnumTraces) = echoTimeUTC;
                    tmpLat(1:CGnumTraces) = Lat;
                    tmpLon(1:CGnumTraces) = Lon;
                    tmpSurfInds(:,1:CGnumTraces) = SurfInds;
                    tmpPlaneHeight(1:CGnumTraces) = PlaneHeight;
                    tmpFlightElev(1:CGnumTraces) = FlightElev;
                    
                    numRemainingTraces = CGnumTraces;
                end
                while(numRemainingTraces >= bedPickTracesPerFile)
                    if(sigOpt ==1)
                        Data = reshape(mean(reshape(tmpecho(1:truncLengths{processChan(chan_num)},1:bedPickTracesPerFile),...
                            truncLengths{processChan(chan_num)},bedPickDecimate,bedPickTracesPerFile/bedPickDecimate),2),...
                            truncLengths{processChan(chan_num)},bedPickTracesPerFile/bedPickDecimate);
                        UTC_time = mean(reshape(tmputc(1:bedPickTracesPerFile),bedPickDecimate,bedPickTracesPerFile/bedPickDecimate),1);
                        Latitude = mean(reshape(tmpLat(1:bedPickTracesPerFile),bedPickDecimate,bedPickTracesPerFile/bedPickDecimate),1);
                        Longitude = mean(reshape(tmpLon(1:bedPickTracesPerFile),bedPickDecimate,bedPickTracesPerFile/bedPickDecimate),1);
                        Elevation = mean(reshape(tmpFlightElev(1:bedPickTracesPerFile),bedPickDecimate,bedPickTracesPerFile/bedPickDecimate),1);
                        layerInd = reshape(round(mean(reshape(tmpSurfInds(1:numSurfPicks,1:bedPickTracesPerFile),...
                            numSurfPicks,bedPickDecimate,bedPickTracesPerFile/bedPickDecimate),2)),numSurfPicks,bedPickTracesPerFile/bedPickDecimate);
                        planeHeight = mean(reshape(tmpPlaneHeight(1:bedPickTracesPerFile),bedPickDecimate,bedPickTracesPerFile/bedPickDecimate),1);
                    elseif (sigOpt ==2)
                        Data = tmpecho(1:truncLengths{processChan(chan_num)},1:bedPickDecimate:bedPickTracesPerFile);
                        UTC_time = tmputc(1:bedPickDecimate:bedPickTracesPerFile);
                        Latitude = tmpLat(1:bedPickDecimate:bedPickTracesPerFile);
                        Longitude = tmpLon(1:bedPickDecimate:bedPickTracesPerFile);
                        Elevation = tmpFlightElev(1:bedPickDecimate:bedPickTracesPerFile);
                        layerInd = tmpSurfInds(:,1:bedPickDecimate:bedPickTracesPerFile);
                        planeHeight = tmpPlaneHeight(1:bedPickDecimate:bedPickTracesPerFile);
                    elseif (sigOpt ==3)
                        Data = filter(bedpickFilt,1,tmpecho(1:truncLengths{processChan(chan_num)},1:bedPickTracesPerFile),[],2);
                        Data = Data(:,1:bedPickDecimate:end);
                        UTC_time = tmputc(1:bedPickDecimate:bedPickTracesPerFile);
                        Latitude = tmpLat(1:bedPickDecimate:bedPickTracesPerFile);
                        Longitude = tmpLon(1:bedPickDecimate:bedPickTracesPerFile);
                        Elevation = tmpFlightElev(1:bedPickDecimate:bedPickTracesPerFile);
                        layerInd = tmpSurfInds(:,1:bedPickDecimate:bedPickTracesPerFile);
                        planeHeight = tmpPlaneHeight(1:bedPickDecimate:bedPickTracesPerFile);
                    end
                    
                    Surface = Time(min(layerInd,[],1));
                    if iscolumn(Surface)
                        Surface = Surface';
                    end
                    Surface(Surface> Time(end-2)) = NaN;
                    Surf_Elev = Elevation - Surface*cAir/2;
                    
                    numRemainingTraces = numRemainingTraces - bedPickTracesPerFile;
                    
                    tmpecho(1:truncLengths{processChan(chan_num)},1:numRemainingTraces) = tmpecho(1:truncLengths{processChan(chan_num)},bedPickTracesPerFile+1:numRemainingTraces+bedPickTracesPerFile);
                    tmputc(1:numRemainingTraces) = tmputc(bedPickTracesPerFile+1:numRemainingTraces+bedPickTracesPerFile);
                    tmpLat(1:numRemainingTraces) = tmpLat(bedPickTracesPerFile+1:numRemainingTraces+bedPickTracesPerFile);
                    tmpLon(1:numRemainingTraces) = tmpLon(bedPickTracesPerFile+1:numRemainingTraces+bedPickTracesPerFile);
                    tmpFlightElev(1:numRemainingTraces) = tmpFlightElev(bedPickTracesPerFile+1:numRemainingTraces+bedPickTracesPerFile);
                    tmpSurfInds(1:numRemainingTraces) = tmpSurfInds(bedPickTracesPerFile+1:numRemainingTraces+bedPickTracesPerFile);
                    tmpPlaneHeight(1:numRemainingTraces) = tmpPlaneHeight(bedPickTracesPerFile+1:numRemainingTraces+bedPickTracesPerFile);
                    outfileCount = outfileCount + 1;
                    
                    if ~iscell(prodLevel.desc)
                        oldProdLevel = prodLevel;
                        clear prodLevel;
                        prodLevel.name = oldProLevel.name;
                        prodLevel.desc = {oldProdLevel.desc};
                        prodLevel.time = {'?'};
                    end
                    prodLevel.name = 'level2a';
                    prodLevel.desc{numel(prodLevel.desc)+1} = 'bedPickDecimated';
                    prodLevel.time{numel(prodLevel.time)+1} = bedPickDecimatedTime;
                    
                    if old_fileformat
                        save([bedPickDecimatedPath,fileNameFields{1},fileFieldSep,fileNameFields{2},fileFieldSep,datestr(unix2matlabtime(UTC_time(1)),...
                            [outFileDateFormat,fileFieldSep,outFileTimeFormat]),fileFieldSep,fileNameFields{3},fileFieldSep,prodLevel.name,fileFieldSep,Sensor,fileFieldSep,num2str(outfileCount),'.mat'],...
                            'Data','UTC_time','Time','Campaign','echotype','FlightNo','numAvg','Latitude','Longitude','Elevation','layerInd','Surface','Surf_Elev','prodLevel','bedPickDecimatedTime');
                    else
                        save([bedPickDecimatedPath,fileNameFields{1},fileFieldSep,fileNameFields{2},fileFieldSep,datestr(unix2matlabtime(UTC_time(1)),...
                            [outFileDateFormat,fileFieldSep,outFileTimeFormat]),fileFieldSep,fileNameFields{5},fileFieldSep,prodLevel.name,fileFieldSep,Sensor,fileFieldSep,num2str(outfileCount),'.mat'],...
                            'Data','UTC_time','Time','Campaign','echotype','FlightNo','numAvg','Latitude','Longitude','Elevation','layerInd','Surface','Surf_Elev','prodLevel','bedPickDecimatedTime');
                    end                 
                        
                    if ~configFileCopied
                        [~,configFilename,~] = fileparts(configScript);
                        copyfile([configFilename,'.mat'],[bedPickDecimatedPath,configFilename,'.mat'])
                        configFileCopied = true;
                    end
                end
            end
            if(numRemainingTraces> prfPreSar*60) % if there is more than a minute of data at the end of the batch save it
                numRemainingTraces = numRemainingTraces - mod(numRemainingTraces,bedPickDecimate);
                if(sigOpt ==1)
                    Data = reshape(mean(reshape(tmpecho(1:truncLengths{processChan(chan_num)},1:numRemainingTraces),...
                        truncLengths{processChan(chan_num)},bedPickDecimate,numRemainingTraces/bedPickDecimate),2),...
                        truncLengths{processChan(chan_num)},numRemainingTraces/bedPickDecimate);
                    UTC_time = mean(reshape(tmputc(1:numRemainingTraces),bedPickDecimate,numRemainingTraces/bedPickDecimate),1);
                    Latitude = mean(reshape(tmpLat(1:numRemainingTraces),bedPickDecimate,numRemainingTraces/bedPickDecimate),1);
                    Longitude = mean(reshape(tmpLon(1:numRemainingTraces),bedPickDecimate,numRemainingTraces/bedPickDecimate),1);
                    Elevation = mean(reshape(tmpFlightElev(1:numRemainingTraces),bedPickDecimate,numRemainingTraces/bedPickDecimate),1);
                    layerInd = reshape(round(mean(reshape(tmpSurfInds(1:numSurfPicks,1:numRemainingTraces),...
                        numSurfPicks,bedPickDecimate,numRemainingTraces/bedPickDecimate),2)),numSurfPicks,numRemainingTraces/bedPickDecimate);
                    planeHeight = mean(reshape(tmpPlaneHeight(1:numRemainingTraces),bedPickDecimate,numRemainingTraces/bedPickDecimate),1);
                elseif (sigOpt ==2)
                    Data = tmpecho(1:truncLengths{processChan(chan_num)},1:bedPickDecimate:numRemainingTraces);
                    UTC_time = tmputc(1:bedPickDecimate:numRemainingTraces);
                    Latitude = tmpLat(1:bedPickDecimate:numRemainingTraces);
                    Longitude = tmpLon(1:bedPickDecimate:numRemainingTraces);
                    Elevation = tmpFlightElev(1:bedPickDecimate:numRemainingTraces);
                    layerInd = tmpSurfInds(:,1:bedPickDecimate:numRemainingTraces);
                    planeHeight = tmpPlaneHeight(1:bedPickDecimate:numRemainingTraces);
                elseif (sigOpt ==3)
                    Data = filter(bedpickFilt,1,tmpecho(1:truncLengths{processChan(chan_num)},1:numRemainingTraces),[],2);
                    Data = Data(:,1:bedPickDecimate:end);
                    UTC_time = tmputc(1:bedPickDecimate:numRemainingTraces);
                    Latitude = tmpLat(1:bedPickDecimate:numRemainingTraces);
                    Longitude = tmpLon(1:bedPickDecimate:numRemainingTraces);
                    Elevation = tmpFlightElev(1:bedPickDecimate:numRemainingTraces);
                    layerInd = tmpSurfInds(:,1:bedPickDecimate:numRemainingTraces);
                    planeHeight = tmpPlaneHeight(1:bedPickDecimate:numRemainingTraces);
                end
                    Surface = Time(min(layerInd,[],1));
                    if iscolumn(Surface)
                        Surface = Surface';
                    end
                    Surface(Surface> Time(end-2)) = NaN;
                    Surf_Elev = Elevation - Surface*cAir/2;
                    
                    outfileCount = outfileCount + 1;                  
                    
                if old_fileformat
                    save([bedPickDecimatedPath,fileNameFields{1},fileFieldSep,fileNameFields{2},fileFieldSep,datestr(unix2matlabtime(UTC_time(1)),...
                        [outFileDateFormat,fileFieldSep,outFileTimeFormat]),fileFieldSep,fileNameFields{3},fileFieldSep,prodLevel.name,fileFieldSep,Sensor,fileFieldSep,num2str(outfileCount),'.mat'],...
                        'Data','UTC_time','Time','Campaign','echotype','FlightNo','numAvg','Latitude','Longitude','Elevation','layerInd','Surface','Surf_Elev','prodLevel','bedPickDecimatedTime');
                else
                    save([bedPickDecimatedPath,fileNameFields{1},fileFieldSep,fileNameFields{2},fileFieldSep,datestr(unix2matlabtime(UTC_time(1)),...
                        [outFileDateFormat,fileFieldSep,outFileTimeFormat]),fileFieldSep,fileNameFields{5},fileFieldSep,prodLevel.name,fileFieldSep,Sensor,fileFieldSep,num2str(outfileCount),'.mat'],...
                        'Data','UTC_time','Time','Campaign','echotype','FlightNo','numAvg','Latitude','Longitude','Elevation','layerInd','Surface','Surf_Elev','prodLevel','bedPickDecimatedTime');
                end
                
                if ~configFileCopied
                    [~,configFilename,~] = fileparts(configScript);
                    copyfile([configFilename,'.mat'],[bedPickDecimatedPath,configFilename,'.mat'])
                    configFileCopied = true;
                end
                
            end
        end
    end
    warning('on','all');
    diary off;
    if(configFileCopied)    % if yes, then the line was most likely processed
        [~,logFileNameOnly,logFileExt] = fileparts(logFile);
        if ~exist(processLogPath,'dir')
            mkdir(processLogPath);
        end
        movefile(logFile,[processLogPath,logFileNameOnly,fileFieldSep,'decimate',datestr(bedPickDecimatedTime,'_yyyymmdd_hhMMss'),logFileExt]);
        [~,configFileNameOnly,~] = fileparts(configScript);
        copyfile(configScript,[processLogPath,configFileNameOnly,fileFieldSep,'decimate',datestr(bedPickDecimatedTime,'_yyyymmdd_hhMMss'),'.m.log']);
    else % delete existing diary and start a fresh one
        [~,~,logext] = fileparts(logFile);
        if strcmpi(logext,'.log') %% just making sure it's a .log file before deletion
            delete(logFile);
        end
    end
    diary(logFile);
end
    if localDiary % if diary was locally started - delete it
        if strcmpi(get(0,'Diary'),'on')
            diary off;
            [~,~,logext] = fileparts(logFile);
            if strcmpi(logext,'.log') %% just making sure it's a .log file before deletion
                delete(logFile);
            end
        end
    end
fprintf('diceDecimateLevel2: Done!\n')
toc
catch errMsg
    fprintf('diceDecimateLevel2: Encountered an error\n');
    fprintf('%s\n',errMsg.message);
    fprintf('%s\n',errMsg.identifier);
    for jj = 1:numel(errMsg.stack)
        fprintf('In Function %s, Line %d\n',errMsg.stack(jj).name,errMsg.stack(jj).line)
    end
    warning('on','all');
    % move the log file
    diary off;
    if exist(logFile,'file') && exist('processLogPath','var')
        [~,logFileNameOnly,logFileExt] = fileparts(logFile);
        if ~exist(processLogPath,'dir')
            mkdir(processLogPath);
        end
        movefile(logFile,[processLogPath,logFileNameOnly,datestr(now,'_yyyymmdd_hhMMss'),logFileExt]);
    end
    if debugEnable
        fprintf('diceDecimateLevel2: Halting for debugging\n')
        keyboard
    end
end
end
