function [] = diceGeoLocate(newConfig,logFile)
% Parameters: newConfig is the name of the script that initializes all required variables
% if no argument is passed "diceProcessConfig.m" is run by default
% This code reads the pulse Compressed data and gets GeoLocates it for further
% Processsing.
%   - Reads GPS file, converts to mat if needed and interpolates Nav data
%     to match radar echo time
%   - Append nav data to existing level1 file
% Initialize the settings and variables
try
fprintf('************** diceGeoLocate.m ****************\n')

if nargin <1
    configScript = 'diceProcessConfig.m';
    logFile = 'diceProcessLog.log';
    diary off;
    if exist(logFile,'file')
        warning('diceGeoLocate:existingLogFile','A logfile %s already exists!\n, Please cleanup the logs from %s for future processing',logFile,pwd);
        [~,logFileNameOnly,logFileExt] = fileparts(logFile);
        logFile = [logFileNameOnly,datestr(now,'_yyyymmdd_hhMMss'),logFileExt];
    end
    diary(logFile);
    localDiary = true;
elseif nargin <2
    configScript = newConfig;
    logFile = 'diceProcessLog.log';
    diary off;
    if exist(logFile,'file')
        warning('diceGeoLocate:existingLogFile','A logfile %s already exists!\n, Please cleanup the logs from %s for future processing',logFile,pwd);
        [~,logFileNameOnly,logFileExt] = fileparts(logFile);
        logFile = [logFileNameOnly,datestr(now,'_yyyymmdd_hhMMss'),logFileExt];
    end
    diary(logFile);
    localDiary = true;
else
    configScript = newConfig;
    localDiary = false;
end

run(configScript);
tic

%% Work

for ff = 1:numel(uniqueFlightNum)
    %% Check files and directories:
    FlightNo = uniqueFlight{ff};
    geoLocatedTime = now;
    pulseCompressedDataPath = [sensorRootPath,filesep,process1Dir,filesep,stackDir,filesep,FlightNo,filesep,dataDir,filesep];
    processLogPath = [sensorRootPath,filesep,process1Dir,filesep,stackDir,filesep,FlightNo,filesep,logDir,filesep];
    if exist(pulseCompressedDataPath,'dir')==0
        warning('diceGeoLocate:noFlightDir','Level1 Flight %s Directory does not exists!',FlightNo);
        continue;
    end
    
    fprintf('diceGeoLocate: Processing Flight %s\n',FlightNo)

            %% load the GPS file for this flight
        gpsInpath = [gpsDataPath,filesep,trajectoryDir,filesep,FlightNo,filesep,txtDir,filesep];
        gpsFileList = dir([gpsInpath,'*',gpsProd,'*.txt']);
        
        % Take care of the case where there are gps files in parts
        try
            if numel(gpsFileList) == 1
                gpsFile = [gpsInpath,gpsFileList(1).name];
            elseif numel(gpsFileList) >1
                gpsFile = combineGPStxtFileParts(gpsInpath,gpsProd,overwriteGPSmatFile);
            else
                warning('diceGeoLocate:NoGPSFiles','No %s files available for this flight\n It will not be processed',gpsProd);
                continue;
            end
        catch errMsg
            fprintf('%s\n',errMsg.message);
            fprintf('%s\n',errMsg.identifier);
            for ejj = 1:numel(errMsg.stack)
                fprintf('In Function %s, Line %d\n',errMsg.stack(ejj).name,errMsg.stack(ejj).line)
            end
            warning('diceGeoLocate:failedToGetGPSData','Failed to get GPS data for %s\n It will not be processed',gpsProd,FlightNo);
            continue;
        end
        
        gpsMatFile = convertGPStxt2mat(gpsFile,25,inf,[],overwriteGPSmatFile);
        load(gpsMatFile,'Lat','Lon','Hell','Pitch','Roll','Heading','GPSUTC');
        
        configFileCopied = false;
    
    %% Find the files for each line process.
    for chan_num = 1:numel(processChan)
        tmpFiles = dir([pulseCompressedDataPath,'*',processChanAttbr{processChan(chan_num)},'*.mat',tmpExt]);
        if ~isempty(tmpFiles)
            fprintf('diceGeoLocate: Recovering corrupted mat files from previous run...\n');
            for tt = 1:numel(tmpFiles)
                copyfile([pulseCompressedDataPath,tmpFiles(tt).name],[pulseCompressedDataPath,tmpFiles(tt).name(1:end-numel(tmpExt))])
                delete([pulseCompressedDataPath,tmpFiles(tt).name]);
            end
        end
        
        level1Files=dir([pulseCompressedDataPath,'*',processChanAttbr{processChan(chan_num)},'*.mat']);
        numfiles=numel(level1Files);
        if(numfiles == 0)
            warning(['diceGeoLocate:','no',processChanAttbr{processChan(chan_num)},'Files'],'No level1 %s files found!',processChanAttbr{processChan(chan_num)})
            continue;
        else
            for ii = 1:numfiles
                fileNameFields = strsplit(level1Files(ii).name,fileFieldSep);
                try
                     level1Files(ii).utc = matlab2unixtime(datenum([fileNameFields{dateField},fileNameFields{timeField}],[outFileDateFormat,outFileTimeFormat]));
                catch
                    level1Files(ii).utc = matlab2unixtime(datenum([fileNameFields{dateField+1},fileNameFields{timeField+1}],[outFileDateFormat,outFileTimeFormat]));
                    warning('diceGeoLocate:oldFileNameFormat','File names are in old format. Run dicePulseCompress to rename the files')
                    warning('off','diceGeoLocate:oldFileNameFormat');
                end
            end
            [~,sx] = sort([level1Files.utc]);
            level1Files = level1Files(sx);
        end
        for ii = 1:numfiles
            varlist = whos('-file',[pulseCompressedDataPath,level1Files(ii).name]);
            fprintf('diceGeoLocate: Processing %s File %d of %d\n',processChanAttbr{processChan(chan_num)},ii,numfiles);
            if all(ismember({'lat';'lon';'hell';'pitch';'roll';'heading'},{varlist.name}))
                if(~forceGeolocate)
                    warning('diceGeoLocate:alreadyGeoLocated',['One or more files are already geo-located\n',...
                    'Set "forceGeolocate = true" to reprocess these files']);
                    warning('off','diceGeoLocate:alreadyGeoLocated');
                    continue;
                end
            end
            if ~configFileCopied
                [~,configFilename,~] = fileparts(configScript);
                copyfile([configFilename,'.mat'],[pulseCompressedDataPath,configFilename,'.mat'])
                configFileCopied = true;
            end
            
            load([pulseCompressedDataPath,level1Files(ii).name],'echotimeutc','prodLevel')
            % Interpolate navigation information he-re and save with the radar data
            lat     = interp1(GPSUTC,Lat,echotimeutc,'linear');
            lon     = interpDegWithJumps(GPSUTC,Lon,echotimeutc,'linear');
            hell    = interp1(GPSUTC,Hell,echotimeutc,'linear');
            pitch   = interpDegWithJumps(GPSUTC,Pitch,echotimeutc,'linear');
            roll    = interpDegWithJumps(GPSUTC,Roll,echotimeutc,'linear');
            heading = interpDegWithJumps(GPSUTC,Heading,echotimeutc,'linear');
            
            % backward compatibility
            if ~iscell(prodLevel.desc)
                oldProdLevel = prodLevel;
                clear prodLevel;
                prodLevel.name = oldProLevel.name;
                prodLevel.desc = {oldProdLevel.desc};
                prodLevel.time = {'?'};
            end
            prodLevel.name = 'level1';
            prodLevel.desc{numel(prodLevel.desc)+1} = 'geoLocated';
            prodLevel.time{numel(prodLevel.time)+1} = datestr(geoLocatedTime);
            
            if save_append_backup
                copyfile([pulseCompressedDataPath,level1Files(ii).name],[pulseCompressedDataPath,level1Files(ii).name,tmpExt]);
            end
            save([pulseCompressedDataPath,level1Files(ii).name],'lat','lon','hell','pitch','roll','heading','prodLevel','geoLocatedTime','-append');
            if save_append_backup
                delete([pulseCompressedDataPath,level1Files(ii).name,tmpExt]);
            end
        end
    end
    warning('on','all');
    diary off;
    if(configFileCopied)    % if true, then the flight was most likely processed
        [~,logFileNameOnly,logFileExt] = fileparts(logFile);
        if ~exist(processLogPath,'dir')
            mkdir(processLogPath);
        end
        movefile(logFile,[processLogPath,logFileNameOnly,fileFieldSep,'geolocate',datestr(geoLocatedTime,'_yyyymmdd_hhMMss'),logFileExt]);
        if runCount == 1
            [~,configFileNameOnly,~] = fileparts(configScript);
            copyfile(configScript,[processLogPath,configFileNameOnly,fileFieldSep,'geolocate',datestr(geoLocatedTime,'_yyyymmdd_hhMMss'),'.m.log']);
        end
    else % delete existing diary and start a fresh one
        [~,~,logext] = fileparts(logFile);
        if strcmpi(logext,'.log') %% just making sure it's a .log file before deletion
            delete(logFile);
        end
    end
    diary(logFile);
end
    if localDiary % if diary was locally started - delete it
        if strcmpi(get(0,'Diary'),'on')
            diary off;
            [~,~,logext] = fileparts(logFile);
            if strcmpi(logext,'.log') %% just making sure it's a .log file before deletion
                delete(logFile);
            end
        end
    end
fprintf('diceGeoLocate: Done!\n')
toc
catch errMsg
    fprintf('diceGeoLocate: Encountered an error\n');
    fprintf('%s\n',errMsg.message);
    fprintf('%s\n',errMsg.identifier);
    for jj = 1:numel(errMsg.stack)
        fprintf('In Function %s, Line %d\n',errMsg.stack(jj).name,errMsg.stack(jj).line)
    end
    warning('on','all');
    % move the log file
    diary off;
    if exist(logFile,'file') && exist('processLogPath','var')
        [~,logFileNameOnly,logFileExt] = fileparts(logFile);
        if ~exist(processLogPath,'dir')
            mkdir(processLogPath);
        end
        movefile(logFile,[processLogPath,logFileNameOnly,'_crash',datestr(now,'_yyyymmdd_hhMMss'),logFileExt]);
    end
    if debugEnable
        fprintf('diceGeoLocate: Halting for debugging\n')
        keyboard
    end
end
end