function [] = dicePickSurf(newConfig,logFile)
% Parameters: newConfig is the name of the script that initializes all required variables
% if no argument is passed "diceProcessConfig.m" is run by default
% This code reads the geoLocated data and processed further to get it ready
% for SAR
%   - Find surface from each radar trace
%   - Apply smoothing across files before picking surface
try
    fprintf('************** dicePickSurf.m ****************\n')
    if nargin <1
        configScript = 'diceProcessConfig.m';
        logFile = 'diceProcessLog.log';
        diary off;
        if exist(logFile,'file')
            warning('dicePickSurf:existingLogFile','A logfile %s already exists!\n, Please cleanup the logs from %s for future processing',logFile,pwd);
            [~,logFileNameOnly,logFileExt] = fileparts(logFile);
            logFile = [logFileNameOnly,datestr(now,'_yyyymmdd_hhMMss'),logFileExt];
        end
        diary(logFile);
        localDiary = true;
    elseif nargin <2
        configScript = newConfig;
        logFile = 'diceProcessLog.log';
        diary off;
        if exist(logFile,'file')
            warning('dicePickSurf:existingLogFile','A logfile %s already exists!\n, Please cleanup the logs from %s for future processing',logFile,pwd);
            [~,logFileNameOnly,logFileExt] = fileparts(logFile);
            logFile = [logFileNameOnly,datestr(now,'_yyyymmdd_hhMMss'),logFileExt];
        end
        diary(logFile);
        localDiary = true;
    else
        configScript = newConfig;
        localDiary = false;
    end
    
    run(configScript);
    configParamsCurr = configParams;
    tic
    
    maxnumTraces = ceil(fileLenTime*prfPreSar*filesperbatch*1.1)*3;
    
    tempInd = zeros(numSurfPicks,ceil(fileLenTime*prfPreSar*filesperbatch*1.1)+surfFilterWin*4);
    tmptxon = false(1,maxnumTraces);
    tmpecho = zeros(max(truncLenp1,truncLenp2),maxnumTraces);
    truncLengths = {truncLenp1;truncLenp1;truncLenp2;truncLenp2};
    if exist('surfacePickWindowType','var')
        if strcmpi(surfacePickWindowType,'adaptive')
            fixedInds = false;
        else
            fixedInds = true;
        end
        fprintf('dicePickSurf: Using %s window for picking surface ...\n',surfacePickWindowType)
    else
        fixedInds = true;
        fprintf('dicePickSurf: Using a fixed window for picking surface ...\n')
    end
    
    %% Future work to incorporate external surface
    % if exist('surfaceFile','var')
    %     if (exist(surfaceFile,'file')==2)
    %         [pathstr,fname,fext] = fileparts(surface_file);
    %         if (strcmp(fext,'.mat')==0) % file entered is not a mat file need to process.
    %             surface_matfile = importFlightElev(surface_file, 30,inf);
    %         else
    %             surface_matfile = surface_file;
    %         end
    %     end
    %     load(surface_matfile,'GPSUTCSurf','SurfaceHell');
    %     extSurf = 1;
    % end
    
    
    %% Load all the files in the level1 directory
    for ff = 1:numel(uniqueFlightNum)
        %% Check files and directories:
        FlightNo = uniqueFlight{ff};
        surfPickedTime = now;
        configFileCopied = false;
        geoLocatedDataPath = [sensorRootPath,filesep,process1Dir,filesep,stackDir,filesep,FlightNo,filesep,dataDir,filesep];
        processLogPath = [sensorRootPath,filesep,process1Dir,filesep,stackDir,filesep,FlightNo,filesep,logDir,filesep];
        if exist(geoLocatedDataPath,'dir')==0
            warning('dicePickSurf:noLevel1Dir','Level1 Flight %s Directory does not exists!',FlightNo);
            continue;
        end
        
        pcFlist = dir([geoLocatedDataPath,'*',configFnameRoot,'*.mat']);
        if numel(pcFlist)>0
            [~,sx] = sort([pcFlist.datenum]);
            pcFlist = pcFlist(sx);
            fprintf('dicePickSurf: Checking existing configuration data...\n')
            load([pcFlist(end).name],'configParams');
            if isequal(configParamsCurr.pickSurf,configParams.pickSurf)
                configmatch = true;
            else
                configmatch = false;
            end
        end
        
        fprintf('dicePickSurf: Processing Flight %s\n',FlightNo)
        
        for chan_num = 1:numel(processChan)
            tmpFiles = dir([geoLocatedDataPath,'*',processChanAttbr{processChan(chan_num)},'*.mat',tmpExt]);
            if ~isempty(tmpFiles)
                fprintf('dicePickSurf: Recovering corrupted mat files from previous run...\n');
                for tt = 1:numel(tmpFiles)
                    copyfile([geoLocatedDataPath,tmpFiles(tt).name],[geoLocatedDataPath,tmpFiles(tt).name(1:end-numel(tmpExt))])
                    delete([geoLocatedDataPath,tmpFiles(tt).name]);
                end
            end
            level1Files{processChan(chan_num)}=dir([geoLocatedDataPath,['*',processChanAttbr{processChan(chan_num)},'*.mat']]);
            numfiles=numel(level1Files{processChan(chan_num)});
            if(numfiles == 0)
                warning(['dicePickSurf:','no',processChanAttbr{processChan(chan_num)},'Files'],'No level1 %s files found!',processChanAttbr{processChan(chan_num)})
            else
                for ii = 1:numfiles
                    fileNameFields = strsplit(level1Files{processChan(chan_num)}(ii).name,fileFieldSep);
                    try
                        level1Files{processChan(chan_num)}(ii).utc = matlab2unixtime(datenum([fileNameFields{dateField},fileNameFields{timeField}],[outFileDateFormat,outFileTimeFormat]));
                    catch
                        level1Files{processChan(chan_num)}(ii).utc = matlab2unixtime(datenum([fileNameFields{dateField+1},fileNameFields{timeField+1}],[outFileDateFormat,outFileTimeFormat]));
                        warning('dicePickSurf:oldFileFormat','Files in old filename format. Re-run diceProcessConfig to rename the files.');
                        warning('off','dicePickSurf:oldFileFormat');
                    end
                end
                [~,sx] = sort([level1Files{processChan(chan_num)}.utc]);
                level1Files{processChan(chan_num)} = level1Files{processChan(chan_num)}(sx);
            end
        end
        %% Compute the surfaces for each channel first
        % The radar surface is spiky so it need to be smoothed. To mitigate the
        % edge effects of smoothing is done by loading 3 files at a time
        fprintf('dicePickSurf: Computing Surface from Radar ... \n');
        for chan_num = 1:numel(processChan)
            refsurfind = initSurfInd;  % chsurfwinind is updated for each batch
            gotsurf = false;
            numfiles=numel(level1Files{processChan(chan_num)});
            fTime = [level1Files{processChan(chan_num)}.utc];
            dTime = diff(fTime);
            batchind = find(dTime >filesperbatch*fileLenTime*1.1);
            numbatches = numel(batchind)+1;
            batchmin(1) = 1;
            if isempty(batchind)
                batchmax(1) = numfiles;
            else
                batchmax(1) = batchind(1);
                batchmin(numbatches) = batchind(end)+1;
                batchmax(numbatches) = numfiles;
            end
            for bb=2:numbatches-1
                batchmin(bb) = batchind(bb-1)+1;
                batchmax(bb) = batchind(bb);
            end
            if numbatches>1
                warning('dicePickSurf:FilesNotContiguous','Files are not contiguous!');
            end
            
            minfNum=batchmin;
            maxfNum=batchmax;
            
            for batch=1:numbatches
                thisFileAlreadyPicked = false;
                gotsurfIndRange = false;
                if(maxfNum(batch)-minfNum(batch) +1 >=3)
                    varlist = whos('-file',[geoLocatedDataPath,level1Files{processChan(chan_num)}(minfNum(batch)).name]);
                    thisFileAlreadyPicked = all(ismember({'surfIndRad';'planeHeight'},{varlist.name}));
                    % always load the first file fo the batch
                    load([geoLocatedDataPath,level1Files{processChan(chan_num)}(minfNum(batch)).name],'radecho','twtt','txOn','prodLevel')
                    
                    % backward compatibility
                    if ~iscell(prodLevel.desc)
                        oldProdLevel = prodLevel;
                        clear prodLevel;
                        prodLevel.name = oldProdLevel.name;
                        prodLevel.desc = {oldProdLevel.desc};
                        prodLevel.time = {'?'};
                    end
                    
                    filesize1 = size(radecho,2);
                    fileInds1 = 1:filesize1;
                    tmpecho(1:truncLengths{processChan(chan_num)},fileInds1) = radecho;
                    
                    prodLevel1 = prodLevel;
                    if ~exist('txOn','var')
                        txOn = 20*log10(max(abs(radecho(TxInd-10:TxInd+10,:)),[],1))>txThreshold(processChan(chan_num));
                    end
                    txOn1 = txOn;
                    clear txOn;
                    % always load the second file of the batch
                    load([geoLocatedDataPath,level1Files{processChan(chan_num)}(minfNum(batch)+1).name],'radecho','twtt','txOn','prodLevel')
                    
                    % backward compatibility
                    if ~iscell(prodLevel.desc)
                        oldProdLevel = prodLevel;
                        clear prodLevel;
                        prodLevel.name = oldProdLevel.name;
                        prodLevel.desc = {oldProdLevel.desc};
                        prodLevel.time = {'?'};
                    end
                    
                    filesize2 = size(radecho,2);
                    fileInds2 = (filesize1+1):(filesize1+filesize2);
                    tmpecho(1:truncLengths{processChan(chan_num)},fileInds2) = radecho;
                    
                    prodLevel2 = prodLevel;
                    
                    if ~exist('txOn','var')
                        txOn = 20*log10(max(abs(radecho(TxInd-10:TxInd+10,:)),[],1))>txThreshold(processChan(chan_num));
                    end
                    txOn2 = txOn;
                    clear txOn;
                    fprintf('dicePickSurf: Processing %s File %d of %d\n',processChanAttbr{processChan(chan_num)},minfNum(batch),numfiles);
                    
                    if ~fixedInds
                        gotsurfIndRange = false;
                        [~,tmpSurfInds] = findpeaksMat(20*log10(abs(tmpecho(:,1:floor((filesize1+surfFilterWin*2)*.05):(filesize1+surfFilterWin*2)))),1,surfPickOption,surfThresh(processChan(chan_num)));
                        if numel(find(~isnan(tmpSurfInds)))/numel(tmpSurfInds)>0.4 % 40% of the echoes returned surface indices
                            gotsurfIndRange = true;
                            surfWinInd = [max(2,nanmin(tmpSurfInds) - ceil(surfaceTrackingIndWin/2)) min(numel(twtt),nanmax(tmpSurfInds) + ceil(surfaceTrackingIndWin/2))];
                        end
                    end
                    
                    if fixedInds || ~gotsurfIndRange                  
                        if ~gotsurf
                            surfWinInd = [max(2,refsurfind - ceil(surfaceAcqIndWin/2)),min(numel(twtt),refsurfind + ceil(surfaceAcqIndWin/2))];
                        else
                            surfWinInd = [max(2,refsurfind - ceil(surfaceTrackingIndWin/2)),min(numel(twtt),refsurfind + ceil(surfaceTrackingIndWin/2))];
                        end
                    end
                        
                    [~,tempInd(:,1:filesize1+surfFilterWin*2)] = findpeaksMat(intuitive_lowpass2(20*log10(abs(tmpecho([surfWinInd(1):surfWinInd(2)],1:(filesize1+surfFilterWin*2)))),surfFilterWin,1),numSurfPicks,surfPickOption,surfThresh(processChan(chan_num)));
                    surfIndRad = surfWinInd(1) + tempInd(:,1:filesize1) -1;
                    surfIndRad(:,~txOn1) = noTxSurfInd;
                    minInd = min(surfIndRad,[],1);
                    % lets check and see if a surface is found..
                    if any(isnan(minInd))
                        warning('dicePickSurf:noSurfFound',['No surfaces above specified threshold\n',...
                            'File %s'],level1Files{processChan(chan_num)}(minfNum(batch)).name);
                        minInd(isnan(minInd)) = 1;
                        planeHeight = (twtt(minInd)-twtt(TxInd))'*cAir/2;
                        planeHeight(planeHeight<0) = NaN;
                    else
                        planeHeight = (twtt(minInd)-twtt(TxInd))'*cAir/2;
                    end
                    
                    
                    % update the refsurfIndRad
                    % update the refsurfIndRad
                    tmpGoodSurf = (minInd(minInd>2));
                    tmpGoodSurf = median(tmpGoodSurf(floor(numel(tmpGoodSurf)*0.9):end));
                    
                    if ~isnan(tmpGoodSurf)
                        refsurfind = tmpGoodSurf;
                        gotsurf = true;
                    else
                        gotsurf = false;
                    end
                        
                    % file is only saved if it hasn't been picked or
                    % if overwrite is enabled when: there is not config
                    % match or configmatch is ignored
                    if (~thisFileAlreadyPicked || ((~configmatch || ignoreConfigMatch) && overwriteoutput))
                        if save_append_backup
                            copyfile([geoLocatedDataPath,level1Files{processChan(chan_num)}(minfNum(batch)).name],[geoLocatedDataPath,level1Files{processChan(chan_num)}(minfNum(batch)).name,tmpExt]);
                        end
                        
                        prodLevel1.name = 'level1';
                        prodLevel1.desc{numel(prodLevel1.desc)+1} = 'surfPicked';
                        prodLevel1.time{numel(prodLevel1.time)+1} = datestr(surfPickedTime);
                        if any(ismember(prodLevel1.desc,'singleSurf'))
                            prodLevel1.desc{ismember(prodLevel1.desc,'singleSurf')} = 'singleSurfRem';
                        end
                        prodLevel = prodLevel1;
                        if ~configFileCopied
                            [~,configFilename,~] = fileparts(configScript);
                            copyfile([configFilename,'.mat'],[geoLocatedDataPath,configFilename,'.mat'])
                            configFileCopied = true;
                        end
                        
                        save([geoLocatedDataPath,level1Files{processChan(chan_num)}(minfNum(batch)).name],'surfIndRad','planeHeight','prodLevel','surfPickedTime','-append');
                        
                        if save_append_backup
                            delete([geoLocatedDataPath,level1Files{processChan(chan_num)}(minfNum(batch)).name,tmpExt]);
                        end
                    else
                        fprintf('dicePickSurf: %d Skipped!\n',minfNum(batch));
                        if configmatch && ~ignoreConfigMatch
                            warning('diceGeoLocate:conFigsMatch',['Existing surfaces use exact same pick configurations\n',...
                                'Set "ignoreConfigMatch = true" to reprocess these files']);
                            warning('off','diceGeoLocate:conFigsMatch');
                        end
                        if ~overwriteoutput
                            warning('diceGeoLocate:cannotOverWrite',['Existing surface Data cannot be overwritten\n',...
                                'Set "overwriteoutput = true" to reprocess these files']);
                            warning('off','diceGeoLocate:cannotOverWrite');
                        end
                    end
                    
                    for ii = minfNum(batch)+1:maxfNum(batch)-1
                        varlist = whos('-file',[geoLocatedDataPath,level1Files{processChan(chan_num)}(ii).name]);
                        thisFileAlreadyPicked = all(ismember({'surfIndRad';'planeHeight'},{varlist.name}));
                        % load the next file
                        load([geoLocatedDataPath,level1Files{processChan(chan_num)}(ii+1).name],'radecho','twtt','txOn','prodLevel')
                        
                        % backward compatibility
                        if ~iscell(prodLevel.desc)
                            oldProdLevel = prodLevel;
                            clear prodLevel;
                            prodLevel.name = oldProdLevel.name;
                            prodLevel.desc = {oldProdLevel.desc};
                            prodLevel.time = {'?'};
                        end
                        
                        filesize3 = size(radecho,2);
                        % recompute inds for new iteration
                        fileInds3 = (filesize1 + filesize2 + 1): (filesize1+filesize2+filesize3);
                        tmpecho(1:truncLengths{processChan(chan_num)},fileInds3) = radecho;
                        
                        
                        prodLevel3 = prodLevel;
                        if ~exist('txOn','var')
                            txOn = 20*log10(max(abs(radecho(TxInd-10:TxInd+10,:)),[],1))>txThreshold(processChan(chan_num));
                        end
                        txOn3 = txOn;
                        clear txOn;
                        % findpeaksMat picks nSurfPick surfaces with selected options
                        fprintf('dicePickSurf: Processing %s File %d of %d\n',processChanAttbr{processChan(chan_num)},ii,numfiles);
                        varlist = whos('-file',[geoLocatedDataPath,level1Files{processChan(chan_num)}(ii).name]);
                        if thisFileAlreadyPicked && configmatch
                            if(~forceSurfPick )
                                warning('diceGeoLocate:alreadyGeoLocated',['One or more files already have surfaces\n',...
                                    'Set "forceSurfPick = true" to reprocess these files']);
                                warning('off','diceGeoLocate:alreadyGeoLocated');
                                tmpecho(1:truncLengths{processChan(chan_num)},1:filesize2) = tmpecho(1:truncLengths{processChan(chan_num)},fileInds2);
                                tmpecho(1:truncLengths{processChan(chan_num)},(filesize2+1):(filesize2+filesize3)) = tmpecho(1:truncLengths{processChan(chan_num)},fileInds3);
                                fileInds1 = 1:filesize2;
                                fileInds2 = (filesize2+1):(filesize2+filesize3);
                                filesize1 = filesize2;
                                filesize2 = filesize3;
                                txOn1 = txOn2;
                                txOn2 = txOn3;
                                prodLevel2 = prodLevel3;
                                fprintf('dicePickSurf: %d Skipped!\n',ii);
                                continue;
                            end
                        end
                        
                        if ~fixedInds
                            gotsurfIndRange = false;
                            [~,tmpSurfInds] = findpeaksMat(20*log10(abs(tmpecho(:,(fileInds1(end)-surfFilterWin*2+1):floor(((fileInds2(end)+surfFilterWin*2)-(fileInds1(end)-surfFilterWin*2+1)+1)*0.05):(fileInds2(end)+surfFilterWin*2)))),1,surfPickOption,surfThresh(processChan(chan_num)));
                            if numel(find(~isnan(tmpSurfInds)))/numel(tmpSurfInds)>0.4 % 40% of the echoes returned surface indices
                                gotsurfIndRange = true;
                                surfWinInd = [max(2,nanmin(tmpSurfInds) - ceil(surfaceTrackingIndWin/2)) min(numel(twtt),nanmax(tmpSurfInds) + ceil(surfaceTrackingIndWin/2))];
                            end
                        end
                    
                    if fixedInds || ~gotsurfIndRange 
                        if ~gotsurf
                        surfWinInd = [max(2,refsurfind - ceil(surfaceAcqIndWin/2)),min(numel(twtt),refsurfind + ceil(surfaceAcqIndWin/2))];
                    else
                        surfWinInd = [max(2,refsurfind - ceil(surfaceTrackingIndWin/2)),min(numel(twtt),refsurfind + ceil(surfaceTrackingIndWin/2))];
                        end
                    end
                    
                        [~,tempInd(:,1:filesize2+surfFilterWin*4)] = ...
                            findpeaksMat(intuitive_lowpass2(20*log10(abs(tmpecho([surfWinInd(1):surfWinInd(2)],(fileInds1(end)-surfFilterWin*2+1):(fileInds2(end)+surfFilterWin*2)))),surfFilterWin,1),numSurfPicks,surfPickOption,surfThresh(processChan(chan_num)));
                        surfIndRad = surfWinInd(1) + tempInd(:,surfFilterWin*2+1:filesize2+surfFilterWin*2) -1;
                        surfIndRad(:,~txOn2) = noTxSurfInd;
                        minInd = min(surfIndRad,[],1);
                        % lets check and see if a surface is found..
                        if any(isnan(minInd))
                            warning('dicePickSurf:noSurfFound',['No surfaces above specified threshold\n',...
                                'File %s'],level1Files{processChan(chan_num)}(ii).name);
                            minInd(isnan(minInd)) = 1;
                            planeHeight = (twtt(minInd)-twtt(TxInd))'*cAir/2;
                            planeHeight(planeHeight<0) = NaN;
                        else
                            planeHeight = (twtt(minInd)-twtt(TxInd))'*cAir/2;
                        end
                        
                    % update the refsurfIndRad
                    tmpGoodSurf = (minInd(minInd>2));
                    tmpGoodSurf = median(tmpGoodSurf(floor(numel(tmpGoodSurf)*0.9):end));
                    if ~isnan(tmpGoodSurf)
                        refsurfind = tmpGoodSurf;
                        gotsurf = true;
                    else
                        gotsurf = false;
                    end
                    
                        % file is only saved if it hasn't been picked or
                        % if overwrite is enabled when: there is not config
                        % match or configmatch is ignored
                        if (~thisFileAlreadyPicked || ((~configmatch || ignoreConfigMatch) && overwriteoutput))
                            
                            prodLevel2.name = 'level1';
                            prodLevel2.desc{numel(prodLevel2.desc)+1} = 'surfPicked';
                            prodLevel2.time{numel(prodLevel2.time)+1} = datestr(surfPickedTime);
                            if any(ismember(prodLevel2.desc,'singleSurf'))
                                prodLevel2.desc{ismember(prodLevel2.desc,'singleSurf')}  = 'singleSurfRem';
                            end
                            prodLevel = prodLevel2;
                            if ~configFileCopied
                                [~,configFilename,~] = fileparts(configScript);
                                copyfile([configFilename,'.mat'],[geoLocatedDataPath,configFilename,'.mat'])
                                configFileCopied = true;
                            end
                            
                            if save_append_backup
                                copyfile([geoLocatedDataPath,level1Files{processChan(chan_num)}(ii).name],[geoLocatedDataPath,level1Files{processChan(chan_num)}(ii).name,tmpExt]);
                            end
                            save([geoLocatedDataPath,level1Files{processChan(chan_num)}(ii).name],'surfIndRad','planeHeight','prodLevel','surfPickedTime','-append');
                            if save_append_backup
                                delete([geoLocatedDataPath,level1Files{processChan(chan_num)}(ii).name,tmpExt]);
                            end
                        else
                            fprintf('dicePickSurf: %d Skipped!\n',ii);
                            if configmatch && ~ignoreConfigMatch
                                warning('diceGeoLocate:conFigsMatch',['Existing surfaces use exact same pick configurations\n',...
                                    'Set "ignoreConfigMatch = true" to reprocess these files']);
                                warning('off','diceGeoLocate:conFigsMatch');
                            end
                            if ~overwriteoutput
                                warning('diceGeoLocate:cannotOverWrite',['Existing surface Data cannot be overwritten\n',...
                                    'Set "overwriteoutput = true" to reprocess these files']);
                                warning('off','diceGeoLocate:cannotOverWrite');
                            end
                        end
                        
                        tmpecho(1:truncLengths{processChan(chan_num)},1:filesize2) = tmpecho(1:truncLengths{processChan(chan_num)},fileInds2);
                        tmpecho(1:truncLengths{processChan(chan_num)},(filesize2+1):(filesize2+filesize3)) = tmpecho(1:truncLengths{processChan(chan_num)},fileInds3);
                        fileInds1 = 1:filesize2;
                        fileInds2 = (filesize2+1):(filesize2+filesize3);
                        filesize1 = filesize2;
                        filesize2 = filesize3;
                        txOn1 = txOn2;
                        txOn2 = txOn3;
                        prodLevel2 = prodLevel3;
                    end
                    varlist = whos('-file',[geoLocatedDataPath,level1Files{processChan(chan_num)}(ii+1).name]);
                    thisFileAlreadyPicked = all(ismember({'surfIndRad';'planeHeight'},{varlist.name}));
                    
                    % process the last file in the batch
                    fprintf('dicePickSurf: Processing %s File %d of %d\n',processChanAttbr{processChan(chan_num)},ii+1,numfiles);
                    
                    if (~thisFileAlreadyPicked || ((~configmatch || ignoreConfigMatch) && overwriteoutput))
                        
                        if ~fixedInds
                            gotsurfIndRange = false;
                            [~,tmpSurfInds] = findpeaksMat(20*log10(abs(tmpecho(:,(fileInds1(end)-surfFilterWin*2+1):floor(((fileInds2(end)+surfFilterWin*2)-(fileInds1(end)-surfFilterWin*2+1)+1)*0.05):(fileInds2(end)+surfFilterWin*2)))),1,surfPickOption,surfThresh(processChan(chan_num)));
                            if numel(find(~isnan(tmpSurfInds)))/numel(tmpSurfInds)>0.4 % 40% of the echoes returned surface indices
                                gotsurfIndRange = true;
                                surfWinInd = [max(2,nanmin(tmpSurfInds) - ceil(surfaceTrackingIndWin/2)) min(numel(twtt),nanmax(tmpSurfInds) + ceil(surfaceTrackingIndWin/2))];
                            end
                        end
                    
                    if fixedInds || ~gotsurfIndRange 
                        if ~gotsurf
                        surfWinInd = [max(2,refsurfind - ceil(surfaceAcqIndWin/2)),min(numel(twtt),refsurfind + ceil(surfaceAcqIndWin/2))];
                    else
                        surfWinInd = [max(2,refsurfind - ceil(surfaceTrackingIndWin/2)),min(numel(twtt),refsurfind + ceil(surfaceTrackingIndWin/2))];
                        end
                    end
                    
                        [~,tempInd(:,1:filesize2+surfFilterWin*4)] = ...
                            findpeaksMat(intuitive_lowpass2(20*log10(abs(tmpecho([surfWinInd(1):surfWinInd(2)],(fileInds1(end)-surfFilterWin*2+1):(fileInds2(end)+surfFilterWin*2)))),surfFilterWin,1),numSurfPicks,surfPickOption,surfThresh(processChan(chan_num)));
                        surfIndRad = surfWinInd(1) + tempInd(:,surfFilterWin*2+1:filesize2+surfFilterWin*2) -1;
                        surfIndRad(:,~txOn3) = noTxSurfInd;
                        minInd = min(surfIndRad,[],1);
                        % lets check and see if a surface is found..
                        if any(isnan(minInd))
                            warning('dicePickSurf:noSurfFound',['No surfaces above specified threshold\n',...
                                'File %s'],level1Files{processChan(chan_num)}(ii+1).name);
                            minInd(isnan(minInd)) = 1;
                            planeHeight = (twtt(minInd)-twtt(TxInd))'*cAir/2;
                            planeHeight(planeHeight<0) = NaN;
                        else
                            planeHeight = (twtt(minInd)-twtt(TxInd))'*cAir/2;
                        end
                        
                                            % update the refsurfIndRad
                   % update the refsurfIndRad
                    tmpGoodSurf = (minInd(minInd>2));
                    tmpGoodSurf = median(tmpGoodSurf(floor(numel(tmpGoodSurf)*0.9):end));
                    if ~isnan(tmpGoodSurf)
                        refsurfind = tmpGoodSurf;
                        gotsurf = true;
                    else
                        gotsurf = false;
                    end
                    
                        prodLevel2.name = 'level1';
                        prodLevel2.desc{numel(prodLevel2.desc)+1} = 'surfPicked';
                        prodLevel2.time{numel(prodLevel2.time)+1} = datestr(surfPickedTime);
                        if any(ismember(prodLevel2.desc,'singleSurf'))
                            prodLevel2.desc{ismember(prodLevel2.desc,'singleSurf')} = 'singleSurfRem';
                        end
                        prodLevel = prodLevel2;
                        
                        if ~configFileCopied
                            [~,configFilename,~] = fileparts(configScript);
                            copyfile([configFilename,'.mat'],[geoLocatedDataPath,configFilename,'.mat'])
                            configFileCopied = true;
                        end
                        
                        if save_append_backup
                            copyfile([geoLocatedDataPath,level1Files{processChan(chan_num)}(ii+1).name],[geoLocatedDataPath,level1Files{processChan(chan_num)}(ii+1).name,tmpExt]);
                        end
                        save([geoLocatedDataPath,level1Files{processChan(chan_num)}(ii+1).name],'surfIndRad','planeHeight','prodLevel','surfPickedTime','-append');
                        if save_append_backup
                            delete([geoLocatedDataPath,level1Files{processChan(chan_num)}(ii+1).name,tmpExt]);
                        end
                    else
                        fprintf('dicePickSurf: %d Skipped!\n',ii+1);
                        if configmatch && ~ignoreConfigMatch
                            warning('diceGeoLocate:conFigsMatch',['Existing surfaces use exact same pick configurations\n',...
                                'Set "ignoreConfigMatch = true" to reprocess these files']);
                            warning('off','diceGeoLocate:conFigsMatch');
                        end
                        if ~overwriteoutput
                            warning('diceGeoLocate:cannotOverWrite',['Existing surface Data cannot be overwritten\n',...
                                'Set "overwriteoutput = true" to reprocess these files']);
                            warning('off','diceGeoLocate:cannotOverWrite');
                        end
                    end
                else
                    for ii = minfNum(batch):maxfNum(batch)
                        fprintf('dicePickSurf: Processing %s File %d of %d\n',processChanAttbr{processChan(chan_num)},ii,numfiles);
                        varlist = whos('-file',[geoLocatedDataPath,level1Files{processChan(chan_num)}(ii).name]);
                        thisFileAlreadyPicked = all(ismember({'surfIndRad';'planeHeight'},{varlist.name}));
                        
                        if thisFileAlreadyPicked && configmatch
                            if(~forceSurfPick)
                                warning('diceGeoLocate:alreadyGeoLocated',['One or more files are already geo-located\n',...
                                    'Set "forceGeolocate = true" to reprocess these files']);
                                warning('off','diceGeoLocate:alreadyGeoLocated');
                                fprintf('dicePickSurf: %d Skipped!\n',ii);
                                continue;
                            end
                        end
                        load([geoLocatedDataPath,level1Files{processChan(chan_num)}(ii).name],'radecho','twtt','prodLevel')
                        
                        % backward compatibility
                        if ~iscell(prodLevel.desc)
                            oldProdLevel = prodLevel;
                            clear prodLevel;
                            prodLevel.name = oldProdLevel.name;
                            prodLevel.desc = {oldProdLevel.desc};
                            prodLevel.time = {'?'};
                        end
                        
                        if ~fixedInds
                            gotsurfIndRange = false;
                            [~,tmpSurfInds] = findpeaksMat(20*log10(abs(radecho(:,1:floor(size(radecho,2)*0.05):end))),1,surfPickOption,surfThresh(processChan(chan_num)));
                            if numel(find(~isnan(tmpSurfInds)))/numel(tmpSurfInds)>0.4 % 40% of the echoes returned surface indices
                                gotsurfIndRange = true;
                                surfWinInd = [max(2,nanmin(tmpSurfInds) - ceil(surfaceTrackingIndWin/2)) min(numel(twtt),nanmax(tmpSurfInds) + ceil(surfaceTrackingIndWin/2))];
                            end
                        end
                    
                    if fixedInds || ~gotsurfIndRange 
                        if ~gotsurf
                        surfWinInd = [max(2,refsurfind - ceil(surfaceAcqIndWin/2)),min(numel(twtt),refsurfind + ceil(surfaceAcqIndWin/2))];
                    else
                        surfWinInd = [max(2,refsurfind - ceil(surfaceTrackingIndWin/2)),min(numel(twtt),refsurfind + ceil(surfaceTrackingIndWin/2))];
                        end
                    end
                    
                        [~,surfIndRad] = findpeaksMat(intuitive_lowpass2(20*log10(abs(radecho([surfWinInd(1):surfWinInd(2)],:))),surfFilterWin,1),numSurfPicks,surfPickOption,surfThresh(processChan(chan_num)));
                        surfIndRad(:,:) = surfWinInd(1) + surfIndRad - 1;
                        minInd = min(surfIndRad,[],1);
                        % lets check and see if a surface is found..
                        if any(isnan(minInd))
                            warning('dicePickSurf:noSurfFound',['No surfaces above specified threshold\n',...
                                'File %s'],level1Files{processChan(chan_num)}(ii).name);
                            minInd(isnan(minInd)) = 1;
                            planeHeight = (twtt(minInd)-twtt(TxInd))'*cAir/2;
                            planeHeight(planeHeight<0) = NaN;
                        else
                            planeHeight = (twtt(minInd)-twtt(TxInd))'*cAir/2;
                        end
                        
                                            % update the refsurfIndRad
                    % update the refsurfIndRad
                    tmpGoodSurf = (minInd(minInd>2));
                    tmpGoodSurf = median(tmpGoodSurf(floor(numel(tmpGoodSurf)*0.9):end));
                    
                    if ~isnan(tmpGoodSurf)
                        refsurfind = tmpGoodSurf;
                        gotsurf = true;
                    else
                        gotsurf = false;
                    end
                        
                        if (~thisFileAlreadyPicked || ((~configmatch || ignoreConfigMatch) && overwriteoutput))
                            
                            prodLevel.name = 'level1';
                            prodLevel.desc{numel(prodLevel.desc)+1} = 'surfPicked';
                            prodLevel.time{numel(prodLevel.time)+1} = datestr(surfPickedTime);
                            if any(ismember(prodLevel.desc,'singleSurf'))
                                prodLevel.desc{ismember(prodLevel.desc,'singleSurf')} = 'singleSurfRem';
                            end
                            if ~configFileCopied
                                [~,configFilename,~] = fileparts(configScript);
                                copyfile([configFilename,'.mat'],[geoLocatedDataPath,configFilename,'.mat'])
                                configFileCopied = true;
                            end
                            
                            if save_append_backup
                                copyfile([geoLocatedDataPath,level1Files{processChan(chan_num)}(ii).name],[geoLocatedDataPath,level1Files{processChan(chan_num)}(ii).name,tmpExt]);
                            end
                            save([geoLocatedDataPath,level1Files{processChan(chan_num)}(ii).name],'surfIndRad','planeHeight','prodLevel','surfPickedTime','-append');
                            if save_append_backup
                                delete([geoLocatedDataPath,level1Files{processChan(chan_num)}(ii).name,tmpExt]);
                            end
                        else
                            fprintf('dicePickSurf: %d Skipped!\n',ii);
                            if configmatch && ~ignoreConfigMatch
                                warning('diceGeoLocate:conFigsMatch',['Existing surfaces use exact same pick configurations\n',...
                                    'Set "ignoreConfigMatch = true" to reprocess these files']);
                                warning('off','diceGeoLocate:conFigsMatch');
                            end
                            if ~overwriteoutput
                                warning('diceGeoLocate:cannotOverWrite',['Existing surface Data cannot be overwritten\n',...
                                    'Set "overwriteoutput = true" to reprocess these files']);
                                warning('off','diceGeoLocate:cannotOverWrite');
                            end
                        end
                    end
                end
            end
        end
        warning('on','all');
        diary off;
        if(configFileCopied)    % if true, then the flight was most likely processed
            [~,logFileNameOnly,logFileExt] = fileparts(logFile);
            if ~exist(processLogPath,'dir')
                mkdir(processLogPath);
            end
            movefile(logFile,[processLogPath,logFileNameOnly,fileFieldSep,'surfpick',datestr(surfPickedTime,'_yyyymmdd_hhMMss'),logFileExt]);
            if runCount == 1
                [~,configFileNameOnly,~] = fileparts(configScript);
                copyfile(configScript,[processLogPath,configFileNameOnly,fileFieldSep,'surfpick',datestr(surfPickedTime,'_yyyymmdd_hhMMss'),'.m.log']);
            end
        else % delete existing diary and start a fresh one
            [~,~,logext] = fileparts(logFile);
            if strcmpi(logext,'.log') %% just making sure it's a .log file before deletion
                delete(logFile);
            end
        end
        diary(logFile);
    end
    if localDiary % if diary was locally started - delete it
        if strcmpi(get(0,'Diary'),'on')
            diary off;
            [~,~,logext] = fileparts(logFile);
            if strcmpi(logext,'.log') %% just making sure it's a .log file before deletion
                delete(logFile);
            end
        end
    end
    fprintf('dicePickSurf: Done!\n');
    toc
catch errMsg
    fprintf('dicePickSurf: Encountered an error\n');
    fprintf('%s\n',errMsg.message);
    fprintf('%s\n',errMsg.identifier);
    for jj = 1:numel(errMsg.stack)
        fprintf('In Function %s, Line %d\n',errMsg.stack(jj).name,errMsg.stack(jj).line)
    end
    warning('on','all');
    % move the log file
    diary off;
    if exist(logFile,'file') && exist('processLogPath','var')
        [~,logFileNameOnly,logFileExt] = fileparts(logFile);
        if ~exist(processLogPath,'dir')
            mkdir(processLogPath);
        end
        movefile(logFile,[processLogPath,logFileNameOnly,'_crash',datestr(now,'_yyyymmdd_hhMMss'),logFileExt]);
    end
    if debugEnable
        fprintf('dicePickSurf: Halting for debugging\n')
        keyboard
    end
end
end