function [] = dicePreSarAssemble(newConfig,logFile)
% Parameters: None, all required variables are loaded by running
% diceProcessConfig.m
% This code reads the pulse Compressed data and gets GeoLocates it for further
% Processsing.
%   - Rearranges the data so each file contains same number of echoes
%     required for SAR as specified in SAR window
% Initialize the settings and variables
try
fprintf('************** dicePreSarAssemble.m ****************\n')
if nargin <1
    configScript = 'diceProcessConfig.m';
    logFile = 'diceProcessLog.log';
    diary off;
    if exist(logFile,'file')
        warning('dicePreSarAssemble:existingLogFile','A logfile %s already exists!\n, Please cleanup the logs from %s for future processing',logFile,pwd);
        [~,logFileNameOnly,logFileExt] = fileparts(logFile);
        logFile = [logFileNameOnly,datestr(now,'_yyyymmdd_hhMMss'),logFileExt];
    end
    diary(logFile);
    localDiary = true;
elseif nargin <2
    configScript = newConfig;
    logFile = 'diceProcessLog.log';
    diary off;
    if exist(logFile,'file')
        warning('dicePreSarAssemble:existingLogFile','A logfile %s already exists!\n, Please cleanup the logs from %s for future processing',logFile,pwd);
        [~,logFileNameOnly,logFileExt] = fileparts(logFile);
        logFile = [logFileNameOnly,datestr(now,'_yyyymmdd_hhMMss'),logFileExt];
    end
    diary(logFile);
    localDiary = true;
else
    configScript = newConfig;
    localDiary = false;
end

run(configScript);
configParamsCurr = configParams;
tic

maxnumTraces = ceil(fileLenTime/PRI/4/numAvg*1.1*filesperbatch)+sarTracesPerFile;
tmpecho = zeros(max(truncLenp1,truncLenp2),maxnumTraces);
tmpLat = zeros(1,maxnumTraces);
tmpLon =  zeros(1,maxnumTraces);
tmpSurfInds =  zeros(numSurfPicks,maxnumTraces);
tmpPlaneHeight =  zeros(1,maxnumTraces);
tmpFlightElev =  zeros(1,maxnumTraces);

tmputc = zeros(1,maxnumTraces);

truncLengths = {truncLenp1;truncLenp1;truncLenp2;truncLenp2};
%% Work

for ff = 1:numel(uniqueFlightNum)
    %% Check files and directories:
    FlightNo = uniqueFlight{ff};
    preSarAssembledTime = now;
    configFileCopied = false;
    surfPickedDataPath = [sensorRootPath,filesep,process1Dir,filesep,stackDir,filesep,FlightNo,filesep,dataDir,filesep];
    preSarDataPath = [sensorRootPath,filesep,process1Dir,filesep,stackDir,filesep,FlightNo,filesep,dataDir,filesep,tmpPreSarDir,filesep];
    processLogPath = [sensorRootPath,filesep,process1Dir,filesep,stackDir,filesep,FlightNo,filesep,logDir,filesep];
    
    if exist(surfPickedDataPath,'dir')==0
        warning('dicePreSarAssemble:noLevel1Dir','Level1 Flight %s Directory does not exists!',FlightNo);
        continue;
    else
        anyFiles=dir([surfPickedDataPath,'*',FlightNo,'*',Sensor,'*.mat']);
        if isempty(anyFiles)
            warning('dicePreSarAssemble:noLevel1Dir','Level1 Flight %s has no data to assemble!',FlightNo);
            continue;
        end
    end
    
    fprintf('dicePreSarAssemble: Processing Flight %s\n',FlightNo)
    % Make output folder, if necessary:
    if exist(preSarDataPath,'dir')==0
        mkdir(preSarDataPath)
    else
        dataFlist = dir([preSarDataPath,'*',FlightNo,'*.mat']);
        if numel(dataFlist)>0
            fprintf('dicePreSarAssemble: Found existing data files\n');
            pcFlist = dir([preSarDataPath,'*',configFnameRoot,'*.mat']);
            if numel(pcFlist)>0
                [~,sx] = sort([pcFlist.datenum]);
                pcFlist = pcFlist(sx);
                fprintf('dicePreSarAssemble: Checking existing configuration data...\n')
                load([preSarDataPath,pcFlist(end).name],'configParams');
                if isequal(configParamsCurr,configParams) && ~ignoreConfigMatch
                    fprintf(['dicePreSarAssemble: Current settings are identical to the one %s processed data used.\n\t',...
                        '%s will not be processed.\n\tTo force processing, Remove configuration mat file from\n\t%s\n'],FlightNo,FlightNo,preSarDataPath);
                    continue;
                else
                    if overwriteoutput == 1
                        warning('dicePulseCompress:FlightDirExists',['All existing files will be deleted!\n',...
                            'If this is a mistake, press Ctrl + C to cancel'],FlightNo);
                        
                        fprintf('dicePreSarAssemble: Files will be deleted in %d seconds',9);
                        for ss = 9:-1:0
                            fprintf('\b\b\b\b\b\b\b\b\b%d seconds',ss);
                            pause(1);
                        end
                        fprintf('\ndicePreSarAssemble: Deleted!\n')
                        delete([preSarDataPath,'*',FlightNo,'*.mat']);
                        delete([preSarDataPath,'*onfig*.mat']);
                    else
                        warning('dicePulseCompress:FlightDirExists','Flight %s Directory already exists - Flight will not be processed!\n Set "overwriteoutput = 1" to force processing',uniqueFlight{ff});
                        continue;
                    end
                end
            else    %if for any reason there is no configuration file
                if overwriteoutput == 1
                    warning('dicePulseCompress:FlightDirExists',['All existing files will be deleted!\n',...
                        'If this is a mistake, press Ctrl + C to cancel'],FlightNo);
                    
                    fprintf('dicePreSarAssemble: Files will be deleted in %d seconds',9);
                    for ss = 9:-1:0
                        fprintf('\b\b\b\b\b\b\b\b\b%d seconds',ss);
                        pause(1);
                    end
                    fprintf('\ndicePreSarAssemble: Deleted!\n')
                    delete([preSarDataPath,'*',FlightNo,'*.mat']);
                    delete([preSarDataPath,'*onfig*.mat']);
                else
                    warning('dicePulseCompress:FlightDirExists','Flight %s Directory already exists - Flight will not be processed!\n Set "overwriteoutput = 1" to force processing',uniqueFlight{ff});
                    continue;
                end
            end
        end
    end

    %% Find the files for each line process.
    for chan_num = 1:numel(processChan)
        level1Files=dir([surfPickedDataPath,'*',processChanAttbr{processChan(chan_num)},'*.mat']);
        numfiles=numel(level1Files);
        if(numfiles == 0)
            warning(['dicePreSarAssemble:NoFiles','no',processChanAttbr{processChan(chan_num)},'Files'],'No level0 %s files found!',processChanAttbr{processChan(chan_num)})
        else
            for ii = 1:numfiles
                fileNameFields = strsplit(level1Files(ii).name,fileFieldSep);
                try
                    old_fileformat = 0;
                    level1Files(ii).utc = matlab2unixtime(datenum([fileNameFields{dateField},fileNameFields{timeField}],[outFileDateFormat,outFileTimeFormat]));
                catch
                    level1Files(ii).utc = matlab2unixtime(datenum([fileNameFields{dateField+1},fileNameFields{timeField+1}],[outFileDateFormat,outFileTimeFormat]));
                    warning('dicePreSarAssemebd:oldFileNameFormat','File names are in old format')
                    warning('off','dicePreSarAssemebd:oldFileNameFormat');
                    old_fileformat = 1;
                end
            end
            [~,sx] = sort([level1Files.utc]);
            level1Files = level1Files(sx);
        end
        fTime = [level1Files.utc];
        dTime = diff(fTime);
        batchind = find(dTime >filesperbatch*fileLenTime*1.1);
        numbatches = numel(batchind)+1;
        batchmin(1) = 1;
        if isempty(batchind)
            batchmax(1) = numfiles;
        else
            batchmax(1) = batchind(1);
            batchmin(numbatches) = batchind(end)+1;
            batchmax(numbatches) = numfiles;
        end
        for bb=2:numbatches-1
            batchmin(bb) = batchind(bb-1)+1;
            batchmax(bb) = batchind(bb);
        end
        if numbatches>1
            warning('dicePreSarAssemble:FilesNotContiguous','Files are not contiguous!');
        end
        minfNum=batchmin;
        maxfNum=batchmax;
        
        load([surfPickedDataPath,level1Files(1).name],'twtt','numAvg')
        for batch=1:numbatches
            numRemainingTraces = 0;
            
            for ii = minfNum(batch):maxfNum(batch)
                load([surfPickedDataPath,level1Files(ii).name],'radecho','echotimeutc','Campaign','echotype','FlightNo','lat','lon','hell','surfIndRad','planeHeight','prodLevel')
                fprintf('dicePreSarAssemble: Processing %s File %d of %d\n',processChanAttbr{processChan(chan_num)},ii,numfiles);
                numTraces = size(radecho,2);
                if(numRemainingTraces > 0)
                    tmpecho(1:truncLengths{processChan(chan_num)},1:numRemainingTraces+numTraces) = [tmpecho(1:truncLengths{processChan(chan_num)},1:numRemainingTraces),radecho];
                    tmputc(1:numRemainingTraces+numTraces) = [tmputc(1:numRemainingTraces),echotimeutc];
                    tmpLat(1:numRemainingTraces+numTraces) = [tmpLat(1:numRemainingTraces),lat];
                    tmpLon(1:numRemainingTraces+numTraces) = [tmpLon(1:numRemainingTraces),lon];
                    tmpSurfInds(:,1:numRemainingTraces+numTraces) = [tmpSurfInds(:,1:numRemainingTraces),surfIndRad];
                    tmpPlaneHeight(1:numRemainingTraces+numTraces) = [tmpPlaneHeight(1:numRemainingTraces),planeHeight];
                    tmpFlightElev(1:numRemainingTraces+numTraces) = [tmpFlightElev(1:numRemainingTraces),hell];
                    
                    numRemainingTraces = numRemainingTraces + numTraces;
                else
                    tmpecho(1:truncLengths{processChan(chan_num)},1:numTraces) = radecho;
                    tmputc(1:numTraces) = echotimeutc;
                    tmpLat(1:numTraces) = lat;
                    tmpLon(1:numTraces) = lon;
                    tmpSurfInds(:,1:numTraces) = surfIndRad;
                    tmpPlaneHeight(1:numTraces) = planeHeight;
                    tmpFlightElev(1:numTraces) = hell;
                    numRemainingTraces = numTraces;
                end
                while(numRemainingTraces >= sarTracesPerFile)
                    
                    radecho = tmpecho(1:truncLengths{processChan(chan_num)},1:sarTracesPerFile);
                    echotimeutc = tmputc(1:sarTracesPerFile);
                    lat = tmpLat(1:sarTracesPerFile);
                    lon = tmpLon(1:sarTracesPerFile);
                    hell = tmpFlightElev(1:sarTracesPerFile);
                    surfIndRad = tmpSurfInds(:,1:sarTracesPerFile);
                    planeHeight = tmpPlaneHeight(1:sarTracesPerFile);
                    
                    numRemainingTraces = numRemainingTraces - sarTracesPerFile;
                    
                    tmpecho(1:truncLengths{processChan(chan_num)},1:numRemainingTraces) = tmpecho(1:truncLengths{processChan(chan_num)},sarTracesPerFile+1:numRemainingTraces+sarTracesPerFile);
                    tmputc(1:numRemainingTraces) = tmputc(sarTracesPerFile+1:numRemainingTraces+sarTracesPerFile);
                    tmpLat(1:numRemainingTraces) = tmpLat(sarTracesPerFile+1:numRemainingTraces+sarTracesPerFile);
                    tmpLon(1:numRemainingTraces) = tmpLon(sarTracesPerFile+1:numRemainingTraces+sarTracesPerFile);
                    tmpFlightElev(1:numRemainingTraces) = tmpFlightElev(sarTracesPerFile+1:numRemainingTraces+sarTracesPerFile);
                    tmpSurfInds(:,1:numRemainingTraces) = tmpSurfInds(:,sarTracesPerFile+1:numRemainingTraces+sarTracesPerFile);
                    tmpPlaneHeight(1:numRemainingTraces) = tmpPlaneHeight(sarTracesPerFile+1:numRemainingTraces+sarTracesPerFile);
                    
                    % backward compatibility
                    if ~iscell(prodLevel.desc)
                        oldProdLevel = prodLevel;
                        clear prodLevel;
                        prodLevel.name = oldProLevel.name;
                        prodLevel.desc = {oldProdLevel.desc};
                        prodLevel.time = {'?'};
                    end
                    
                    prodLevel.name = 'level1';
                    prodLevel.desc{numel(prodLevel.desc)+1} = 'preSar';
                    prodLevel.time{numel(prodLevel.time)+1} = datestr(preSarAssembledTime);
                    
                    if ~old_fileformat
                        save([preSarDataPath,fileNameFields{1},fileFieldSep,fileNameFields{2},fileFieldSep,datestr(unix2matlabtime(echotimeutc(1)),...
                            [outFileDateFormat,fileFieldSep,outFileTimeFormat]),fileFieldSep,fileNameFields{5},fileFieldSep,prodLevel.name,fileFieldSep,Sensor,'.mat'],...
                            'radecho','echotimeutc','Campaign','echotype','FlightNo','numAvg','twtt','lat','lon','hell','surfIndRad','planeHeight','prodLevel','preSarAssembledTime');
                    else
                        save([preSarDataPath,fileNameFields{1},fileFieldSep,fileNameFields{2},fileFieldSep,datestr(unix2matlabtime(echotimeutc(1)),...
                            [outFileDateFormat,fileFieldSep,outFileTimeFormat]),fileFieldSep,fileNameFields{3},fileFieldSep,prodLevel.name,fileFieldSep,Sensor,'.mat'],...
                            'radecho','echotimeutc','Campaign','echotype','FlightNo','numAvg','twtt','lat','lon','hell','surfIndRad','planeHeight','prodLevel','preSarAssembledTime');
                    end
                    if ~configFileCopied
                        [~,configFilename,~] = fileparts(configScript);
                        copyfile([configFilename,'.mat'],[preSarDataPath,configFilename,'.mat'])
                        configFileCopied = true;
                    end
                end
            end
        end
    end
    warning('on','all');
    diary off;
    if(configFileCopied)    % if true, then the flight was most likely processed
        [~,logFileNameOnly,logFileExt] = fileparts(logFile);
        if ~exist(processLogPath,'dir')
            mkdir(processLogPath);
        end
        try
            movefile(logFile,[processLogPath,logFileNameOnly,fileFieldSep,'presarassemble',datestr(preSarAssembledTime,'_yyyymmdd_hhMMss'),logFileExt]);
        catch
            warning('dicePreSarAssemebd:fileInUse','Cannot Move LogFile!\n move %s manually to %s',logFile,processLogPath);
        end
        if runCount == 1
            [~,configFileNameOnly,~] = fileparts(configScript);
            copyfile(configScript,[processLogPath,configFileNameOnly,fileFieldSep,'presarassemble',datestr(preSarAssembledTime,'_yyyymmdd_hhMMss'),'.m.log']);
        end
    else % delete existing diary and start a fresh one
        [~,~,logext] = fileparts(logFile);
        if strcmpi(logext,'.log') %% just making sure it's a .log file before deletion
            delete(logFile);
        end
    end
    diary(logFile);
end
    if localDiary % if diary was locally started - delete it
        if strcmpi(get(0,'Diary'),'on')
            diary off;
            [~,~,logext] = fileparts(logFile);
            if strcmpi(logext,'.log') %% just making sure it's a .log file before deletion
                delete(logFile);
            end
        end
    end
fprintf('dicePreSarAssemble: Done!\n')
toc
catch errMsg
    fprintf('dicePreSarAssemble: Encountered an error\n');
    fprintf('%s\n',errMsg.message);
    fprintf('%s\n',errMsg.identifier);
    for jj = 1:numel(errMsg.stack)
        fprintf('In Function %s, Line %d\n',errMsg.stack(jj).name,errMsg.stack(jj).line)
    end
    warning('on','all');
    % move the log file
    diary off;
    if exist(logFile,'file') && exist('processLogPath','var')
        [~,logFileNameOnly,logFileExt] = fileparts(logFile);
        if ~exist(processLogPath,'dir')
            mkdir(processLogPath);
        end
        movefile(logFile,[processLogPath,logFileNameOnly,'_crash',datestr(now,'_yyyymmdd_hhMMss'),logFileExt]);
    end
    if debugEnable
        fprintf('dicePreSarAssemble: Halting for debugging\n')
        keyboard
    end
end
end