function [] = diceProcess(configFile,logFile)
% This is the top level function for processing Deep Ice Radar data. This script
% calls all functions required to process selected lines. All
% configurations for processing are set in "processConfig.m". In order to
% proces a new flight or line, change the settings in "processConfig.m"

try
    fprintf('*--------------- diceProcess.m -----------------*\n')
    %% Through all scripts/functions to SAR process DICE
% comment a function as needed if the process is not desired
if ~exist('logFile','var')
    logFile = 'diceProcessLog.log';
end
close_diary = onCleanup(@()diary('off'));
    if exist(logFile,'file')
        warning('diceProcess:existingLogFile','A logfile %s already exists!\n, Please cleanup the logs from %s for future processing',logFile,pwd);
        [~,logFileNameOnly,logFileExt] = fileparts(logFile);
        logFile = [logFileNameOnly,datestr(now,'_yyyymmdd_hhMMss'),logFileExt];
    end
diary(logFile);

if ~exist('configFile','var')
    configFile = 'diceProcessConfig.m';
end

%% Data parsing, pulse compression etc happens here
dicePulseCompress(configFile,logFile);

%% Pick the surface for each channel and add to geoLocated data
dicePickSurf(configFile,logFile);
%% Replace the surface on each channel with single channel selected
% (surfaceChan)
% Be careful with this one - if a wrong channel is selected re-run
% dicePickSurf first.
diceSingleSurf(configFile,logFile);

%% Compute any time offset and apply to radar time tag - WIP
% diceFixTimeOffset(configFile,logFile);

%% Add GPS info to the Radar data
diceGeoLocate(configFile,logFile);
% 
% Move traces to make each file have same number of traces equal to SAR Aperture (sarAperture)
dicePreSarAssemble(configFile,logFile);
% % Perform 1D sar migration 
diceSar1D(configFile,logFile);

% decimate for bed-picking
diceDecimateLevel2(configFile,logFile);

    if strcmpi(get(0,'Diary'),'on')  %% if diary is still on - delete it
        diary off;
        fprintf('diceProcess: Cleaning Up!\n');
        [~,~,logext] = fileparts(logFile);
        if strcmpi(logext,'.log') %% just making sure it's a .log file before deletion
            delete(logFile);
        end
    end
    
fprintf('diceProcess: Done!\n');
catch errMsg
    % In case of error, display it and quit:
    fprintf('sirProcess: Encountered an error\n');
    fprintf('%s\n',errMsg.message);
    fprintf('%s\n',errMsg.identifier);
    for jj = 1:numel(errMsg.stack)
        fprintf('In Function %s, Line %d\n',errMsg.stack(jj).name,errMsg.stack(jj).line)
    end
    warning('on','all');
    disp('diceProcess: Done!')
  	clear close_diary;
    if ~usejava('desktop')  % exit if running in command line
        exit;
    end
end