% Tej Dhakal 02/10/2016
% This file gathers all the processing settings, file paths etc in one
% location so that can be done in multiple steps - this script should be
% called by all processing scripts in order to initialize  required
% variables

%% Radar Settings:  These should not be changed
version = '3.11.1';
try
    [~, commit_hash] = system('git rev-parse HEAD');
    version = [version,'.',commit_hash(1:8)];
catch 
    warning('diceProcessConfig:InvalidPath','Cannot read commit hash - version number not updated');
end
if usejava('desktop')
    debugEnable = true;
else
    debugEnable = false;
end

PRI = 160e-6;       % Pulse Repitition Interval (s)
sampWin = 80e-6;    % Period of time data is collected for
fs = 150e6;         % Sampling Frequency (Hz)
pw1 = 3e-6;         % pulse width for 1st plse (10 us)
pw2 = 1e-6;         % pulse width for 2nd pulse( p2)
bw = 60e6;          % Bandwidth
fc = 38e6;          % Center Frequency
fdemod = fc;        % Baseband demodulation frequency
datalenp1 = 12000;  % Number of samples in 3 us data (integer)
datalenp2 = 12000;  % Number of samples in 1 us data (integer)
tdata = [0:1/fs:sampWin-1/fs]';     % time for each data sample
numHWStacks = 16;   % Number of hardware stacks
numbits=16;         % Number of digitizer bits (integer)
numMetadataSamples=16;	% Number of samples in the data file that are metadata (integer)
c = 299792458;      % m/s
nAir = 1.00029;     % Refractive index of Air    
nIce = 1.78;        % Refractive index of Ice
cAir = c/nAir;      % m/s
cIce = c/nIce;      % m/s
%% Project Campaign and data - These determine the input data location and ouotput product paths so type with care
rootpath = '/local/data/hestia/trunk/';   % Root path for data repository

platform = 'ICEPOD';
platformDir = 'icepod';             % Root dir by project for level2 product and above
theaterDir = 'antarctica';             % Root dir by theatre for level1 product below
rawDir = 'raw';
CampaignYear = '20172018';          % This refers to campaign year name
Campaign = 'AN04';                  % Make sure this is correct- will be used in processed file names
Project = 'RS03';                   % Make sure this is correct- will be used in processed file names
Sensor = 'DICE';
DeploymentNum = 1;                  % Number of deployments this year

% Reference function for the the project

refFile = 'E:\Users\Tej\Desktop\GIT\DICE\DiceTookit\chirps_nl\RefnLSignalsIdeal.mat'; % file that holds the reference signal for dice compression
refvarp1 = 'sig10usref';      % variable name for pulse 1
refvarp2 = 'sig1usref';      % variable name for pulse 2
%% File level Directories
sensorDir = 'dice';
sensorTypeDir = 'radar';
logDir = 'log';
dataDir = 'mat';
matDir = 'mat';
txtDir = 'txt';
rawDataDir = '';
decimatedDir = 'decimated';
tmpPreSarDir = 'tmp';       % This will be created inside level1 dir to store pre-sar data

%% Path for support functions required for dice processing
matlab_common_path = '/local/data/hestia/GIT2/common/Matlab';
if ~(exist(matlab_common_path,'dir')==7)
    warning('diceProcessConfig:InvalidPath','Invalid path for matlab common functions. Please update the path');
    warning('off','diceProcessConfig:InvalidPath');
else
    addpath(matlab_common_path);
end
%% Generate data and projec paths here
sensorRootPath = [rootpath,filesep,platformDir,filesep,theaterDir,filesep,CampaignYear,filesep,sensorTypeDir,filesep,sensorDir,filesep];

% External surface file 
% surfaceFile = 'C:\Users\Tej\Desktop\Icepod\DICE\Matlab\AN1-F001-20141117-SPAN-OLD-FORMAT.txt';
surfaceSource = 'None'; % Tag to identify external surface
lineFileName = [platform,'_',CampaignYear,'_LINES.txt'];   % This file is expected to be in /doc/projectlines dir
forceLinesReprocess = false;
%% Select lines and channels to process
processLines ={'L560','L580'};  % 'gs', {'L380','L650','L530'};     % List of Lines to process e.g. {'L650','L670','L630','L380'}
processChan = [1,3];            % 1 = p1 CH1; 2 = p1 CH2; 3 = p2 CH1; 4 = p2 CH2, can be vector to process multiple channels
%% All setting most likely to be changed are listed 
% Constants
TxInd = 318;        % Index of Transmit signal
doCompression = true;      % if disabled raw signal will be saved instead
uselpf = true;             % use band pass filter for data - typically always
IdealRef = false;           % Choose between Ideal and calibrated chirp
usenotchfilter = true;     % Use notch filters to take  out specific frequencies- change the frequencies in the code as required
notchList = [9.11]*1e6;    % MHz, Add a frequency here to notch it out

% Processing Settings
numAvg = 1;         % Software Stacking Length for this processing (integer)
filesperbatch = 6;  % Number of files compiled into each batch (integer)
maxIceDepthp1 = 1000;   % m, This is the maximum expected real target (bed?)
maxIceDepthp2 = 1000;   % m, typically, same as 10 us
removespikes = true;    % Controls whether to replace spikes in data with small noise

% Flight Parameters
surveyElev = 3500;      % ft, this is average survey elevation in ft, Changing this requires pulse-compression again
surveySpeed = 100;      % m/s, this is nominal speed
maxFlightHours = 10;   % hours, maximum system up time, used fo allocation

% SAR settings
sarAperture = 750;     % m. SAR aperture window ~6 km /min
prfPreSar = 1/(PRI*4*numHWStacks*numAvg);
sarTracesPerFile = ceil(sarAperture/surveySpeed *prfPreSar);
sarOverlapLength= sarTracesPerFile-2;%;ceil(sarTracesPerFile*0.1);     % This is the actual sar aperture 

% Surface 
surfaceFilterLen = 20;     % m, smoothens the surface by this window
surfaceAcqWin = 600;        % m, Used for initial acquisition of surface for each batch
surfaceTrackingWin = 200;       % m, Used for tracking the surface from echo to echo, once acquired 
numSurfPicks = 2;           % surfPick option is applied before picking n surfaces. PlaneHeight always uses the closest surface.
surfThresh = [-50 -50 -50 -50];   % Threshold amplitude value for the peak (dB) ignore any peaks below this.
sarSurf = 'first';          % 'first' or 'maxamp'. Only used if more than 1 surfaces are picked. 'first' uses the surface closest to the a/c
% surfaceChan = 3;            % If this is commented - each channel will use it's own surface
noTxSurfInd = 1;            % This is the index for surface when there is no transmission
surfacePickWindowType = 'adaptive'; % 'adaptive' or 'fixed'
% 'adaptive' window selects random echoes across the file and computes the
% appropriate window for each file. 'fixed' will use the surfacTrackingWin
% and surfaceAcqWin and will be faster (in some cases) but will not work if
% the surface elevation varies by > surfaceTrackingWin/2 within the batch
% (1 minute typical)


% Bed pick data output
bedPickFilePeriod = 5;              % minutes. files will be broken into
bpSubsampleOption = 'average';     % 'decimate' or 'average' are recognized for now

% Data output
overwriteoutput = true;      % If this is set, the output directories will be wiped.
ignoreConfigMatch = true;    % This forces processing even if the configuration for already processed data is identical to current one  
% level 1 processing flags
forceGeolocate = false;
forceSurfPick = true;

% Reference signal processing
usetukey = false;           % enable to use tukey taper on reference signal- also specify taper below
taper = 0.3;            % Adjustable parameter to go with the taper (0-1).
usewin = false;
wintype = 'kaiser';      % specifies which window to use on reference signal before fft, 'cheb' ro 'kaiser'
winparam = 2.5;          % Changes depending on wintype, kaiser- (1-10) typical, for 'cheb' this is rolloff in dBs (70-100)

txThreshold = [-15 -15 -15 -15];    %Threshold power to determin if transmission was on
%% Create a structure with most changed setting so it can be used for comparison with already processed data
configParams.pulseCompress = struct('doCompression',doCompression,'uselpf',uselpf,...
    'IdealRef',IdealRef,'usenotchfilter',usenotchfilter,'notchList',notchList,'numAvg',numAvg,...
    'filesperbatch',filesperbatch,'maxIceDepthp1',maxIceDepthp1,'maxIceDepthp2',maxIceDepthp2,...
    'removespikes',removespikes,'surveyElev',surveyElev,'usetukey',usetukey,'taper',taper,'usewin',usewin,...
    'wintype',wintype,'winparam',winparam,'txThreshold',txThreshold);

if ~exist('surfaceChan','var')
    surfaceChan = [];
end

configParams.pickSurf = struct('surveySpeed',surveySpeed,'surfaceFilterLen',surfaceFilterLen,...
    'numSurfPicks',numSurfPicks,'surfThresh',surfThresh,'surfaceChan',surfaceChan,'TxInd',TxInd,...
    'sarOverlapLength',sarOverlapLength,'noTxSurfInd',noTxSurfInd,'surfaceAcqWin',surfaceAcqWin,'surfaceTrackingWin',surfaceTrackingWin);

configParams.decimateLevel1 = struct('bedPickFilePeriod',bedPickFilePeriod,'bpSubsampleOption',bpSubsampleOption);
configParams.presarAssemble = struct('sarAperture',sarAperture);
configParams.sar1D = struct('nAir',nAir,'nIce',nIce,'TxInd',TxInd,'sarSurf',sarSurf);
configParams.decimateLevel2 = struct('bedPickFilePeriod',bedPickFilePeriod,'bpSubsampleOption',bpSubsampleOption);

%% 
configFnameRoot = 'onfig';  % forcing the configuration script 
if isempty(regexp(mfilename,configFnameRoot,'once'))
    error('diceProcessConfig:invalidFileName',['Filename for the configuraiton script should have "config" or "Config" somewhere\n'...
        'This will make things easier down the line'])
end

dateTimeFormat = 'yyyy/mm/dd HH:MM:SS';

%%
stackDir = num2str(numAvg,'stack%02d');
process1Dir = 'compressed';       % Geolocated compressed data
process2Dir = 'sar';       % SAR processed data

%% Line Directory
projectlinesDir = 'projectlines';
docDir = 'doc';

%% GPS file Info
pntDir = 'pnt';
gpsDir =   'pod-ins';           % sensor directory for GPS data
trajectoryDir = 'trajectory';   % 
gpsProd = 'SPAN_FINAL';         % product to use - this must match the file naming scheme
gpsFileNameFormat = 'AN02_F1006_20151117_SPAN_FINAL.txt'; % Assumes gps file has this format
gpsDataPath = [rootpath,filesep,platformDir,filesep,theaterDir,filesep,CampaignYear,filesep,pntDir,filesep,gpsDir,filesep];
overwriteGPSmatFile = false;    % this ignores existing mat files and overwrites them      % This ignores existing mat files and overwrites them
%%
%%
% Read the line file and determine flights to process
[~,lineFileNameOnly,~]= fileparts(lineFileName);
lineFile = [rootpath,filesep,platformDir,filesep,theaterDir,filesep,CampaignYear,filesep,docDir,filesep,projectlinesDir,filesep,txtDir,filesep,lineFileName];
lineMatFile = convertLineCsv2mat(lineFile,[],[],[],forceLinesReprocess);
clear lineFile;
load(lineMatFile,'FlightNum','LineNum','LineTime');

%% Find Flight Numbers to Process
if numel(processLines)>=1
    if strcmpi(processLines{1},'all')
        processLines = LineNum;
    end
else
    warning('diceProcessConfig:NoLinesToProcess','No Lines Selected for Processing');
end
if any(~ismember(processLines,LineNum))
    fprintf('diceProcessConfig: Invalid Line Name(s)\nPlease remove any imaginary lines from the processLines list\n');
    fprintf('diceProcessConfig: Imaginary Lines:\n');
    InvalidLines = processLines(~ismember(processLines,LineNum));
    for invLn = 1:numel(InvalidLines)
        fprintf('%s\n',InvalidLines{invLn});
    end
    error('diceProcessConfig:invalidLineNum','Invalid Lines');
end

lineInds = ismember(LineNum,processLines);
LineNum = {LineNum{lineInds}}';
LineTime = LineTime(lineInds,:);
FlightNum = FlightNum(lineInds);
Flight = cellstr(num2str(1000*DeploymentNum+FlightNum,'F%04d'));
uniqueFlightNum = unique(FlightNum);
uniqueFlight = cellstr(num2str(1000*DeploymentNum+uniqueFlightNum,'F%04d'));

% startTime=  '2015/11/12 01:00:00';  % UTC time of data to start processing at.  Format: dateTimeFormat Comment this to process start at first file
% endTime=    '2015/11/12 01:20:00';  % UTC time of data to end processing at.  Format: dateTimeFormat Comment this to process up to the last file

%% Data file stuff
rawFileNameFormat = 'AN01_F1002_20141119_084121_0430.dice';
fileFieldSep = '_';
numRawFileFields =  numel(strsplit(rawFileNameFormat,fileFieldSep));
CampaignField = 1;
ProjectField = 1;
FlightNumField = 2;
LineNumField = 2;
dateField = 3;
timeField = 4;
ChannelField = 5;
LevelField = 6;
SensorField = 7;
fileLenTime = 10;      
rawfileExt = 'dice';
rawFileDateFormat = 'yyyymmdd';
rawFileTimeFormat = 'HHMMSS';
outFileDateFormat = 'yyyymmdd';
outFileTimeFormat = 'HHMMSS';
level1FileNameFormat = 'AN02_L0605_1usCh1_20151112_010003_level1.mat';
level2FileNameFormat = 'AN02_L0605_1usCh1_20151112_010003_level2.mat';
level3FileNameFormat = 'AN02_L0605_1usCh1_20151112_0100_0120_level3.pdf';

rawFileKey = ['*.',rawfileExt];

save_append_backup = false;     % whether or not to save a backup copy of data during append - will slow things down if enabled
tmpExt = '.tmp';

% bedPick output Configurations
bedPickDecimate = max(round(16/numAvg),1);      % decimation factor for bed pick data

% This is number of traces before decimation - should 
bedPickTracesPerFile  = bedPickFilePeriod*60*prfPreSar - mod(bedPickFilePeriod*60*prfPreSar,bedPickDecimate);

% Select one channel to use the surface from
if exist('surfaceChan','var') && ~isempty(surfaceChan)
    if ~any(ismember(processChan,surfaceChan))
        error('processConfig:SurfChannelNotPresent','Selected Surface channels is not processed')
    end
end

surfaceAcqIndWin = ceil(surfaceAcqWin*2/cAir*fs); 
surfaceTrackingIndWin = ceil(surfaceTrackingWin*2/cAir*fs);

initSurfInd = ceil(surveyElev/3.28*2/cAir*fs) + TxInd;


surfFilterWin = ceil(surfaceFilterLen/surveySpeed *prfPreSar);
surfFilter = ones(1,surfFilterWin)/surfFilterWin;

% default setup of surfPickOption = 'descend' and numSurfPicks=2 will output
% surfIndRad output that has two rows. the first row is the indices with
% surface return with max power. Inorder to get the topmost surface, use
% min(surfIndRad)
surfPickOption = 'descend'; % No other options available for now. This picks and sorts by amplitude

% Data manipulation
spikeStartIndp2 = [1, 570];    % Starting index of data spike replacement (integer) (1 for p1, 1 for p2)
spikeStopIndp2 = [280, 970];   % Ending index of data spike replacement (integer)  (270 for p1, 270 for p2)
spikeStartIndp1 = [1, 880];    % Starting index of data spike replacement (integer) (2712 for p1, 630 for p2)
spikeStopIndp1 = [280, 1103];  % Ending index of data spike replacement (integer)  (3172 for p1, 990 for p2)
noiseVal = .004;               % standard deviation of noise that replaces the spikes

if(removespikes == 0)
    warning('processConfig:CantTrustSurface',['Radar surface picks cannot be trusted without removing switching spikes!\n',...
        'Set "removespikes" to 1']);
end

% Notch Filter Settings
nfBW = 0.5*1e6;            % MHz, Bandwidth
numOrder = 8;              % Bandwidth Attenuation -dB at nfBW

process_metadata = false;   % parses and stores metadata for each echo. Disabling this might speed things up a bit..

prodLevel.name = '?';
prodLevel.desc = {'?'};     % just a place holder - will be updated in each processing script
prodLevel.time = {'?'};
configmatch = 0;        % leave this to 0
p1Len = '3us';          % Text describing the signal type/length - used to name processed files
p2Len = '1us';          %
processChanAttbr = {[p1Len,'Ch1'];[p1Len,'Ch2'];[p2Len,'Ch1'];[p2Len,'Ch2']};
%% Generate/Load Reference signal
% Load reference signals
if ~isempty(refFile)
    if ~exist(refFile,'file')
        warning('processConfig:RefFileDoesNotExist','Reference file %s not found',refFile);
        clear refFile;
    end
end

if exist('refFile','var') && ~IdealRef
    load(refFile);
    sigp1ref = eval(refvarp1);
    sigp2ref = eval(refvarp2);
    
    if isrow(sigp1ref)
        sigp1ref = sigp1ref';
    end
    if isrow(sigp2ref)
        sigp2ref = sigp2ref';
    end
    
    Nrefp2 = length(sigp2ref);
    Nrefp1 = length(sigp1ref);
    sigp2ref = sigp2ref/max(sigp2ref);
    sigp1ref = sigp1ref/max(sigp1ref);
    t1 = linspace(0,Nrefp1-1,Nrefp1)'/fs;
    t2 = linspace(0,Nrefp2-1,Nrefp2)'/fs;
else
    % Generate Reference signals
    t1 = [0:1/fs:pw1-1/fs]';
    t2 = [0:1/fs:pw2-1/fs]';
    % compute chirp phase
    freq1 = fc-bw/2 + bw/pw1*t1/2;
    freq2 = fc-bw/2 + bw/pw2*t2/2;
    sigp1ref = exp(1j*2*pi*freq1.*t1);
    sigp2ref = exp(1j*2*pi*freq2.*t2);
    clear freq1 freq2
    % Figure out size of reference signals:
    Nrefp2 = length(sigp2ref);
    Nrefp1 = length(sigp1ref);
end
% Let us compute the truncation length to twice the minimum allowed ice depth (for bed multiple)
% This speeds up processing and also reduces volume of data to be stored.
% uses c_ice = 1.68e8 m/s
mintruncLenp1 = ceil((surveyElev*2/3.28/3e8*fs+maxIceDepthp1*2/1.68e8*fs)*2.2);
mintruncLenp2 = ceil((surveyElev*2/3.28/3e8*fs+maxIceDepthp2*2/1.68e8*fs)*2.2);
truncLenp1 = mintruncLenp1 + Nrefp1 -1;
while(1)
    if(any(factor(truncLenp1+Nrefp1 -1)>5))
        truncLenp1 = truncLenp1+1;
    else
        break;
    end
end
truncLenp2 = mintruncLenp2 + Nrefp2 -1;
while(1)
    if(any(factor(truncLenp2+Nrefp2 -1)>5))
        truncLenp2 = truncLenp2+1;
    else
        break;
    end
end

%% Just one more thing - keep track of how many times the script has been called
% runCount resets everytime diceProcessConfig.m is changed
% runCount is used to keep a single copy of the config script in the log
% directory
callername = dbstack;
clear ans;
if numel(callername)>1  % only updates the count if some other funciton is calling on diceProcessConfigru
    save([mfilename,'.mat']);    % save all variables in the workspace
    runCount = runtracker();
end
function count = runtracker()
persistent runNum;
if isempty(runNum)
    runNum = 1;
else
    runNum = runNum + 1;
end
count = runNum;
end

