function [] = diceSar1D(newConfig,logFile)
% Parameters: newConfig is the name of the script that initializes all required variables
% if no argument is passed "diceProcessConfig.m" is run by default
% This performs migration GeoLocated, pulseCompressed dat using
% an algorithm based on Hiliere et al, 2007.  It assumes that the
% ice surfaces can be described as flat within the chosen Synthetic Aperture.
% Processsing:
%   - Uses Radar surface by default and ice refractive index to compute a
%   migration curve on one file at a time
%   - Three files are loaded at a time to remove edge effects
%   - First are and last files are not migrated.

% Tej Dhakal, 10/12/2014- modified to work with DICE data
% Optimized for IceShelf mapping ROSETTA for reduced data rate
% 03/23/2016 - Code now picks up processing where it left off. last two
% files will still be reprocessed.
try
    fprintf('************** diceSar1D.m ****************\n')
    if nargin <1
        configScript = 'diceProcessConfig.m';
        logFile = 'diceProcessLog.log';
        diary off;
        if exist(logFile,'file')
            warning('diceSar1D:existingLogFile','A logfile %s already exists!\n, Please cleanup the logs from %s for future processing',logFile,pwd);
            [~,logFileNameOnly,logFileExt] = fileparts(logFile);
            logFile = [logFileNameOnly,datestr(now,'_yyyymmdd_hhMMss'),logFileExt];
        end
        diary(logFile);
        localDiary = true;
    elseif nargin <2
        configScript = newConfig;
        logFile = 'diceProcessLog.log';
        diary off;
        if exist(logFile,'file')
            warning('diceSar1D:existingLogFile','A logfile %s already exists!\n, Please cleanup the logs from %s for future processing',logFile,pwd);
            [~,logFileNameOnly,logFileExt] = fileparts(logFile);
            logFile = [logFileNameOnly,datestr(now,'_yyyymmdd_hhMMss'),logFileExt];
        end
        diary(logFile);
        localDiary = true;
    else
        configScript = newConfig;
        localDiary = false;
    end
    
    run(configScript);
    tic
    configParamsCurr = configParams;
    %% Parameters:
    
    tSamp=1/fs;             % s
    
    %% Work:
    
    % Dependent parameters:
    lambda_air=cAir/fs;
    dy_air=.5*cAir*tSamp;
    dy_ice=.5*cIce*tSamp;
    epsilon=nIce^2;
    midpoint=floor((sarTracesPerFile+2*sarOverlapLength)/2);
    maxtruncLen = max(truncLenp1,truncLenp2);
    truncLen = [truncLenp1,truncLenp1,truncLenp2,truncLenp2];
    
    %% Pre allocate everything
    CG1=zeros(maxtruncLen,sarTracesPerFile);
    CG2=zeros(maxtruncLen,sarTracesPerFile);
    CG3=zeros(maxtruncLen,sarTracesPerFile);
    utc1 = zeros(1,sarTracesPerFile);
    utc2 = zeros(1,sarTracesPerFile);
    utc3 = zeros(1,sarTracesPerFile);
    Lat1 = zeros(1,sarTracesPerFile);
    Lon1 = zeros(1,sarTracesPerFile);
    Elev1 = zeros(1,sarTracesPerFile);
    Lat2 = zeros(1,sarTracesPerFile);
    Lon2 = zeros(1,sarTracesPerFile);
    Elev2 = zeros(1,sarTracesPerFile);
    Lat3= zeros(1,sarTracesPerFile);
    Lon3 = zeros(1,sarTracesPerFile);
    Elev3= zeros(1,sarTracesPerFile);
    SurfElev1 = zeros(1,sarTracesPerFile);
    SurfElev2 = zeros(1,sarTracesPerFile);
    SurfElev3 = zeros(1,sarTracesPerFile);
    SurfInds1 = zeros(numSurfPicks,sarTracesPerFile);
    SurfInds2 = zeros(numSurfPicks,sarTracesPerFile);
    SurfInds3 = zeros(numSurfPicks,sarTracesPerFile);
    PlaneHeight1 = zeros(1,sarTracesPerFile);
    PlaneHeight2 = zeros(1,sarTracesPerFile);
    PlaneHeight3 = zeros(1,sarTracesPerFile);
    p = zeros(sarTracesPerFile,3);
    Lat=zeros(1,sarTracesPerFile);
    Lon=zeros(1,sarTracesPerFile);
    FlightElev=zeros(1,sarTracesPerFile);
    SurfElev=zeros(1,sarTracesPerFile);
    SurfInds = zeros(numSurfPicks,sarTracesPerFile);
    PlaneHeight=zeros(1,sarTracesPerFile);
    echoTimeUTC=zeros(1,sarTracesPerFile);
    DeltaTau=zeros(1,sarOverlapLength);
    DeltaPhi=zeros(1,sarOverlapLength);
    DeltaRa=zeros(1,sarOverlapLength);
    DeltaRi=zeros(1,sarOverlapLength);
    DeltaRt=zeros(1,sarOverlapLength);
    S=zeros(1,sarOverlapLength);
    Ref=zeros(1,sarTracesPerFile+2*sarOverlapLength);
    RefTransform = zeros(1,sarTracesPerFile+2*sarOverlapLength);
    convolution = zeros(1,sarTracesPerFile+2*sarOverlapLength);
    Block = zeros(maxtruncLen,sarTracesPerFile+2*sarOverlapLength);
    BlockTransform = zeros(maxtruncLen,sarTracesPerFile+2*sarOverlapLength);
    
    ApertureSize=zeros(maxtruncLen,1);
    
    truncLengths = {truncLenp1;truncLenp1;truncLenp2;truncLenp2};
    if numAvg>1
        warning('numAvg = %d, Is this the right data for SAR?\n',numAvg)
        pause(2);
    end
    for ll = 1:numel(LineNum)   %% process each line separately
        sarMigratedTime = now;
        configFileCopied = false;   % Just to make sure config file is copied on first processed channel only
        startTime = LineTime(ll,1);
        stopTime = LineTime(ll,2);
        preSarDataPath = [sensorRootPath,filesep,process1Dir,filesep,stackDir,filesep,Flight{ll},filesep,dataDir,filesep,tmpPreSarDir,filesep];
        sarDataPath = [sensorRootPath,filesep,process2Dir,filesep,stackDir,filesep,LineNum{ll},filesep,dataDir,filesep];
        processLogPath = [sensorRootPath,filesep,process2Dir,filesep,stackDir,filesep,LineNum{ll},filesep,logDir,filesep];
        
        if exist(preSarDataPath,'dir')==0
            warning('diceSar1D:noLevel1Dir','Level1 Flight %s tmp Directory does not exists!',Flight{ll});
            continue;
        else
            anyFiles=dir([preSarDataPath,'*',Flight{ll},'*',Sensor,'*.mat']);
            if isempty(anyFiles)
                warning('diceSar1D:noLevel1Files','Level1 Flight %s Directory has no data to migrate!',LineNum{ll});
                continue;
            elseif numel(anyFiles)<3
                warning('diceSar1D:notEnoughFiles','Cannot do SAR processing with only %d Files in %s!',numel(anyFiles),LineNum{ll})
                continue;
            end
        end
        
        if exist(sarDataPath,'dir')
            outputDirExists = true;         % A line directory already exists
            createdNewOutputDir = false;    % A new dir was not created
            dataFlist = dir([sarDataPath,'*',LineNum{ll},'*.mat']);
            if numel(dataFlist)>0
                fprintf('diceSar1D: Found existing data files\n');
                pcFlist = dir([sarDataPath,'*',configFnameRoot,'*.mat']);
                if numel(pcFlist)>0
                    [~,sx] = sort([pcFlist.datenum]);
                    pcFlist = pcFlist(sx);
                    fprintf('diceSar1D: Checking existing configuration data...\n')
                    load([sarDataPath,pcFlist(end).name],'configParams');
                    if isequal(configParamsCurr,configParams)
                        if ignoreConfigMatch && overwriteoutput
                            warning('diceSar1D:FlightDirExists',['All existing files in %s will be deleted!\n',...
                                'If this is not intended, press Ctrl + C to cancel'],LineNum{ll});
                            
                            fprintf('diceSar1D: Files will be deleted in %d seconds',9);
                            for ss = 9:-1:0
                                fprintf('\b\b\b\b\b\b\b\b\b%d seconds',ss);
                                pause(1);
                            end
                            fprintf('\ndiceSar1D: Deleted!\n')
                            delete([sarDataPath,'*',LineNum{ll},'*.mat']);
                            delete([sarDataPath,'*onfig*.mat']);
                        else
                            fprintf(['diceSar1D: Current settings are identical to the one %s processed data used.\n\t',...
                                'Existing %s data will not be re-processed.\n\tTo force processing, Remove configuration mat file from\n\t%s\n'],LineNum{ll},LineNum{ll},sarDataPath);
                        end
                    else
                        if overwriteoutput == 1
                            warning('diceSar1D:FlightDirExists',['All existing files in %s will be deleted!\n',...
                                'If this is not intended, press Ctrl + C to cancel'],LineNum{ll});
                            
                            fprintf('diceSar1D: Files will be deleted in %d seconds',9);
                            for ss = 9:-1:0
                                fprintf('\b\b\b\b\b\b\b\b\b%d seconds',ss);
                                pause(1);
                            end
                            fprintf('\ndiceSar1D: Deleted!\n')
                            delete([sarDataPath,'*',LineNum{ll},'*.mat']);
                            delete([sarDataPath,'*onfig*.mat']);
                        else
                            warning('diceSar1D:LineDirExists','Line %s Directory already exists - Line will not be processed!\n Set "overwriteoutput = 1" to force processing',LineNum{ll});
                            continue;
                        end
                    end
                else    %if for any reason there is no configuration file
                    if overwriteoutput == 1
                        warning('diceSar1D:FlightDirExists',['All existing files in %s will be deleted!\n',...
                            'If this is a mistake, press Ctrl + C to cancel'],LineNum{ll});
                        
                        fprintf('diceSar1D: Files will be deleted in %d seconds',9);
                        for ss = 9:-1:0
                            fprintf('\b\b\b\b\b\b\b\b\b%d seconds',ss);
                            pause(1);
                        end
                        fprintf('\ndiceSar1D: Deleted!\n')
                        delete([sarDataPath,'*',LineNum{ll},'*.mat']);
                        delete([sarDataPath,'*onfig*.mat']);
                    else
                        warning('diceSar1D:LineDirExists','Line %s Directory already exists - Line will not be processed!\n Set "overwriteoutput = 1" to force processing',LineNum{ll});
                        continue;
                    end
                end
            end
        else
            outputDirExists = false;    % will be set to true when new dir is created
            createdNewOutputDir = false;
        end
        
        
        fprintf('diceSar1D: Processing Line %s\n',LineNum{ll})
        %% Find the files for each line and prcesse - if there is break in the data process in separate batches
        for chan_num = 1:numel(processChan)
            fprintf('diceSar1D: Processing %s data\n',processChanAttbr{processChan(chan_num)})
            CG=zeros(truncLen(processChan(chan_num)),sarTracesPerFile);
            twttTx = zeros(truncLen(processChan(chan_num)),1);
            VertScale = zeros(truncLen(processChan(chan_num)),1);
            level1Files=dir([preSarDataPath,['*',processChanAttbr{processChan(chan_num)},'*.mat']]);
            numfiles=length(level1Files);
            if(numfiles == 0)
                warning(['diceSar1D:','no',processChanAttbr{processChan(chan_num)},'Files'],'No level1 %s files found for %s!',processChanAttbr{processChan(chan_num)},LineNum{ll})
                continue;
            elseif numfiles < 3
                warning('diceSar1D:notEnoughFilesinLine','Cannot do SAR processing with only %d Files for %s data in %s!',numfiles,processChanAttbr{processChan(chan_num)},LineNum{ll})
                continue;
            else
                for ii = 1:numfiles
                    fileNameFields = strsplit(level1Files(ii).name,fileFieldSep);
                    try
                        old_fileformat = 0;
                        level1Files(ii).utc = matlab2unixtime(datenum([fileNameFields{dateField},fileNameFields{timeField}],[outFileDateFormat,outFileTimeFormat]));
                    catch
                        level1Files(ii).utc = matlab2unixtime(datenum([fileNameFields{dateField+1},fileNameFields{timeField+1}],[outFileDateFormat,outFileTimeFormat]));
                        old_fileformat = 1;
                        warning('diceSar1D:oldFileNameFormat','Data Files are in old file name format')
                        warning('off','diceSar1D:oldFileNameFormat')
                    end
                end
                [~,sx] = sort([level1Files.utc]);
                level1Files = level1Files(sx);
            end
            
            %% If there are already level2 files present, lets figure out where to pick up from
            if outputDirExists && ~createdNewOutputDir % if a new directory was created for previous channel in the same run this check will be skipped
                level2dataFiles=dir([sarDataPath,['*',processChanAttbr{processChan(chan_num)},'*.mat']]);
                numfiles=numel(level2dataFiles);
                if(numfiles >1)
                    for ii = 1:numfiles
                        fileNameFields = strsplit(level2dataFiles(ii).name,fileFieldSep);
                        try
                            level2dataFiles(ii).utc = matlab2unixtime(datenum([fileNameFields{dateField},fileNameFields{timeField}],[outFileDateFormat,outFileTimeFormat]));
                        catch
                            level2dataFiles(ii).utc = matlab2unixtime(datenum([fileNameFields{dateField+1},fileNameFields{timeField+1}],[outFileDateFormat,outFileTimeFormat]));
                            warning('diceSar1D:oldFileNameFormat','Data Files are in old file name format')
                            warning('off','diceSar1D:oldFileNameFormat')
                        end
                    end
                    [~,sx] = sort([level2dataFiles.utc]);
                    level2dataFiles = level2dataFiles(sx);
                    startTime= level2dataFiles(numfiles-1).utc;
                end
            end
            
            % this picks the start file one file before the start time and
            % stop file one file after the end time
            
            startfile=find([level1Files.utc]<startTime,1,'last');
            endfile=find([level1Files.utc]>stopTime,1,'first');
            
            % if end file is empty - it might mean this line goes beyond the
            % last file
            if isempty(endfile)
                endfile=numel(level1Files);
            end
            
            if endfile <3
                warning('diceSar1D:notEnoughFilesinLine','Cannot do SAR processing with only %d Files for %s data!',endfile,LineNum{ll})
                continue;
            elseif abs(level1Files(endfile).utc - level1Files(endfile-1).utc)>sarTracesPerFile/prfPreSar*1.1
                endfile = endfile-1;
            end
            
            % line starts before any available data
            if isempty(startfile)
                startfile=1;
            end
            
            % check for the case when there is no data available for the given
            % line but there is data before the line time starts, in this case the end file and start file would be the last
            % file in the flight
            if (endfile - startfile < 1)
                warning('diceSar1D:NoFilesSelected','No Files found for Line %s! Make sure Level1 data exists for this line',LineNum{ll})
                continue;
            elseif (endfile - startfile < 2)
                warning('diceSar1D:TooFewFilesInALine','Need at least 3 files for Sar Processing, only %d files available for %s',endfile-startfile+1,LineNum{ll})
            end
            
            % take care of the case when there is a big gap between start file
            % time and the next file - might happen if the level0 processed
            % files are not continuous
            if (abs(level1Files(startfile+1).utc - level1Files(startfile).utc)>sarTracesPerFile/prfPreSar*1.1)
                startfile = startfile+1;
            end
            
            if(endfile - startfile <2)
                warning('diceSar1D:TooFewFilesInALine','Need at least 3 files for Sar Processing, only %d files available for %s',endfile-startfile+1,LineNum{ll})
                continue;
            end
            if ~outputDirExists
                mkdir(sarDataPath)
                outputDirExists = true;
                createdNewOutputDir = true;
            end
            
            if ~configFileCopied
                % Looks like processing will continue for this line, let's copy the
                % config file
                [~,configFilename,~] = fileparts(configScript);
                copyfile([configFilename,'.mat'],[sarDataPath,configFilename,'.mat'])
                configFileCopied = true;
            end
            
            fTime = [level1Files(startfile:endfile).utc];
            dTime = diff(fTime);
            batchind = find(dTime > (ceil(sarTracesPerFile/prfPreSar) + 2));
            numbatches = numel(batchind)+1;
            batchmin(1) = startfile;
            if isempty(batchind)
                batchmax(1) = endfile;
            else
                batchmax(1) = batchind(1)+startfile-1;
                batchmin(numbatches) = batchind(end)+startfile;
                batchmax(numbatches) = endfile;
            end
            for bb=2:numbatches-1
                batchmin(bb) = batchind(bb-1)+startfile;
                batchmax(bb) = batchind(bb)+startfile-1;
            end
            if numbatches>1
                warning('diceSar1D:FilesNotContiguous','Files are not contiguous!');
            end
            
            minfNum=batchmin;
            maxfNum=batchmax;
            
            %%
            for batch = 1:numbatches
                if(maxfNum - minfNum +1) <3
                    warning('diceSar1D:notEnoughFilesinBatch','Cannot do SAR processing with only %d Files for %s data, batch %d in %s!',maxfNum(batch) - minfNum(batch) +1,processChanAttbr{processChan(chan_num)},batch,LineNum{ll})
                    continue;
                end
                disp('Loading first two files...')
                starter = minfNum(batch);
                load([preSarDataPath,level1Files(starter).name],'radecho','echotimeutc','surfIndRad','lat','lon','hell','planeHeight');
                CG1(1:truncLen(processChan(chan_num)),:)=radecho;
                utc1(:) = echotimeutc;
                Lat1(:) = lat;
                Lon1(:) = lon;
                Elev1(:) = hell;
                SurfInds1(:,:) = surfIndRad;
                PlaneHeight1(:) = planeHeight;
                load([preSarDataPath,level1Files(starter+1).name],'radecho','echotimeutc','surfIndRad','lat','lon','hell','planeHeight','prodLevel');
                               
%                 backward compatibility
                if ~iscell(prodLevel.desc)
                    oldProdLevel = prodLevel;
                    clear prodLevel;
                    prodLevel.name = oldProLevel.name;
                    prodLevel.desc = {oldProdLevel.desc};
                    prodLevel.time = {'?'};
                end
                prodLevel2 = prodLevel;
                
                CG2(1:truncLen(processChan(chan_num)),:)= radecho;
                utc2(:) = echotimeutc;
                Lat2(:) = lat;
                Lon2(:) = lon;
                Elev2(:) = hell;
                SurfInds2(:,:) = surfIndRad;
                PlaneHeight2(:) = planeHeight;
                
                % Loop through files
                for jj=minfNum(batch)+1:maxfNum(batch)-1
                    fprintf('diceSar1D: Processing File %d of %d of batch %d of %d\n',jj-minfNum(batch),maxfNum(batch)-minfNum(batch)-1,batch,numbatches)
                    % load LG data and position data from next file:
                    load([preSarDataPath,level1Files(jj+1).name],'radecho','echotimeutc','surfIndRad','lat','lon','hell','planeHeight','twtt','prodLevel');
                    %                 backward compatibility
                    if ~iscell(prodLevel.desc)
                        oldProdLevel = prodLevel;
                        clear prodLevel;
                        prodLevel.name = oldProLevel.name;
                        prodLevel.desc = {oldProdLevel.desc};
                        prodLevel.time = {'?'};
                    end
                    prodLevel3 = prodLevel;
                    
                    
                    CG3(1:truncLen(processChan(chan_num)),:)= radecho;
                    utc3(:) = echotimeutc;
                    Lat3(:) = lat;
                    Lon3(:) = lon;
                    Elev3(:) = hell;
                    SurfInds3(:,:) = surfIndRad;
                    PlaneHeight3(:) = planeHeight;
                    
                    % Start SAR stuff here
                    [p(:,1),p(:,2),p(:,3)]= geodetic2enu(Lat2',Lon2',Elev2',Lat2(1),Lon2(1),Elev2(1),wgs84Ellipsoid,'degrees');
                    dx= median((diff(p(:,1)).^2+diff(p(:,2)).^2).^.5);
                    
                    % Assign scalar indices and height:
                    if strcmp(sarSurf,'first')
                        surfind=round(nanmean(min(SurfInds2,[],1)));
                    else
                        surfind=round(nanmean(SurfInds2(1,:)));
                        planeHeight2(:) = twtt(SurfInds2(1,:)-TxInd)*cAir/2;
                    end
                    h=nanmean(PlaneHeight2);
                    if ~isnan(surfind)  % true if tx was off - better test than all(~txOn)
                        twttTx(:) = twtt-twtt(TxInd);
                        % Create vertical scales:
                        VertScale(1:surfind)=dy_air*linspace(-(surfind-1),0,surfind)';
                        VertScale(surfind+1:end)=dy_ice*linspace(1,truncLen(processChan(chan_num))-surfind,truncLen(processChan(chan_num))-surfind)';
                        
                        Block(1:truncLen(processChan(chan_num)),:) = [CG1(1:truncLen(processChan(chan_num)),sarTracesPerFile-sarOverlapLength+1:end),CG2(1:truncLen(processChan(chan_num)),:),CG3(1:truncLen(processChan(chan_num)),1:sarOverlapLength)];
                        BlockTransform(1:truncLen(processChan(chan_num)),:) = fft(Block(1:truncLen(processChan(chan_num)),:),[],2);
                        
                        % Define the Reference Function and perform SAR for each row
                        for sample=TxInd+1:truncLen(processChan(chan_num))
                            
                            % Define depth of row:
                            d=(sample-surfind)*dy_ice;
                            
                            % Calculate move-out curve:
                            if sample>surfind
                                for trace=1:sarOverlapLength
                                    x=trace*dx;
                                    theseroots=roots([epsilon-1,-2*x*(epsilon-1),x^2*(epsilon-1)+h^2*epsilon-d^2,2*x*d^2,-1*x^2*d^2]);
                                    ind=find(abs(theseroots)<x,1,'last');
                                    S(trace)=theseroots(ind);
                                    DeltaRa(trace)=2*(sqrt(h^2+(x-S(trace))^2)-h);
                                    DeltaRi(trace)=2*(sqrt(d^2+S(trace)^2)-d);
                                    DeltaRt(trace)=DeltaRa(trace)+nIce*DeltaRi(trace);
                                    DeltaTau(trace)=DeltaRt(trace)/cAir;
                                    DeltaPhi(trace)=-2*pi*DeltaRt(trace)/lambda_air;
                                    if DeltaTau(trace)>tSamp
                                        ind1=trace;
                                        break
                                    end
                                end
                            else
                                thisheight=(sample-TxInd)*dy_air;
                                DeltaRa(:)=(thisheight^2+(dx*linspace(1,sarOverlapLength,sarOverlapLength)).^2).^.5-thisheight;
                                DeltaRi(:)=zeros(1,sarOverlapLength);
                                DeltaRt(:)=DeltaRa;
                                DeltaTau(:)=DeltaRt/cAir;
                                DeltaPhi(:)=-2*pi*DeltaRt/lambda_air;
                                ind1=find(DeltaTau>tSamp,1,'first');
                            end
                            
                            % Assign aperture size:
                            ApertureSize(sample)=(2*ind1+1)*dx;
                            % Create reference function:
                            Win=tukeywin(2*ind1+1,1)'; %set the Tukey window taper=1
                            Ref(midpoint-ind1:midpoint+ind1)=Win.*exp(1i*[fliplr(DeltaPhi(1:ind1)),0,DeltaPhi(1:ind1)]);
                            Ref(:)=Ref/sum(abs(Ref));  % scale the reference function
                            
                            % Perform convolution:
                            RefTransform(:)=fft(fliplr(conj(Ref)),[],2);
                            convolution(:)=fftshift(ifft(RefTransform.*BlockTransform(sample,:)),2);
                            CG(sample,:)=convolution(midpoint-floor(sarTracesPerFile(1)/2)+1:midpoint+ceil(sarTracesPerFile(1)/2));
                        end
                        
                        % Add a nonmigrated buffer above the breakthrough:
                        CG(1:TxInd,:)=Block(1:TxInd,midpoint-floor(sarTracesPerFile/2)+1:midpoint+ceil(sarTracesPerFile/2));
                        
                        % Assemble variables for central file:
                        %             X=X2;
                        %             Y=Y2;
                                                
                        prodLevel2.name = 'level2';
                        prodLevel2.desc{numel(prodLevel2.desc)+1} = 'sar';
                        prodLevel2.desc{numel(prodLevel2.time)+1} = datestr(sarMigratedTime);
                        
                        prodLevel = prodLevel2;
                        
                        Lat(:)=Lat2;
                        Lon(:)=Lon2;
                        FlightElev(:)=Elev2;
                        SurfElev(:)=SurfElev2;
                        PlaneHeight(:)=PlaneHeight2;
                        echoTimeUTC(:) = utc2;
                        SurfInds(:,:) = SurfInds2;

                        
                        % Save the results:
                        if ~old_fileformat
                            save([sarDataPath,Project,fileFieldSep,LineNum{ll},fileFieldSep,datestr(unix2matlabtime(echoTimeUTC(1)),...
                                [outFileDateFormat,fileFieldSep,outFileTimeFormat]),fileFieldSep,fileNameFields{5},fileFieldSep,prodLevel.name,fileFieldSep,Sensor,'.mat'],...
                                'CG','Lat','Lon','FlightElev','ApertureSize','twttTx','VertScale','PlaneHeight','TxInd','cAir','cIce','dx','dy_air','dy_ice','fs','h','nIce','tSamp','SurfInds','echoTimeUTC','sarSurf','prodLevel','sarMigratedTime');
                        else
                            save([sarDataPath,Project,fileFieldSep,LineNum{ll},fileFieldSep,datestr(unix2matlabtime(echoTimeUTC(1)),...
                                [outFileDateFormat,fileFieldSep,outFileTimeFormat]),fileFieldSep,fileNameFields{3},fileFieldSep,prodLevel.name,fileFieldSep,Sensor,'.mat'],...
                                'CG','Lat','Lon','FlightElev','ApertureSize','twttTx','VertScale','PlaneHeight','TxInd','cAir','cIce','dx','dy_air','dy_ice','fs','h','nIce','tSamp','SurfInds','echoTimeUTC','sarSurf','prodLevel','sarMigratedTime');
                        end
                    else
                        warning('diceSar1D:noSurfaceFound','Skipping %s, No Surface found',level1Files(jj+1).name);
                    end
                    % Prepare for next file:
                    CG1(1:truncLen(processChan(chan_num)),:)=CG2(1:truncLen(processChan(chan_num)),:);
                    CG2(1:truncLen(processChan(chan_num)),:)=CG3(1:truncLen(processChan(chan_num)),:);
                    SurfElev1(:)=SurfElev2;
                    SurfElev2(:)=SurfElev3;
                    SurfInds1(:,:)=SurfInds2;
                    SurfInds2(:,:)=SurfInds3;
                    Lat1(:)=Lat2;
                    Lat2(:)=Lat3;
                    Lon1(:)=Lon2;
                    Lon2(:)=Lon3;
                    Elev1(:)=Elev2;
                    Elev2(:)=Elev3;
                    utc1(:) = utc2;
                    utc2(:) = utc3;
                    PlaneHeight1(:)=PlaneHeight2;
                    PlaneHeight2(:)=PlaneHeight3;
                    prodLevel2 = prodLevel3;
                end
            end
        end
        warning('on','all');
        diary off;
        if(configFileCopied)    % if yes, then the line was most likely processed
            [~,logFileNameOnly,logFileExt] = fileparts(logFile);
            if ~exist(processLogPath,'dir')
                mkdir(processLogPath);
            end
            movefile(logFile,[processLogPath,logFileNameOnly,fileFieldSep,'sar',datestr(sarMigratedTime,'_yyyymmdd_hhMMss'),logFileExt]);
            [~,configFileNameOnly,~] = fileparts(configScript);
            copyfile(configScript,[processLogPath,configFileNameOnly,fileFieldSep,'sar',datestr(sarMigratedTime,'_yyyymmdd_hhMMss'),'.m.log']);
            else % delete existing diary and start a fresh one
            [~,~,logext] = fileparts(logFile);
            if strcmpi(logext,'.log') %% just making sure it's a .log file before deletion
                delete(logFile);
            end
        end
        diary(logFile);
    end
    if localDiary % if diary was locally started - delete it
        if strcmpi(get(0,'Diary'),'on')
            diary off;
            [~,~,logext] = fileparts(logFile);
            if strcmpi(logext,'.log') %% just making sure it's a .log file before deletion
                delete(logFile);
            end
        end
    end
    fprintf('diceSar1D: Done!\n')
    toc
catch errMsg
    fprintf('diceSar1D: Encountered an error\n');
    fprintf('%s\n',errMsg.message);
    fprintf('%s\n',errMsg.identifier);
    for jj = 1:numel(errMsg.stack)
        fprintf('In Function %s, Line %d\n',errMsg.stack(jj).name,errMsg.stack(jj).line)
    end
    warning('on','all');
    % move the log file
    diary off;
    if exist(logFile,'file') && exist('processLogPath','var')
        [~,logFileNameOnly,logFileExt] = fileparts(logFile);
        if ~exist(processLogPath,'dir')
            mkdir(processLogPath);
        end
        movefile(logFile,[processLogPath,logFileNameOnly,'_crash',datestr(now,'_yyyymmdd_hhMMss'),logFileExt]);
    end
    if debugEnable
        fprintf('diceSar1D: Halting for debugging\n')
        keyboard
    end
end
end
