function [] = diceSingleSurf(newConfig,logFile)
% Parameters: newConfig is the name of the script that initializes all required variables
% if no argument is passed "diceProcessConfig.m" is run by default
% This code reads the geoLocated data and processed further to get it ready
% for SAR
%   - Replaces Surface from each channel with one from a selected channel

%% Now that all level1 files have surface info in them,
% If use of surface from one particular channel is desired, that is taken
% care of here. This runs through all the files and replaces the
% surface and plane height varaibles from the selected channel.
% This is desirable when there is rising terrain or just low flight in
% general due to which significant portion of surface return for the longer pulse
% (p1) is blocked by switching spikes.
% This is enabled by setting "surfaceChan" variable to the desired channel
try
fprintf('************** diceSingleSurf.m ****************\n')
if nargin <1
    configScript = 'diceProcessConfig.m';
    logFile = 'diceProcessLog.log';
    diary off;
    if exist(logFile,'file')
        warning('diceSingleSurf:existingLogFile','A logfile %s already exists!\n, Please cleanup the logs from %s for future processing',logFile,pwd);
        [~,logFileNameOnly,logFileExt] = fileparts(logFile);
        logFile = [logFileNameOnly,datestr(now,'_yyyymmdd_hhMMss'),logFileExt];
    end
    diary(logFile);
    localDiary = true;
elseif nargin <2
    configScript = newConfig;
    logFile = 'diceProcessLog.log';
    diary off;
    if exist(logFile,'file')
        warning('diceSingleSurf:existingLogFile','A logfile %s already exists!\n, Please cleanup the logs from %s for future processing',logFile,pwd);
        [~,logFileNameOnly,logFileExt] = fileparts(logFile);
        logFile = [logFileNameOnly,datestr(now,'_yyyymmdd_hhMMss'),logFileExt];
    end
    diary(logFile);
    localDiary = true;
else
    configScript = newConfig;
    localDiary = false;
end

run(configScript);
if isempty(surfaceChan)
    fprintf('diceSingleSurf: No Single surface selected - skippin\n')
    return;
end

tic
%% Load all the files in the level1 directory
for ff = 1:numel(uniqueFlightNum)
    %% Check files and directories:
    FlightNo = uniqueFlight{ff};
    singleSurfReplacedTime = now;
    configFileCopied = false;
    surfPickedDataPath = [sensorRootPath,filesep,process1Dir,filesep,stackDir,filesep,FlightNo,filesep,dataDir,filesep];
    processLogPath = [sensorRootPath,filesep,process1Dir,filesep,stackDir,filesep,FlightNo,filesep,logDir,filesep];    
    surf_chan_num = find(processChan == surfaceChan);
    if exist(surfPickedDataPath,'dir')==0
        warning('diceSingleSurf:noLevel1Dir','Level1 Flight %s Directory does not exists!',FlightNo);
        continue;
    end
    fprintf('diceSingleSurf: Processing Flight %s\n',FlightNo)
    for chan_num = 1:numel(processChan)
                tmpFiles = dir([surfPickedDataPath,['*',processChanAttbr{processChan(chan_num)},'*.mat',tmpExt]]);
        if ~isempty(tmpFiles)
            fprintf('diceSingleSurf: Recovering corrupted mat files from previous run...\n');
            for tt = 1:numel(tmpFiles)
                copyfile([surfPickedDataPath,tmpFiles(tt).name],[surfPickedDataPath,tmpFiles(tt).name(1:end-numel(tmpExt))])
                delete([surfPickedDataPath,tmpFiles(tt).name]);
            end
        end
        
        level1Files{processChan(chan_num)}=dir([surfPickedDataPath,['*',processChanAttbr{processChan(chan_num)},'*.mat']]);
        numfiles=numel(level1Files{processChan(chan_num)});
        if(numfiles == 0)
            warning(['diceSingleSurf:','no',processChanAttbr{processChan(chan_num)},'Files'],'No level1 %s files found!',processChanAttbr{processChan(chan_num)})
        else
            for ii = 1:numfiles
                fileNameFields = strsplit(level1Files{processChan(chan_num)}(ii).name,fileFieldSep);
                try
                level1Files{processChan(chan_num)}(ii).utc = matlab2unixtime(datenum([fileNameFields{dateField},fileNameFields{timeField}],[outFileDateFormat,outFileTimeFormat]));
                catch
                    level1Files{processChan(chan_num)}(ii).utc = matlab2unixtime(datenum([fileNameFields{dateField+1},fileNameFields{timeField+1}],[outFileDateFormat,outFileTimeFormat]));
                    warning('diceSingleSurf:oldFileFormat','Files are in old file format. Run dicePulseCompress to rename the files.')
                    warning('off','diceSingleSurf:oldFileFormat');
                end
            end
            [~,sx] = sort([level1Files{processChan(chan_num)}.utc]);
            level1Files{processChan(chan_num)} = level1Files{processChan(chan_num)}(sx);
        end
    end
    
    numfiles = numel(level1Files{processChan(surf_chan_num)});
    nonSurfChans = processChan(processChan~=surfaceChan);
    
    for ii = 1: numfiles
        fprintf('diceSingleSurf: Processing File %d of %d\n',ii,numfiles);
        % load the surface and plane height variable form surface Ch
        load([surfPickedDataPath,level1Files{processChan(surf_chan_num)}(ii).name],'surfIndRad','planeHeight','prodLevel');        
                
        if ismember('singleSurf',prodLevel.desc)
            warning('diceSingleSurf:notOriginalSurf','Channel %s does not have the original surface.',processChanAttbr{surf_chan_num})
            warning('off','diceSingleSurf:notOriginalSurf')
            fprintf('diceSingleSurf: %d Skipped!\n',ii)
            continue;
        end
        
        for jj = 1:numel(nonSurfChans)
            load([surfPickedDataPath,level1Files{nonSurfChans(jj)}(ii).name],'prodLevel');
            
            % backward compatibility
            if ~iscell(prodLevel.desc)
                oldProdLevel = prodLevel;
                clear prodLevel;
                prodLevel.name = oldProLevel.name;
                prodLevel.desc = {oldProdLevel.desc};
                prodLevel.time = {'?'};
            end
                        
            if ismember('singleSurf',prodLevel.desc)
                warning('diceSingleSurf:surfAlreadyReplaced','Channel %s surface has already been replaced - Re-run dicePickSurf to restore original surfaces first.',processChanAttbr{surf_chan_num})
                warning('off','diceSingleSurf:surfAlreadyReplaced')
                fprintf('diceSingleSurf: %d %s Skipped!\n',jj, processChanAttbr{nonSurfChans(jj)})
                continue;
            end
            prodLevel.name = 'level1';
            prodLevel.desc{numel(prodLevel.desc)+1} = 'singleSurf';
            
            if save_append_backup
                copyfile([surfPickedDataPath,level1Files{nonSurfChans(jj)}(ii).name],[surfPickedDataPath,level1Files{nonSurfChans(jj)}(ii).name,tmpExt]);
            end
            save([surfPickedDataPath,level1Files{nonSurfChans(jj)}(ii).name],'surfIndRad','planeHeight','prodLevel','singleSurfReplacedTime','-append');
            if save_append_backup
                delete([surfPickedDataPath,level1Files{nonSurfChans(jj)}(ii).name,tmpExt]);
            end
            if ~configFileCopied
                [~,configFilename,~] = fileparts(configScript);
                copyfile([configFilename,'.mat'],[surfPickedDataPath,configFilename,'.mat'])
                configFileCopied = true;
            end
            
        end
    end
    warning('on','all')
        diary off;        
        if(configFileCopied)    % if true, then the flight was most likely processed
            [~,logFileNameOnly,logFileExt] = fileparts(logFile);
            if ~exist(processLogPath,'dir')
                mkdir(processLogPath);
            end
            movefile(logFile,[processLogPath,logFileNameOnly,fileFieldSep,'singlesurf',datestr(singleSurfReplacedTime,'_yyyymmdd_hhMMss'),logFileExt]);
            if runCount == 1
                [~,configFileNameOnly,~] = fileparts(configScript);
                copyfile(configScript,[processLogPath,configFileNameOnly,fileFieldSep,'singlesurf',datestr(singleSurfReplacedTime,'_yyyymmdd_hhMMss'),'.m.log']);
            end
        else % delete existing diary and start a fresh one
            [~,~,logext] = fileparts(logFile);
            if strcmpi(logext,'.log') %% just making sure it's a .log file before deletion
                delete(logFile);
            end
        end
    diary(logFile);
end
    if localDiary % if diary was locally started - delete it
        if strcmpi(get(0,'Diary'),'on')
            diary off;
            [~,~,logext] = fileparts(logFile);
            if strcmpi(logext,'.log') %% just making sure it's a .log file before deletion
                delete(logFile);
            end
        end
    end
fprintf('diceSingleSurf: Done!\n')
toc
catch errMsg
    fprintf('diceSingleSurf: Encountered an error\n');
    fprintf('%s\n',errMsg.message);
    fprintf('%s\n',errMsg.identifier);
    for jj = 1:numel(errMsg.stack)
        fprintf('In Function %s, Line %d\n',errMsg.stack(jj).name,errMsg.stack(jj).line)
    end
    warning('on','all');
    % move the log file
    diary off;
    if exist(logFile,'file') && exist('processLogPath','var')
        [~,logFileNameOnly,logFileExt] = fileparts(logFile);
        if ~exist(processLogPath,'dir')
            mkdir(processLogPath);
        end
        movefile(logFile,[processLogPath,logFileNameOnly,'_crash',datestr(now,'_yyyymmdd_hhMMss'),logFileExt]);
    end
    if debugEnable
        fprintf('diceSingleSurf: Halting for debugging\n')
        keyboard
    end
end
end
